/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef TATEGAKI_ENCODING_H
#define TATEGAKI_ENCODING_H

#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_TRUETYPE_IDS_H

#include <glib.h>

#define utf8_to_utf16le(in,size,out)  encode((in),(size),(out),"UTF-8","UTF-16LE")
#define utf16le_to_utf8(in,size,out)  encode((in),(size),(out),"UTF-16LE","UTF-8")
#define utf16be_to_cp932(in,size,out) encode((in),(size),(out),"UTF-16BE","CP932")
#define utf16be_to_utf8(in,size,out)  encode((in),(size),(out),"UTF-16BE","UTF-8")
#define symbol_to_utf8(in,size,out)   encode((in),(size),(out),"CP1252","UTF-8")
#define cp932_to_utf16le(in,size,out) encode((in),(size),(out),"CP932","UTF-16LE")
#define cp932_to_utf8(in,size,out)    encode((in),(size),(out),"CP932","UTF-8")
#define gb2312_to_utf8(in,size,out)   encode((in),(size),(out),"EUC-CN","UTF-8")
#define big5_to_utf8(in,size,out)     encode((in),(size),(out),"BIG5","UTF-8")
#define wansung_to_utf8(in,size,out)  encode((in),(size),(out),"EUC-KR","UTF-8")
#define johab_to_utf8(in,size,out)    encode((in),(size),(out),"JOHAB","UTF-8")
#define ucs4_to_utf8(in,size,out)     encode((in),(size),(out),"UCS-4","UTF-8")

#define ucs4_to_gb2312(in,size,out)     encode((in),(size),(out),"UCS-4","EUC-CN")
#define ucs4_to_big5(in,size,out)       encode((in),(size),(out),"UCS-4","BIG5")
#define ucs4_to_wansung(in,size,out)    encode((in),(size),(out),"UCS-4","EUC-KR")
#define ucs4_to_johab(in,size,out)      encode((in),(size),(out),"UCS-4","JOHAB")

//FIXME: make convert tables, g_convert may not have these encoding tables.
#define ucs4_to_mssymbol(in,size,out)      encode((in),(size),(out),"UCS-4","ASCII")
#define ucs4_to_adobestandard(in,size,out) encode((in),(size),(out),"UCS-4","Adobe-Standard-Encoding")
#define ucs4_to_adobeexpert(in,size,out)   encode((in),(size),(out),"UCS-4","ASCII")
#define ucs4_to_adobecustom(in,size,out)   encode((in),(size),(out),"UCS-4","ASCII")
#define ucs4_to_adobelatin1(in,size,out)   encode((in),(size),(out),"UCS-4","iso-8859-1")
#define ucs4_to_adobelatin2(in,size,out)   encode((in),(size),(out),"UCS-4","iso-8859-2")
#define ucs4_to_appleroman(in,size,out)    encode((in),(size),(out),"UCS-4","macintosh")

extern gsize encode (gpointer in, gsize size, gchar **out, const gchar *from, const gchar *to);
extern int check_face_charmap(FT_Face face);
extern FT_UInt unicode_to_charmap_code (FT_Face face, guint32 in);
extern gchar* utf8_nfc_except_cjk_kanji (gchar* src);
#endif
