/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#include <glib.h>
#include "tategaki.h"
#include "font_glyph_cache_table.h"



static gboolean destroy_glyph_in_cache (gpointer key, GlyphCache *gcache, gpointer dummy)
{
	if (gcache) {
		FT_Done_Glyph ( (FT_Glyph)gcache->glyph );
		g_free(gcache);
	}
	return TRUE;
}


void glyph_cache_table_clear (GlyphCacheTable *cache_table)
{
	if (cache_table)
		g_hash_table_foreach_remove (cache_table, (GHRFunc)destroy_glyph_in_cache, NULL);
}


void glyph_cache_table_destroy (GlyphCacheTable *cache_table)
{
	if (cache_table) {
		glyph_cache_table_clear(cache_table);
		g_hash_table_destroy(cache_table);
	}
}


GlyphCacheTable* glyph_cache_table_create ()
{
	return g_hash_table_new (g_direct_hash, g_direct_equal);
}


int glyph_cache_table_set (GlyphCacheTable *cache_table, 
		FT_UInt glyph_id, FT_GlyphSlot slot, gboolean is_vert)
{
	FT_Glyph new_glyph;
	GlyphCache *gcache;
	if (cache_table == NULL) return 1;

	FT_Get_Glyph(slot, &new_glyph);
	
	gcache = g_hash_table_lookup(cache_table, GUINT_TO_POINTER(glyph_id));
	if (gcache) {
		FT_Done_Glyph((FT_Glyph)gcache->glyph);
	} else {
		gcache = g_malloc(sizeof(GlyphCache));
		g_hash_table_insert(cache_table, GUINT_TO_POINTER(glyph_id), gcache);
	}
	gcache->glyph    = new_glyph;
	gcache->metrics  = slot->metrics;
	gcache->glyph_id = glyph_id;
	gcache->is_vert  = is_vert;
	return 0;
}


GlyphCache* glyph_cache_table_get (GlyphCacheTable *cache_table, FT_UInt glyph_id)
{
	return g_hash_table_lookup (cache_table, GUINT_TO_POINTER(glyph_id));
}

/* vim: set fdm=marker ts=4 sw=4:*/
