/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef TATEGAKI_FONT_GSUB_TABLE_H
#define TATEGAKI_FONT_GSUB_TABLE_H

#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_TRUETYPE_TABLES_H
#include FT_TRUETYPE_TAGS_H
#include <glib.h>

typedef struct _GlyphIdList {
	int size;
	guint16 *gid;
} GlyphIdList;

typedef struct _FeatureRecord {
	guint32 tag;
	FT_Byte* feature;
} FeatureRecord;

typedef struct _FeatureTable {
	guint32 tag;
	guint32 lookup_index_size;
	guint32 *lookup_index;
} FeatureTable;

typedef struct _RangeRecord {
	guint16 gid_st;
	guint16 gid_ed;
	guint16 idx_st;
} RangeRecord;

typedef struct _CoverageTable {
	int format;
	int size;
	guint16 *gid_list;//format=1ł̂ݎgp
	RangeRecord *range_record;//format=2ł̂ݎgp
} CoverageTable;

typedef struct _SingleSubst {
	int format;
	int delta;//format1
	int size; //format2
	guint16 *gid_list;//format2
	CoverageTable *coverage_table;
} SingleSubst;

typedef struct _GSUB_Table {
	int table_length;
	FT_Byte *table;

	FT_Byte *script_list_ptr;
	FT_Byte *feature_list_ptr;
	FT_Byte *lookup_list_ptr;
	
	int lookup_size;
	char** lookup;

	gsize feature_table_size;
	FeatureTable *feature_table;

} GSUB_Table;

extern GSUB_Table* gsub_table_create (FT_Face face);
extern void        gsub_table_destroy (GSUB_Table *table);
extern void        gsub_table_get_gid(GSUB_Table *table, guint32 tag, GlyphIdList *in, GlyphIdList *out);
#endif
