/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef TATEGAKI_FONT_MORT_TABLE_H
#define TATEGAKI_FONT_MORT_TABLE_H

#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_TRUETYPE_TABLES_H
#include FT_TRUETYPE_TAGS_H
#include <glib.h>

//#define FEATURE_TYPE_LIGATURES                 1
//#define FEATURE_TYPE_CURSIVE_CONNECTION        2
//#define FEATURE_TYPE_LETTER_CASE               3
//#define FEATURE_TYPE_VERTICAL_SUBSTITUTION     4
//#define FEATURE_TYPE_LINGUISTIC_REARRANGEMENT  5
//#define FEATURE_TYPE_NUMBER_SPACING            6
//#define FEATURE_TYPE_SMART_SWASH               8
//#define FEATURE_TYPE_DIACRITICS                9
//#define FEATURE_TYPE_VERTICAL_POSITION         10
//#define FEATURE_TYPE_FRACTION                  11
//#define FEATURE_TYPE_OVERLAPPING_CHARACERS     13
//#define FEATURE_TYPE_TYPOGRAPHIC_EXTRAS        14
//#define FEATURE_TYPE_MATHMATICAL_EXTRAS        15
//#define FEATURE_TYPE_ORNAMENT_SETS             16
//#define FEATURE_TYPE_CHARACTER_ALTERNATIVES    17
//#define FEATURE_TYPE_DESIGN_COMPLEXITY         18
//#define FEATURE_TYPE_STYLE_OPTIONS             19
//#define FEATURE_TYPE_CHARACTER_SHAPE           20
//#define FEATURE_TYPE_NUMBER_CASE               21
//#define FEATURE_TYPE_TEXT_SPACING              22
//#define FEATURE_TYPE_TRANSLITERATION           23
//#define FEATURE_TYPE_ANNOTATION                24
//#define FEATURE_TYPE_KANA_SPACING              25
//#define FEATURE_TYPE_IDEOGRAPHIC_SPACING       26
//#define FEATURE_TYPE_CJK_ROMAN_SPACING         103

#define MORT_SUBTABLE_TYPE_INDICSTYLE           0
#define MORT_SUBTABLE_TYPE_CONTEXTUAL           1
#define MORT_SUBTABLE_TYPE_LIGATURE             2
//NOTE:3 is reserved
#define MORT_SUBTABLE_TYPE_NONCONTEXTUAL        4
#define MORT_SUBTABLE_TYPE_CONTEXTUAL_INSERTION 5

#define MORT_SUBTABLE_IS_VERTICAL(subtable)       ((subtable)->coverage & 0x8000)
#define MORT_SUBTABLE_IS_DESCENDING(subtable)     ((subtable)->coverage & 0x4000)
#define MORT_SUBTABLE_IS_IGNOREVERTICAL(subtable) ((subtable)->coverage & 0x2000)
#define MORT_SUBTABLE_GET_TYPE(subtable)          ((subtable)->coverage & 0x0007)

typedef struct _mort_Subtable {
	FT_Byte *addr;/*dont free*/
	guint16 coverage;
	guint16 length;
	guint32 sub_feature_flags;
	FT_Byte *data;
	guint32 data_length;
} mort_Subtable;

typedef struct _mort_Feature {
	guint16 type;
	guint16 setting;
	guint32 flags;
} mort_Feature;

typedef struct _mort_Chain {
	FT_Byte *addr;/*dont free*/
	guint32 flags;
	guint32 length;
	guint16 feature_num;
	guint16 subtable_num;
	mort_Feature  *features;
	mort_Subtable *subtables;
} mort_Chain;

typedef struct _mort_Table {
	int         data_length;
	FT_Byte    *data;
	gint32      chain_num;
	mort_Chain *chains;
} mort_Table;

extern mort_Table* mort_table_create (FT_Face face);
extern void mort_table_destroy (mort_Table *table);
extern guint16 mort_table_find_vertical_glyph_id_from_lookup (FT_Byte *lookup_table, guint16 glyph_index);
#endif

