/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#include <stdlib.h>
#include <glib.h>
#include <libgimp/gimp.h>

// Only 2.8 API
typedef	gboolean (*GimpImageInsertLayer)      (gint32, gint32, gint32, gint);
// Deprecated API at 2.8
typedef gboolean (*GimpImageAddLayer)         (gint32, gint32, gint);
typedef gint     (*GimpImageGetLayerPosition) (gint32, gint32);

static void get_version(int *maj, int *min, int *pat);

static void
get_version(int *maj, int *min, int *pat)
{
	static int major = 0;
	static int minor = 0;
	static int patch = 0;

	if (major == 0) {
		gchar *version = gimp_version ();
		gchar **tokens = NULL;
		tokens = g_strsplit (version, ".", 0);
		major = atoi (tokens[0]);
		minor = atoi (tokens[1]);
		patch = atoi (tokens[2]);
		if (tokens) g_strfreev (tokens);
	}

	*maj = major;
	*min = minor;
	*pat = patch;
}

gboolean
stub_func_gimp_image_insert_layer (gint32 image_ID,
								   gint32 layer_ID,
								   gint32 parent_ID,
								   gint   position)
{
	int maj, min, pat;
	get_version(&maj, &min, &pat);
	if (g_module_supported() == 0) {
		g_error ("g_module support is required!");
	}

	if (maj >= 2 && min >= 8) {
		gboolean ret;
		GimpImageInsertLayer func_insert_layer = NULL;
		GModule *module = g_module_open (NULL, G_MODULE_BIND_LAZY);

		if (module == NULL) {
			g_error ("module is not found!");
		}

		g_module_symbol (module, "gimp_image_insert_layer", (gpointer)&func_insert_layer);
		if (func_insert_layer == NULL) {
			g_error ("symbol gimp_image_insert_layer is not found!");
		}

		ret = func_insert_layer (image_ID, layer_ID, 0, position);
		g_module_close (module);
		return ret;
	}

	if (maj == 2 && min < 8) {
		gboolean ret;
		gint32 active_layer;
		GimpImageAddLayer func_add_layer = NULL;
		GimpImageGetLayerPosition func_get_layer_position = NULL;
		GModule *module = g_module_open (NULL, G_MODULE_BIND_LAZY);

		if (module == NULL) {
			g_error ("module is not found!");
		}

		g_module_symbol (module, "gimp_image_add_layer", (gpointer)&func_add_layer);
		if (func_add_layer == NULL) {
			g_error ("symbol gimp_image_add_layer is not found!");
		}

		g_module_symbol (module, "gimp_image_get_layer_position", (gpointer)&func_get_layer_position);
		if (func_get_layer_position == NULL) {
			g_error ("symbol gimp_image_get_layer_position is not found!");
		}

		if (position < 0) {
			active_layer = gimp_image_get_active_layer(image_ID);
			position = func_get_layer_position(image_ID, active_layer);
		}

		ret = func_add_layer (image_ID, layer_ID, position);
		g_module_close (module);
		return ret;
	}

	return FALSE;
}

