/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef TATEGAKI_TYPESET_OBJ_H
#define TATEGAKI_TYPESET_OBJ_H
#include <math.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_OUTLINE_H
#include FT_BBOX_H
#include FT_BITMAP_H
#include <glib.h>
#include "typesetter.h"
#include "typeset_obj_dot.h"

typedef enum _TypesetObjType {
	TYPESET_OBJ_TYPE_LAYOUT,//CAEg
	TYPESET_OBJ_TYPE_LINE,  //s
	TYPESET_OBJ_TYPE_GLYPH, //
	TYPESET_OBJ_TYPE_RUBY,  //r
	TYPESET_OBJ_TYPE_YOKO,  //c
	TYPESET_OBJ_TYPE_DOT    //_
} TypesetObjType;

typedef enum {
	CHAR_CLASS_ALPHABET = 0,
	CHAR_CLASS_NUMBER,      //ꕶ
	CHAR_CLASS_EXCLAM,      //Qꕶ
	CHAR_CLASS_YAKU_KUTEN,  //񕨁BǓ_
	CHAR_CLASS_YAKU_START,  //񕨁Bnߊʗ
	CHAR_CLASS_YAKU_END,    //񕨁BI芇ʗ
	CHAR_CLASS_YAKU_CENTER, //񕨁B_
	CHAR_CLASS_KANA,        //ȁB
	CHAR_CLASS_COMBINING,   //
	CHAR_CLASS_DEFAULT,     //̑

	CHAR_CLASS_NONE
} CharClass;


typedef void (*FuncDestroy)(gpointer);
typedef void (*FuncComputeGboxSize)(gpointer,gpointer,gpointer);//obj,prev,next
typedef void (*FuncResetStyle)(gpointer);
typedef void (*FuncSetStyle)(gpointer,int);
typedef void (*FuncSetDir)(gpointer,DirType);
typedef void (*FuncSetDot)(gpointer,int);
typedef void (*FuncSetBold)(gpointer,double,double);
typedef void (*FuncSetColor)(gpointer,Color);
typedef void (*FuncGetGlyphID)(gpointer);
typedef void (*FuncCombineGlyph)(gpointer);
typedef void (*FuncCacheGlyphImage)(gpointer);
typedef void (*FuncComputeSelfSize)(gpointer);
typedef void (*FuncGetSize)(gpointer,double*,double*,Rect*,Rect*);//obj,advance,offset,real_size,typo_size
typedef void (*FuncRender)(gpointer,BufferContainer*);

typedef struct _TypesetObj {
	struct _TypesetObj	*parent;
	TypesetObjType		type;
	Typesetter			*typesetter;
	DirType				dir;
	DotType				dot;
	int					style;    //font style id
	FontFlag			fontflag; //font rendering flag
	double				scale_x;
	double				scale_y;
	double				bold_x;
	double				bold_y;
	Color				color;
	FuncDestroy			func_destroy;
	FuncComputeGboxSize	func_compute_gbox_size;
	FuncResetStyle		func_reset_style;
	FuncSetStyle		func_set_style;
	FuncSetDir			func_set_dir;
	FuncSetDot			func_set_dot;
	FuncSetBold			func_set_bold;
	FuncSetColor		func_set_color;
	FuncGetGlyphID		func_get_glyph_id;
	FuncCombineGlyph	func_combine_glyph;
	FuncCacheGlyphImage	func_cache_glyph_image;
	FuncGetSize			func_get_size;
	FuncComputeSelfSize	func_compute_self_size;
	FuncRender			func_render;
} TypesetObj;

extern void swap_double (double *a, double *b);
extern void get_yaku_tsume_jisx4051 (CharClass prev, CharClass now, CharClass next, double* offset, double* advance);
extern CharClass get_char_class (gunichar n);
extern GList* combine_glyph (GList *first);
extern void get_typo_metrics (FT_Face face, double *ascender, double *descender, double *linegap);
extern void get_scale (TypesetObj *obj, double *x, double *y);

extern void copy_glyph_image (Color color, gboolean rotate, gboolean antialias, guint8 *Dst, int dx, int dy, int dw, int dh, int planes, guint8 *Src, int sw, int sh, int sp);

extern void obj_init(TypesetObj         *obj,
					 TypesetObj         *parent,
					 TypesetObjType      type,
					 Typesetter         *typesetter,
					 DirType             dir,
					 FuncDestroy         func_destroy,
					 FuncResetStyle      func_reset_style,
					 FuncSetStyle        func_set_style,
					 FuncSetDir          func_set_dir,
					 FuncSetDot          func_set_dot,
					 FuncSetBold         func_set_bold,
					 FuncSetColor        func_set_color,
					 FuncGetGlyphID      func_get_glyph_id,
					 FuncCombineGlyph    func_combine_glyph,
					 FuncCacheGlyphImage func_cache_glyph_image,
					 FuncComputeGboxSize func_compute_gbox_size,
					 FuncComputeSelfSize func_compute_self_size,
					 FuncGetSize         func_get_size,
					 FuncRender          func_render);
#define OBJ_INIT(OBJ,\
				 PARENT,\
				 TYPE,\
				 TYPESETTER,\
				 DIR,\
				 DESTROY,\
				 RESETSTYLE,\
				 SETSTYLE,\
				 SETDIR,\
				 SETDOT,\
				 SETBOLD,\
				 SETCOLOR,\
				 GETGLYPHID,\
				 COMBINEGLYPH,\
				 CACHEGLYPHIMAGE,\
				 COMPUTEGBOXSIZE,\
				 COMPUTESELFSIZE,\
				 GETSIZE,RENDER) \
	obj_init((TypesetObj*)(OBJ), \
			 (PARENT),\
			 (TYPE),\
			 (TYPESETTER),\
			 (DIR),\
			 (FuncDestroy)(DESTROY),\
			 (FuncResetStyle)(RESETSTYLE),\
			 (FuncSetStyle)(SETSTYLE),\
			 (FuncSetDir)(SETDIR),\
			 (FuncSetDot)(SETDOT),\
			 (FuncSetBold)(SETBOLD),\
			 (FuncSetColor)(SETCOLOR),\
			 (FuncGetGlyphID)(GETGLYPHID),\
			 (FuncCombineGlyph)(COMBINEGLYPH),\
			 (FuncCacheGlyphImage)(CACHEGLYPHIMAGE),\
			 (FuncComputeGboxSize)(COMPUTEGBOXSIZE),\
			 (FuncComputeSelfSize)(COMPUTESELFSIZE),\
			 (FuncGetSize)(GETSIZE),\
			 (FuncRender)(RENDER));

#endif
