/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef TATEGAKI_TYPESET_OBJ_LINE_H
#define TATEGAKI_TYPESET_OBJ_LINE_H
#include "typesetter.h"
#include "typeset_obj.h"
#include "typeset_obj_glyphbox.h"

typedef struct _Line {
	struct _TypesetObj	obj;
	GList		*elements;
	double		line_width;
	double		ruby_width;
	int			vert_elem_num;
	double		char_pitch;//Ԋu
	double		typo_height;
	double		offset;
} Line;


extern Line* line_create(Typesetter *typ, TypesetObj *parent, GList *elements, int dir);
extern void line_destroy(Line *line);
extern void line_reset_style(Line *line);
extern void line_set_style(Line *line, int style);
extern void line_set_dir(Line *line, DirType dir);
extern void line_set_dot(Line *line, DotType dot);
extern void line_set_bold(Line *line, double bold_x, double bold_y);
extern void line_set_color(Line *line, Color color);
extern void line_get_glyph_id (Line *line);
extern void line_combine_glyph (Line *line);
extern void line_cache_glyph_image (Line *line);
extern void line_compute_gbox_size (Line *line, TypesetObj *prev, TypesetObj *next);
extern void line_compute_self_size (Line *line);
extern void line_get_size (Line *line, double *advance, double *offset, Rect *real_size, Rect *typo_size);
extern void line_render (Line *line, BufferContainer *buf);

extern void line_update_height_and_offset (Line *line, double max_typo_height);
extern void line_debug_print (Line *line, gpointer data);
#endif