/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#include "typeset_obj_yoko.h"
#include "typeset_obj_glyphbox.h"
#include "typeset_parser.h"

static void yoko_get_size_ltr (Yoko *yoko, double *advance, double *offset, Rect *real_size, Rect *typo_size);

Yoko* yoko_create(Typesetter *typ, TypesetObj *parent, GList *elements, double trans, AlignType align)
{/*{{{*/
	GList *p;
	Yoko *yoko = g_slice_alloc (sizeof(Yoko));
	OBJ_INIT(yoko,
			 parent,
			 TYPESET_OBJ_TYPE_YOKO,
			 typ,
			 DIR_TYPE_TTB,
			 yoko_destroy,
			 yoko_reset_style,
			 yoko_set_style,
			 yoko_set_dir,
			 yoko_set_dot,
			 yoko_set_bold,
			 yoko_set_color,
			 yoko_get_glyph_id,
			 yoko_combine_glyph,
			 yoko_cache_glyph_image,
			 yoko_compute_gbox_size,
			 yoko_compute_self_size,
			 yoko_get_size,
			 yoko_render);
	yoko->elements = elements;
	if (align == BOTH) {
		yoko->scale = 0;
		yoko->align = CENTER;
	} else {
		yoko->scale = trans;
		yoko->align = align;
	}
	
	p = g_list_first(yoko->elements);
	while (p) {
		((TypesetObj*)p->data)->parent = (TypesetObj*)yoko;
		p = g_list_next(p);
	}

	yoko->obj.func_set_dir(yoko, yoko->obj.dir);
	return yoko;
}/*}}}*/

void yoko_destroy(Yoko *yoko)
{/*{{{*/
	GList *p;

	p = g_list_first(yoko->elements);
	while (p) {
		((TypesetObj*)(p->data))->func_destroy(p->data);
		p = g_list_next(p);
	}
	g_list_free(p);

	g_slice_free1(sizeof(Yoko), yoko);
}/*}}}*/

void yoko_reset_style(Yoko *yoko)
{/*{{{*/
	GList *p = yoko->elements;
	while (p) {
		((TypesetObj*)p->data)->func_reset_style(p->data);
		p = g_list_next(p);
	}
}/*}}}*/

void yoko_set_style(Yoko *yoko, int style)
{/*{{{*/
	GList *p = yoko->elements;
	while (p) {
		((TypesetObj*)p->data)->func_set_style(p->data, style);
		p = g_list_next(p);
	}
	yoko->obj.style = style;
}/*}}}*/

void yoko_set_dir(Yoko *yoko, DirType dir)
{/*{{{*/
	GList *p = yoko->elements;
	while (p) {
		//yoko is always LTR!
		((TypesetObj*)p->data)->func_set_dir(p->data, DIR_TYPE_LTR);
		p = g_list_next(p);
	}
	yoko->obj.dir = dir;
}/*}}}*/

void yoko_set_dot(Yoko *yoko, DotType dot)
{/*{{{*/
	GList *p = yoko->elements;
	while (p) {
		((TypesetObj*)p->data)->func_set_dot(p->data, DOT_TYPE_NONE);
		p = g_list_next(p);
	}
	yoko->obj.dot = dot;
}/*}}}*/

void yoko_set_bold(Yoko *yoko, double bold_x, double bold_y)
{/*{{{*/
	GList *p = yoko->elements;
	while (p) {
		((TypesetObj*)p->data)->func_set_bold(p->data, bold_x, bold_y);
		p = g_list_next(p);
	}
	yoko->obj.bold_x += bold_x;
	yoko->obj.bold_y += bold_y;
}/*}}}*/

void yoko_set_color(Yoko *yoko, Color color)
{/*{{{*/
	GList *p = yoko->elements;
	while (p) {
		((TypesetObj*)p->data)->func_set_color(p->data, color);
		p = g_list_next(p);
	}
	yoko->obj.color = color;
}/*}}}*/

void yoko_get_glyph_id (Yoko *yoko)
{/*{{{*/
	GList *p = yoko->elements;
	while (p) {
		((TypesetObj*)p->data)->func_get_glyph_id(p->data);
		p = g_list_next(p);
	}
}/*}}}*/

void yoko_combine_glyph (Yoko *yoko)
{/*{{{*/
	GList *p;
	yoko->elements = combine_glyph(yoko->elements);
	p = g_list_first (yoko->elements);
	while (p) {
		((TypesetObj*)p->data)->func_combine_glyph(p->data);
		p = g_list_next(p);
	}
}/*}}}*/

void yoko_cache_glyph_image (Yoko *yoko)
{/*{{{*/
	GList *p = yoko->elements;
	while (p) {
		((TypesetObj*)(p->data))->func_cache_glyph_image(p->data);
		p = g_list_next(p);
	}
}/*}}}*/

void yoko_compute_gbox_size (Yoko *yoko, TypesetObj *prev, TypesetObj *next)
{/*{{{*/
	GList *p = yoko->elements;
	
	while (p) {
		((TypesetObj*)p->data)->func_compute_gbox_size(p->data,
													   p->prev?p->prev->data:NULL,
													   p->next?p->next->data:NULL);
		p = g_list_next(p);
	}
}/*}}}*/

void yoko_compute_self_size (Yoko *yoko)
{/*{{{*/
	GList *p;
	double ad, of;
	Rect r_size, t_size;
	double tmp_scale = yoko->scale;
	
	yoko->scale = 1;
	p = yoko->elements;
	while (p) {
		((TypesetObj*)p->data)->func_compute_self_size(p->data);
		p = g_list_next(p);
	}
	yoko->scale = tmp_scale;

	if (yoko->scale <= 0) {
		FT_Face face = yoko->obj.typesetter->face_table[yoko->obj.style].face;
		double xppem = face->size->metrics.x_ppem;
		double width, scale_x, scale_y;

		yoko->scale = 1;
		yoko_get_size(yoko, &ad, &of, &r_size, &t_size);
		get_scale((TypesetObj*)yoko, &scale_x, &scale_y);
		width = (t_size.right - t_size.left);
		xppem = scale_x * xppem;

		if (width == 0) { width = xppem; }
		if (width > xppem) {
			yoko->scale = xppem / width;
		} else {
			yoko->scale = 1;
		}
	}
}/*}}}*/

static void yoko_get_size_ltr (Yoko *yoko, double *advance, double *offset, Rect *real_size, Rect *typo_size)
{/*{{{*/
	GList *p;
	double ad, of;
	Rect r_size, t_size;
	Rect zero_size = {0, 0, 0, 0};

	*offset = 0;
	*advance = 0;
	*real_size = zero_size;
	*typo_size = zero_size;

	p = g_list_first (yoko->elements);
	while (p) {
		TypesetObj* obj = p->data;
		obj->func_get_size(p->data, &ad, &of, &r_size, &t_size);

		g_assert(obj->dir == DIR_TYPE_LTR);
		real_size->top    = MIN(real_size->top,    r_size.top);
		real_size->bottom = MAX(real_size->bottom, r_size.bottom);
		real_size->left   = MIN(real_size->left,   *advance + r_size.left);
		real_size->right  = MAX(real_size->right,  *advance + r_size.right);
		typo_size->top    = MIN(typo_size->top,    t_size.top);
		typo_size->bottom = MAX(typo_size->bottom, t_size.bottom);
		typo_size->left   = MIN(typo_size->left,   *advance + t_size.left);
		typo_size->right  = MAX(typo_size->right,  *advance + t_size.right);

		p = g_list_next(p);

		// Hook for Combining Char
		while (p) {
			TypesetObj *obj = p->data;
			GlyphBox *gbox = p->data;
			double cm_ad, cm_of;
			Rect cm_rsize, cm_tsize;
			if (obj->type != TYPESET_OBJ_TYPE_GLYPH) break;
			if (gbox->char_class != CHAR_CLASS_COMBINING) break;

			obj->func_get_size(p->data, &cm_ad, &cm_of, &cm_rsize, &cm_tsize);

			g_assert(obj->dir != DIR_TYPE_TTB);

			if (obj->dir == DIR_TYPE_LTR) {
				real_size->top    = MAX(real_size->top, cm_rsize.top);
				real_size->bottom = MIN(real_size->bottom, cm_rsize.bottom);
				real_size->left   = MIN(real_size->left, cm_rsize.left);
				real_size->right  = MAX(real_size->right, *advance + cm_rsize.right);
				//typo_size̕ύX͂ȂB
			}
			p = g_list_next(p);
		}

		*advance += ad + of;
	}
}/*}}}*/

void yoko_get_size (Yoko *yoko, double *advance, double *offset, Rect *real_size, Rect *typo_size)
{/*{{{*/
	double ad, of;
	Rect zero_size = {0, 0, 0, 0};
	Rect r_size, t_size;

	*offset = 0;
	*advance = 0;
	*real_size = zero_size;
	*typo_size = zero_size;

	//if (yoko->scale == 0) {
	//	yoko_compute_self_size(yoko);
	//}
	yoko_get_size_ltr(yoko, &ad, &of, &r_size, &t_size);

	//LTRWnTTBWnɕϊB
	if (yoko->obj.dir == DIR_TYPE_TTB) {
		FT_Face face = yoko->obj.typesetter->face_table[yoko->obj.style].face;
		int xppem = face->size->metrics.x_ppem;

		switch (yoko->align) {
			case TOP:
				real_size->top     = r_size.top - t_size.top;
				real_size->bottom  = r_size.bottom - t_size.top;
				real_size->left    = r_size.left  - xppem / 2.0;
				real_size->right   = r_size.right - xppem / 2.0;
				typo_size->top     = 0;
				typo_size->bottom  = t_size.bottom  - t_size.top;
				typo_size->left    = -xppem / 2.0;
				typo_size->right   = t_size.right - t_size.left - xppem / 2.0;
				*advance           = typo_size->bottom;
				break;
			case CENTER:
				real_size->top     = r_size.top - t_size.top;
				real_size->bottom  = r_size.bottom - t_size.top;
				real_size->left    = -(t_size.right - t_size.left) / 2.0 + r_size.left;
				real_size->right   = -(t_size.right - t_size.left) / 2.0 + r_size.right;
				typo_size->top     = 0;
				typo_size->bottom  = t_size.bottom  - t_size.top;
				typo_size->left    = -(t_size.right - t_size.left) / 2.0;
				typo_size->right   =  (t_size.right - t_size.left) / 2.0;
				*advance           = typo_size->bottom;
				break;
			case BOTTOM:
				real_size->top     = r_size.top - t_size.top;
				real_size->bottom  = r_size.bottom - t_size.top;
				real_size->left    = xppem / 2.0 + (r_size.left  - r_size.right);
				real_size->right   = xppem / 2.0 - (r_size.right - t_size.right);
				typo_size->top     = 0;
				typo_size->bottom  = t_size.bottom  - t_size.top;
				typo_size->left    = xppem / 2.0 - (t_size.right - t_size.left);
				typo_size->right   = xppem / 2.0;
				*advance           = typo_size->bottom;
				break;
			case BOTH:
				g_assert_not_reached();
		}
	}
	if (yoko->obj.dir == DIR_TYPE_LTR) {
		*real_size = r_size;
		*typo_size = t_size;
		*advance   = typo_size->right;
	}

}/*}}}*/

void yoko_render (Yoko *yoko, BufferContainer *buf)
{/*{{{*/
	double ad, of;
	Rect r_size, t_size;
	double orig_x, orig_y;
	GList *p;

	//save position
	orig_x = buf->x;
	orig_y = buf->y;

	if (yoko->obj.dir == DIR_TYPE_TTB) {
		FT_Face face = yoko->obj.typesetter->face_table[yoko->obj.style].face;
		int xppem = face->size->metrics.x_ppem;

		yoko_get_size_ltr(yoko, &ad, &of, &r_size, &t_size);
		switch (yoko->align) {
			case TOP:    buf->x -= xppem / 2.0; break;
			case CENTER: buf->x -= (t_size.right - t_size.left) / 2.0; break;
			case BOTTOM: buf->x += xppem / 2.0 - (t_size.right - t_size.left); break;
			case BOTH:   g_assert_not_reached();
		}

		buf->y -= t_size.top;
	}
	if (yoko->obj.dir == DIR_TYPE_LTR) {
		//LTR̂Ƃ͊_̕ϊ͕KvȂ
	}

	if (yoko->scale == 1.0) {
		p = g_list_first(yoko->elements);
		while (p) {
			TypesetObj *obj = p->data;
			GlyphBox *gbox = p->data;

			g_assert(obj->dir == DIR_TYPE_LTR);

			if (obj->type == TYPESET_OBJ_TYPE_GLYPH &&
				gbox->char_class == CHAR_CLASS_COMBINING) {
				p = g_list_next(p);
				continue;
			}

			((TypesetObj*)(p->data))->func_get_size(p->data, &ad, &of, &r_size, &t_size);
			// Move to offset
			buf->x += of;
			((TypesetObj*)(p->data))->func_render(p->data, buf);

			// Hook for Rendering Combining Char 
			p = glyphbox_render_combining (p, ad, buf);

			// Move to next position
			buf->x += ad;
			continue;
		}
	} else {
		p = g_list_first(yoko->elements);
		while (p) {
			double ad, of;
			Rect real_size, typo_size;
			TypesetObj *obj = p->data;

			obj->func_get_size(obj, &ad, &of, &real_size, &typo_size);
			// Move to offset
			switch(obj->dir) {
				case DIR_TYPE_TTB: buf->y += of; break;
				case DIR_TYPE_LTR: buf->x += of; break;
				case DIR_TYPE_RTL: buf->x -= of; break;
			}
			obj->func_render(obj, buf);

			// Hook for Rendering Combining Char 
			p = glyphbox_render_combining (p, ad, buf);

			// Move to next position
			buf->x += ad;
		}
	}
	//restore position
	buf->x = orig_x;
	buf->y = orig_y;
}/*}}}*/

void yoko_debug_print (Yoko *yoko, gpointer data)
{/*{{{*/
}/*}}}*/

/* vim: set fdm=marker ts=4 sw=4:*/
