/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef TATEGAKI_TYPESETTER_H
#define TATEGAKI_TYPESETTER_H
#include <ft2build.h>
#include FT_FREETYPE_H
#include <glib.h>
#include "font_gsub_table.h"
#include "font_vert_cache_table.h"
#include "font_glyph_cache_table.h"
#include "tategaki.h"
#include "color.h"


typedef enum _RenderMode {
	RENDER_MODE_GA,   // 2byte(Gray + Alpha)
	RENDER_MODE_RGBA  // 4byte(RGB + Alpha)
} RenderMode;

typedef enum _FontSizeUnit {
	PIXEL, POINT
} FontSizeUnit;

typedef enum _YakuTsumeType {
	BETA,     //lߖBSđSp̃x^B
	TSUME,    //SĂ̖񕨂͔pB
	JISX4051, //JISX4051B{͑Sp̃x^ŁA񕨂Aɋl߂B
} YakuTsumeType;

typedef enum _AlignType {
	TOP,		//㑵()
	BOTTOM,		//(E)
	CENTER,		//
	BOTH,		//[
} AlignType;

typedef enum {
	DIR_TYPE_TTB, //Top to Bottom
	DIR_TYPE_RTL, //Right to Left
	DIR_TYPE_LTR  //Left  to Right
} DirType;

typedef struct _Rect {
	double top;
	double bottom;
	double left;
	double right;
} Rect;

 //tHgX^C̍ő吔
#define FONT_STYLE_MAX		0x0FFF

//tHgX^C 0x0000-0x000F{ݒA0x0010-0x001FVXegpA0x0020-0x0FFF[U[gp
#define FONT_STYLE_BASE		0x0000 //{
#define FONT_STYLE_KANA		0x0001 //
#define FONT_STYLE_RUBY		0x0002 //r
#define FONT_STYLE_ASCII	0x0003 //
#define FONT_STYLE_COUNT	4
#define FONT_STYLE_RUBY2	0x0010 //rX^Cݒ肳ĂȂꍇ̃rX^CB{X^C̓񕪁B
#define FONT_STYLE_USER	    0x0020 //[U[X^CJnʒu


typedef struct _FontStyle {
	gchar			*file_path;
	gchar			*name;
	guint32			face_index;
	FT_F26Dot6		size_x;
	FT_F26Dot6		size_y;
	FontSizeUnit	unit;			//TCY̒PPIXEL/POINT
	guint32			dpi_x;			//𑜓xBunitPIXEL͎̎gpȂB
	guint32			dpi_y;			//𑜓xBunitPIXEL͎̎gpȂB
} FontStyle;

typedef struct _FontFlag {
	FT_Int32 load;
	FT_Int32 render;
} FontFlag;

//gŃIvV
typedef struct _TypesetterOption {
	RenderMode		render_mode;			//_O[h
	DirType			text_dir;				//cg=DIR_TYPE_RTL, g=DIR_TYPE_TTB
	guint32			gyou_okuri;				//sʁBPʂ%
	guint32			moji_okuri;				//ʁBPʂ%
	gboolean		kerning;				//J[jO
	AlignType		line_align;				//s
	YakuTsumeType	tsume_type;				//񕨂̋l߂̕@
	gboolean		antialias;				//A`GCAXON/OFF
	guint32			fuchidori;				//̉ʁBPʂ%B
	gboolean		auto_yoko;				//𗧂ĂB
	guint32			auto_yoko_digit;		//,Q̏čB
	//gboolean		ruby;					//r
	gboolean		ruby_trans;				//rό`ON/OFF
	double			ruby_max_trans_rate;	//rőό`
	gboolean		dot_skip_yaku;			//_񕨂ɕtȂ
	gboolean		dot_align;				//_𐮗񂳂irɌ_ꍇj
	Color			color;					//F
} TypesetterOption;

typedef struct _BufferContainer {
	RenderMode  render_mode;
	guint8     *data;
	int         width;
	int         height;
	int         planes;
	TypesetterOption *typesetter_option;
	/* ȉ͊eIuWFNgrenderɍXVĂ */
	double	x;//`̊_XiW,x[XCł͂Ȃj
	double	y;//`̊_YiW,x[XCł͂Ȃj
} BufferContainer;

typedef struct _FontFaceTable {
	FT_Face			face;
	VertCacheTable	*vcache_table;
	GlyphCacheTable	*gcache_table;
	GSUB_Table		*gsub_table;
	TT_OS2			*os2;
	double			base_line;
	FontStyle		*style;
} FontFaceTable;

typedef struct _Typesetter {
	TypesetterOption	option;
	FT_Library			library;
	GList				*font_list;
	FontFaceTable		*face_table;
} Typesetter;

extern const FT_Fixed FIXED_1;

#define DOUBLE_TO_F26D6(d)   ((FT_F26Dot6)((d) * 64.0))
#define DOUBLE_TO_FIXED(d)   ((FT_Fixed)((d) * 65536.0))
#define F26D6_TO_INT32(f)    ((int)((f) >> 6))
#define FIXED_TO_INT32(f)    ((int)((f) >> 16))
#define INT32_TO_F26D6(i)    (((int)(i)) << 6)
#define INT32_TO_FIXED(i)    (((int)(i)) << 16)
#define F16DOT16_TO_INT32(f) (FIXED_TO_INT32(f))
#define F26D6_TO_DOUBLE(f)   ((double)(f) / 64.0)
#define FIXED_TO_DOUBLE(f)   ((double)(f) / 65536.0)

extern Typesetter* typesetter_create();
extern int typesetter_reset(Typesetter *typesetter);
extern int typesetter_set_style (Typesetter *typesetter, guint32 style_type, FontStyle *style);
extern void typesetter_reset_style (Typesetter *typesetter);
extern int typesetter_get_empty_user_style_slot (Typesetter *typ);
extern int typesetter_get_pixel_size (Typesetter *typesetter, guint32 style_type);
extern int typesetter_render(Typesetter *typesetter, gchar *text, guint8 **buf, gint *width, gint *height);
extern int typesetter_destroy(Typesetter *typesetter);

extern double face_get_base_line (FT_Face face);
extern FT_Error face_set_size (FT_Face face, FontStyle *style);

FontStyle* typesetter_fontstyle_create(gchar *name, gchar *file_path, guint face_index,guint dpi_x, guint dpi_y, FT_F26Dot6 size_x, FT_F26Dot6 size_y, FontSizeUnit unit);
FontStyle* typesetter_fontstyle_copy(FontStyle *style);
void typesetter_fontstyle_destroy(FontStyle *style);
#endif
