#dump color name chart to stdout (utf-8)
#
#kill puts
rename puts tcl_puts
proc puts args {}
source [file join [file dir [file normalize [info script]]] dump_color_name.tcl]
rename puts ""
rename tcl_puts puts

proc rgb {code} {
    scan $code "#%2x%2x%2x" r g b
    set r [expr {$r / 255.0}]
    set g [expr {$g / 255.0}]
    set b [expr {$b / 255.0}]
    return [list $r $g $b]
}

proc rgb_to_hsl {code} {
    lassign [rgb $code] r g b
    set max [expr {max($r, $g, $b)}]
    set min [expr {min($r, $g, $b)}]
    set h 0
    set s 0
    set l [expr {($max + $min) / 2.0}]

    if {$max != $min} {
        set diff [expr {$max - $min}]
        if {$l > 0.5} {
            set s [expr {$diff / (2 - $max - $min)}]
        } else {
            set s [expr {$diff / ($max + $min)}]
        }
        if {$max == $r} {
             set h [expr {($g - $b) / $diff + ($g < $b ? 6 : 0)}]
        } elseif {$max == $g} {
             set h [expr {($b - $r) / $diff + 2}]
        } elseif {$max == $b} {
             set h [expr {($r - $g) / $diff + 4}]
        }
        set h [expr {$h / 6}]
    }
    return [list $h $s $l]
}

proc rgb_to_hsv {code} {
    lassign [rgb $code] r g b
    set max [expr {max($r, $g, $b)}]
    set min [expr {min($r, $g, $b)}]
    set h   [expr {$max - $min}]
    if {$h > 0.0} {
        if {$max == $r} {
            set h [expr {($g - $b) / $h}]
            if {$h < 0.0} {
                set h [expr {$h + 6.0}]
            }
        } elseif {$max == $g} {
            set h [expr {2.0 + ($b - $r) / $h}]
        } else {
            set h [expr {4.0 + ($r - $g) / $h}]
        }
    }
    set h [expr {$h / 6.0}]
    set s [expr {$max - $min}]
    if {$max != 0.0} {
        set s [expr {$s / $max}]
    }
    set v $max
    return [list $h $s $v]
}

set list_all [list]
lappend list_all {*}$list_css
lappend list_all {*}$list_jis_Z8102
lappend list_all {*}$list_ja_ext
set temp [dict create]
foreach {col name} $list_all {
    dict lappend temp $col $name
}
set list_all $temp


fconfigure stdout -encoding utf-8

puts "<html>
<head>
<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">
<title>Tategaki Color Chart</title>
<style type=\"text/css\">
    table {
        border-collapse: collapse;
    }
    th,td {
        border: 1px solid #aaa;
        padding: 0.2em;
    }
    th {
        background-color: #a2a2a2;
    }
    table td+td+td+td+td {
        text-align: right;
    }
</style>
<script>
var prev_col = -1;

function getValue(str, type) {
    if (type == \"number\") {
        return parseFloat(str);
    }
    return str;
}
function sort (col, type) {
    var rows_src = \[\];
    var rows_dst = \[\];

    var t = document.getElementById(\"t\");
    var tbody = t.firstElementChild;
    while (tbody.rows.length > 1) {
        var row = tbody.removeChild(tbody.rows\[1\]);
        rows_src.push(row);
    }

    if (col == prev_col) {
        for (var i = 0; i < rows_src.length; i++) {
            rows_dst.unshift(rows_src\[i\]);
        }
    } else {
        for (var i = 0; i < rows_src.length; i++) {
            var row = rows_src\[i\];
            var val_src = getValue(row.cells\[col\].textContent, type);
            var inserted = false;
            for (var j = 0; j < rows_dst.length; j++) {
                var val_dst = getValue(rows_dst\[j\].cells\[col\].textContent, type);
                if (val_src < val_dst) {
                    rows_dst.splice(j, 0, row);
                    inserted = true;
                    break;
                }
            }
            if (inserted == false) {
                rows_dst.push(row);
            }
        }
    }
    for (var i = 0; i < rows_dst.length; i++) {
        tbody.appendChild(rows_dst\[i\]);
    }
    prev_col = col;
}
</script>
</head>
<body>
<table id=\"t\">
<tr>
<th onclick=\"sort(0, 'number');\">No</th>
<th>color</th>
<th onclick=\"sort(2, 'string');\">#code</th>
<th onclick=\"sort(3, 'string');\">Name</th>
<th onclick=\"sort(4, 'number');\">H(HSV)</th>
<th onclick=\"sort(5, 'number');\">S(HSV)</th>
<th onclick=\"sort(6, 'number');\">V(HSV)</th>
<th onclick=\"sort(7, 'number');\">H(HSL)</th>
<th onclick=\"sort(8, 'number');\">S(HSL)</th>
<th onclick=\"sort(9, 'number');\">L(HSL)</th>
<th>Sample</th>
</tr>"
set no 1
foreach {color name} $list_all {
    lassign [rgb_to_hsv $color] hsv_h hsv_s hsv_v
    lassign [rgb_to_hsl $color] hsl_h hsl_s hsl_l
    puts "<tr>"
    puts "<td>$no</td>"
    puts "<td bgcolor='$color' width=50></td>"
    puts "<td>$color</td>"
    puts "<td>$name</td>"
    puts "<td>[format %.02f [expr $hsv_h * 360]]</td>"
    puts "<td>[format %.02f [expr $hsv_s * 100]]</td>"
    puts "<td>[format %.02f [expr $hsv_v * 100]]</td>"
    puts "<td>[format %.02f [expr $hsl_h * 360]]</td>"
    puts "<td>[format %.02f [expr $hsl_s * 100]]</td>"
    puts "<td>[format %.02f [expr $hsl_l * 100]]</td>"
    puts "<td><font color=$color>ABCDEFG</font></td>"
    puts "</tr>"
    incr no
}
puts "</table>"
puts "</body>" 
puts "</html>" 
