/*
 * GIMP Tategaki Plug-in -- This is a plug-in for The GIMP 2
 * Copyright (C) 2006-2017 OHTSUKA, Yoshio <ohtsuka.yoshio@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "color.h"
static GHashTable *color_table = NULL;

#define R_FROM_INT(V) ((V)>>24)
#define G_FROM_INT(V) (((V)>>16)&0xFF)
#define B_FROM_INT(V) (((V)>> 8)&0xFF)
#define A_FROM_INT(V) ((V)&0xFF)

#define ROUND(F) ((F)>0?floor((F)+0.5):ceil((F)-0.5))

static void init_color_table ()
{
	color_table = g_hash_table_new(g_str_hash, g_str_equal);

#define REGISTER_COLOR_NAME(COLOR_NAME,RGBA) \
	g_hash_table_insert(color_table, (COLOR_NAME), GINT_TO_POINTER((RGBA)))

// use /too/dump_color_name.tcl to dump a snipet.
{

// CSS3 Color Names
REGISTER_COLOR_NAME("ALICEBLUE", 0xF0F8FFFF);
REGISTER_COLOR_NAME("ANTIQUEWHITE", 0xFAEBD7FF);
REGISTER_COLOR_NAME("AQUA", 0x00FFFFFF);
REGISTER_COLOR_NAME("AQUAMARINE", 0x7FFFD4FF);
REGISTER_COLOR_NAME("BEIGE", 0xF5F5DCFF);
REGISTER_COLOR_NAME("BISQUE", 0xFFE4C4FF);
REGISTER_COLOR_NAME("BLACK", 0x000000FF);
REGISTER_COLOR_NAME("BLANCHEDALMOND", 0xFFEBCDFF);
REGISTER_COLOR_NAME("BLUE", 0x0000FFFF);
REGISTER_COLOR_NAME("BLUEVIOLET", 0x8A2BE2FF);
REGISTER_COLOR_NAME("BROWN", 0xA52A2AFF);
REGISTER_COLOR_NAME("BURLYWOOD", 0xDEB887FF);
REGISTER_COLOR_NAME("CADETBLUE", 0x5F9EA0FF);
REGISTER_COLOR_NAME("CHARTREUSE", 0x7FFF00FF);
REGISTER_COLOR_NAME("CHOCOLATE", 0xD2691EFF);
REGISTER_COLOR_NAME("CORAL", 0xFF7F50FF);
REGISTER_COLOR_NAME("CORNFLOWERBLUE", 0x6495EDFF);
REGISTER_COLOR_NAME("CORNSILK", 0xFFF8DCFF);
REGISTER_COLOR_NAME("CRIMSON", 0xDC143CFF);
REGISTER_COLOR_NAME("CYAN", 0x00FFFFFF);
REGISTER_COLOR_NAME("DARKBLUE", 0x00008BFF);
REGISTER_COLOR_NAME("DARKCYAN", 0x008B8BFF);
REGISTER_COLOR_NAME("DARKGOLDENROD", 0xB8860BFF);
REGISTER_COLOR_NAME("DARKGRAY", 0xA9A9A9FF);
REGISTER_COLOR_NAME("DARKGREEN", 0x006400FF);
REGISTER_COLOR_NAME("DARKGREY", 0xA9A9A9FF);
REGISTER_COLOR_NAME("DARKKHAKI", 0xBDB76BFF);
REGISTER_COLOR_NAME("DARKMAGENTA", 0x8B008BFF);
REGISTER_COLOR_NAME("DARKOLIVEGREEN", 0x556B2FFF);
REGISTER_COLOR_NAME("DARKORANGE", 0xFF8C00FF);
REGISTER_COLOR_NAME("DARKORCHID", 0x9932CCFF);
REGISTER_COLOR_NAME("DARKRED", 0x8B0000FF);
REGISTER_COLOR_NAME("DARKSALMON", 0xE9967AFF);
REGISTER_COLOR_NAME("DARKSEAGREEN", 0x8FBC8FFF);
REGISTER_COLOR_NAME("DARKSLATEBLUE", 0x483D8BFF);
REGISTER_COLOR_NAME("DARKSLATEGRAY", 0x2F4F4FFF);
REGISTER_COLOR_NAME("DARKSLATEGREY", 0x2F4F4FFF);
REGISTER_COLOR_NAME("DARKTURQUOISE", 0x00CED1FF);
REGISTER_COLOR_NAME("DARKVIOLET", 0x9400D3FF);
REGISTER_COLOR_NAME("DEEPPINK", 0xFF1493FF);
REGISTER_COLOR_NAME("DEEPSKYBLUE", 0x00BFFFFF);
REGISTER_COLOR_NAME("DIMGRAY", 0x696969FF);
REGISTER_COLOR_NAME("DIMGREY", 0x696969FF);
REGISTER_COLOR_NAME("DODGERBLUE", 0x1E90FFFF);
REGISTER_COLOR_NAME("FIREBRICK", 0xB22222FF);
REGISTER_COLOR_NAME("FLORALWHITE", 0xFFFAF0FF);
REGISTER_COLOR_NAME("FORESTGREEN", 0x228B22FF);
REGISTER_COLOR_NAME("FUCHSIA", 0xFF00FFFF);
REGISTER_COLOR_NAME("GAINSBORO", 0xDCDCDCFF);
REGISTER_COLOR_NAME("GHOSTWHITE", 0xF8F8FFFF);
REGISTER_COLOR_NAME("GOLD", 0xFFD700FF);
REGISTER_COLOR_NAME("GOLDENROD", 0xDAA520FF);
REGISTER_COLOR_NAME("GRAY", 0x808080FF);
REGISTER_COLOR_NAME("GREEN", 0x008000FF);
REGISTER_COLOR_NAME("GREENYELLOW", 0xADFF2FFF);
REGISTER_COLOR_NAME("GREY", 0x808080FF);
REGISTER_COLOR_NAME("HONEYDEW", 0xF0FFF0FF);
REGISTER_COLOR_NAME("HOTPINK", 0xFF69B4FF);
REGISTER_COLOR_NAME("INDIANRED", 0xCD5C5CFF);
REGISTER_COLOR_NAME("INDIGO", 0x4B0082FF);
REGISTER_COLOR_NAME("IVORY", 0xFFFFF0FF);
REGISTER_COLOR_NAME("KHAKI", 0xF0E68CFF);
REGISTER_COLOR_NAME("LAVENDER", 0xE6E6FAFF);
REGISTER_COLOR_NAME("LAVENDERBLUSH", 0xFFF0F5FF);
REGISTER_COLOR_NAME("LAWNGREEN", 0x7CFC00FF);
REGISTER_COLOR_NAME("LEMONCHIFFON", 0xFFFACDFF);
REGISTER_COLOR_NAME("LIGHTBLUE", 0xADD8E6FF);
REGISTER_COLOR_NAME("LIGHTCORAL", 0xF08080FF);
REGISTER_COLOR_NAME("LIGHTCYAN", 0xE0FFFFFF);
REGISTER_COLOR_NAME("LIGHTGOLDENRODYELLOW", 0xFAFAD2FF);
REGISTER_COLOR_NAME("LIGHTGRAY", 0xD3D3D3FF);
REGISTER_COLOR_NAME("LIGHTGREEN", 0x90EE90FF);
REGISTER_COLOR_NAME("LIGHTGREY", 0xD3D3D3FF);
REGISTER_COLOR_NAME("LIGHTPINK", 0xFFB6C1FF);
REGISTER_COLOR_NAME("LIGHTSALMON", 0xFFA07AFF);
REGISTER_COLOR_NAME("LIGHTSEAGREEN", 0x20B2AAFF);
REGISTER_COLOR_NAME("LIGHTSKYBLUE", 0x87CEFAFF);
REGISTER_COLOR_NAME("LIGHTSLATEGRAY", 0x778899FF);
REGISTER_COLOR_NAME("LIGHTSLATEGREY", 0x778899FF);
REGISTER_COLOR_NAME("LIGHTSTEELBLUE", 0xB0C4DEFF);
REGISTER_COLOR_NAME("LIGHTYELLOW", 0xFFFFE0FF);
REGISTER_COLOR_NAME("LIME", 0x00FF00FF);
REGISTER_COLOR_NAME("LIMEGREEN", 0x32CD32FF);
REGISTER_COLOR_NAME("LINEN", 0xFAF0E6FF);
REGISTER_COLOR_NAME("MAGENTA", 0xFF00FFFF);
REGISTER_COLOR_NAME("MAROON", 0x800000FF);
REGISTER_COLOR_NAME("MEDIUMAQUAMARINE", 0x66CDAAFF);
REGISTER_COLOR_NAME("MEDIUMBLUE", 0x0000CDFF);
REGISTER_COLOR_NAME("MEDIUMORCHID", 0xBA55D3FF);
REGISTER_COLOR_NAME("MEDIUMPURPLE", 0x9370DBFF);
REGISTER_COLOR_NAME("MEDIUMSEAGREEN", 0x3CB371FF);
REGISTER_COLOR_NAME("MEDIUMSLATEBLUE", 0x7B68EEFF);
REGISTER_COLOR_NAME("MEDIUMSPRINGGREEN", 0x00FA9AFF);
REGISTER_COLOR_NAME("MEDIUMTURQUOISE", 0x48D1CCFF);
REGISTER_COLOR_NAME("MEDIUMVIOLETRED", 0xC71585FF);
REGISTER_COLOR_NAME("MIDNIGHTBLUE", 0x191970FF);
REGISTER_COLOR_NAME("MINTCREAM", 0xF5FFFAFF);
REGISTER_COLOR_NAME("MISTYROSE", 0xFFE4E1FF);
REGISTER_COLOR_NAME("MOCCASIN", 0xFFE4B5FF);
REGISTER_COLOR_NAME("NAVAJOWHITE", 0xFFDEADFF);
REGISTER_COLOR_NAME("NAVY", 0x000080FF);
REGISTER_COLOR_NAME("OLDLACE", 0xFDF5E6FF);
REGISTER_COLOR_NAME("OLIVE", 0x808000FF);
REGISTER_COLOR_NAME("OLIVEDRAB", 0x6B8E23FF);
REGISTER_COLOR_NAME("ORANGE", 0xFFA500FF);
REGISTER_COLOR_NAME("ORANGERED", 0xFF4500FF);
REGISTER_COLOR_NAME("ORCHID", 0xDA70D6FF);
REGISTER_COLOR_NAME("PALEGOLDENROD", 0xEEE8AAFF);
REGISTER_COLOR_NAME("PALEGREEN", 0x98FB98FF);
REGISTER_COLOR_NAME("PALETURQUOISE", 0xAFEEEEFF);
REGISTER_COLOR_NAME("PALEVIOLETRED", 0xDB7093FF);
REGISTER_COLOR_NAME("PAPAYAWHIP", 0xFFEFD5FF);
REGISTER_COLOR_NAME("PEACHPUFF", 0xFFDAB9FF);
REGISTER_COLOR_NAME("PERU", 0xCD853FFF);
REGISTER_COLOR_NAME("PINK", 0xFFC0CBFF);
REGISTER_COLOR_NAME("PLUM", 0xDDA0DDFF);
REGISTER_COLOR_NAME("POWDERBLUE", 0xB0E0E6FF);
REGISTER_COLOR_NAME("PURPLE", 0x800080FF);
REGISTER_COLOR_NAME("RED", 0xFF0000FF);
REGISTER_COLOR_NAME("ROSYBROWN", 0xBC8F8FFF);
REGISTER_COLOR_NAME("ROYALBLUE", 0x4169E1FF);
REGISTER_COLOR_NAME("SADDLEBROWN", 0x8B4513FF);
REGISTER_COLOR_NAME("SALMON", 0xFA8072FF);
REGISTER_COLOR_NAME("SANDYBROWN", 0xF4A460FF);
REGISTER_COLOR_NAME("SEAGREEN", 0x2E8B57FF);
REGISTER_COLOR_NAME("SEASHELL", 0xFFF5EEFF);
REGISTER_COLOR_NAME("SIENNA", 0xA0522DFF);
REGISTER_COLOR_NAME("SILVER", 0xC0C0C0FF);
REGISTER_COLOR_NAME("SKYBLUE", 0x87CEEBFF);
REGISTER_COLOR_NAME("SLATEBLUE", 0x6A5ACDFF);
REGISTER_COLOR_NAME("SLATEGRAY", 0x708090FF);
REGISTER_COLOR_NAME("SLATEGREY", 0x708090FF);
REGISTER_COLOR_NAME("SNOW", 0xFFFAFAFF);
REGISTER_COLOR_NAME("SPRINGGREEN", 0x00FF7FFF);
REGISTER_COLOR_NAME("STEELBLUE", 0x4682B4FF);
REGISTER_COLOR_NAME("TAN", 0xD2B48CFF);
REGISTER_COLOR_NAME("TEAL", 0x008080FF);
REGISTER_COLOR_NAME("THISTLE", 0xD8BFD8FF);
REGISTER_COLOR_NAME("TOMATO", 0xFF6347FF);
REGISTER_COLOR_NAME("TURQUOISE", 0x40E0D0FF);
REGISTER_COLOR_NAME("VIOLET", 0xEE82EEFF);
REGISTER_COLOR_NAME("WHEAT", 0xF5DEB3FF);
REGISTER_COLOR_NAME("WHITE", 0xFFFFFFFF);
REGISTER_COLOR_NAME("WHITESMOKE", 0xF5F5F5FF);
REGISTER_COLOR_NAME("YELLOW", 0xFFFF00FF);
REGISTER_COLOR_NAME("YELLOWGREEN", 0x9ACD32FF);
// JIS Z 8102:2001 Color Name
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x8D\xE8\x89\xB2", 0xFCA3C2FF);//ƂF
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0xFCA3C2FF);//Ƃ
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x8D", 0xFCA3C2FF);//Ƃ
REGISTER_COLOR_NAME("\xE9\xB4\x87\xE8\x89\xB2", 0xFCA3C2FF);//F
REGISTER_COLOR_NAME("\xE3\x81\xA4\xE3\x81\xA4\xE3\x81\x98\xE3\x81\x84\xE3\x82\x8D", 0xD24782FF);//
REGISTER_COLOR_NAME("\xE3\x81\xA4\xE3\x81\xA4\xE3\x81\x98\xE8\x89\xB2", 0xD24782FF);//F
REGISTER_COLOR_NAME("\xE3\x81\xA4\xE3\x81\xA4\xE3\x81\x98", 0xD24782FF);//
REGISTER_COLOR_NAME("\xE8\xBA\x91\xE8\xBA\x85\xE8\x89\xB2", 0xD24782FF);//UPF
REGISTER_COLOR_NAME("\xE8\xBA\x91\xE8\xBA\x85", 0xD24782FF);//UP
REGISTER_COLOR_NAME("\xE6\xA1\x9C\xE8\x89\xB2", 0xFCDDEBFF);//F
REGISTER_COLOR_NAME("\xE6\xA1\x9C", 0xFCDDEBFF);//
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x81\x8F\xE3\x82\x89\xE3\x81\x84\xE3\x82\x8D", 0xFCDDEBFF);//炢
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x81\x8F\xE3\x82\x89\xE8\x89\xB2", 0xFCDDEBFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x81\x8F\xE3\x82\x89", 0xFCDDEBFF);//
REGISTER_COLOR_NAME("\xE3\x81\xB0\xE3\x82\x89\xE8\x89\xB2", 0xD7456BFF);//΂F
REGISTER_COLOR_NAME("\xE3\x81\xB0\xE3\x82\x89\xE3\x81\x84\xE3\x82\x8D", 0xD7456BFF);//΂炢
REGISTER_COLOR_NAME("\xE3\x81\xB0\xE3\x82\x89", 0xD7456BFF);//΂
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x89\xE3\x81\x8F\xE3\x82\x8C\xE3\x81\xAA\xE3\x81\x84", 0xE65475FF);//炭Ȃ
REGISTER_COLOR_NAME("\xE5\x94\x90\xE7\xB4\x85", 0xE65475FF);//g
REGISTER_COLOR_NAME("\xE9\x9F\x93\xE7\xB4\x85", 0xE65475FF);//؍g
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x82\x93\xE3\x81\x94\xE8\x89\xB2", 0xFF8899FF);//񂲐F
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x82\x93\xE3\x81\x94\xE3\x81\x84\xE3\x82\x8D", 0xFF8899FF);//񂲂
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x82\x93\xE3\x81\x94", 0xFF8899FF);//
REGISTER_COLOR_NAME("\xE7\x8F\x8A\xE7\x91\x9A", 0xFF8899FF);//X
REGISTER_COLOR_NAME("\xE7\xB4\x85\xE6\xA2\x85\xE8\x89\xB2", 0xE08A95FF);//g~F
REGISTER_COLOR_NAME("\xE7\xB4\x85\xE6\xA2\x85", 0xE08A95FF);//g~
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x84\xE3\x81\xB0\xE3\x81\x84\xE3\x81\x84\xE3\x82\x8D", 0xE08A95FF);//΂
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x84\xE3\x81\xB0\xE3\x81\x84\xE8\x89\xB2", 0xE08A95FF);//΂F
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x84\xE3\x81\xB0\xE3\x81\x84", 0xE08A95FF);//΂
REGISTER_COLOR_NAME("\xE6\xA1\x83\xE8\x89\xB2", 0xE48894FF);//F
REGISTER_COLOR_NAME("\xE3\x82\x82\xE3\x82\x82\xE3\x81\x84\xE3\x82\x8D", 0xE48894FF);//
REGISTER_COLOR_NAME("\xE3\x82\x82\xE3\x82\x82\xE8\x89\xB2", 0xE48894FF);//F
REGISTER_COLOR_NAME("\xE3\x82\x82\xE3\x82\x82", 0xE48894FF);//
REGISTER_COLOR_NAME("\xE7\xB4\x85\xE8\x89\xB2", 0xBF1347FF);//gF
REGISTER_COLOR_NAME("\xE7\xB4\x85", 0xBF1347FF);//g
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x81\xAB\xE3\x81\x84\xE3\x82\x8D", 0xBF1347FF);//ׂɂ
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x81\xAB\xE8\x89\xB2", 0xBF1347FF);//ׂɐF
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x81\xAB", 0xBF1347FF);//ׂ
REGISTER_COLOR_NAME("\xE7\xB4\x85\xE8\xB5\xA4", 0xB92145FF);//g
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x81\xAB\xE3\x81\x82\xE3\x81\x8B", 0xB92145FF);//ׂɂ
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x82\x93\xE3\x81\x98", 0xAF3447FF);//
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x82\x93\xE3\x81\x98\xE3\x81\x84\xE3\x82\x8D", 0xAF3447FF);//񂶂
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x82\x93\xE3\x81\x98\xE8\x89\xB2", 0xAF3447FF);//񂶐F
REGISTER_COLOR_NAME("\xE8\x87\x99\xE8\x84\x82", 0xAF3447FF);//b
REGISTER_COLOR_NAME("\xE8\x87\x99\xE8\x84\x82\xE8\x89\xB2", 0xAF3447FF);//bF
REGISTER_COLOR_NAME("\xE8\x98\x87\xE8\x8A\xB3", 0x984851FF);//hF
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\x8A\xE3\x81\x86", 0x984851FF);//
REGISTER_COLOR_NAME("\xE8\x8C\x9C\xE8\x89\xB2", 0xA11B3CFF);//F
REGISTER_COLOR_NAME("\xE8\x8C\x9C", 0xA11B3CFF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\xAD\xE3\x81\x84\xE3\x82\x8D", 0xA11B3CFF);//˂
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\xAD\xE8\x89\xB2", 0xA11B3CFF);//ːF
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\xAD", 0xA11B3CFF);//
REGISTER_COLOR_NAME("\xE8\xB5\xA4", 0xBF1837FF);//
REGISTER_COLOR_NAME("\xE8\xB5\xA4\xE8\x89\xB2", 0xBF1837FF);//ԐF
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\x84\xE3\x82\x8D", 0xBF1837FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE8\x89\xB2", 0xBF1837FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B", 0xBF1837FF);//
REGISTER_COLOR_NAME("\xE6\x9C\xB1\xE8\x89\xB2", 0xED5153FF);//F
REGISTER_COLOR_NAME("\xE6\x9C\xB1", 0xED5153FF);//
REGISTER_COLOR_NAME("\xE7\xB4\x85\xE6\xA8\xBA\xE8\x89\xB2", 0xA14245FF);//gF
REGISTER_COLOR_NAME("\xE7\xB4\x85\xE6\xA8\xBA", 0xA14245FF);//g
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x81\xAB\xE3\x81\x8B\xE3\x81\xB0\xE3\x81\x84\xE3\x82\x8D", 0xA14245FF);//ׂɂ΂
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x81\xAB\xE3\x81\x8B\xE3\x81\xB0\xE8\x89\xB2", 0xA14245FF);//ׂɂΐF
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x81\xAB\xE3\x81\x8B\xE3\x81\xB0", 0xA14245FF);//ׂɂ
REGISTER_COLOR_NAME("\xE7\xB4\x85\xE7\xB7\x8B", 0xED514CFF);//g
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x81\xAB\xE3\x81\xB2", 0xED514CFF);//ׂɂ
REGISTER_COLOR_NAME("\xE9\x89\x9B\xE4\xB8\xB9\xE8\x89\xB2", 0xD44E45FF);//OF
REGISTER_COLOR_NAME("\xE9\x89\x9B\xE4\xB8\xB9", 0xD44E45FF);//O
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x82\x93\xE3\x81\x9F\xE3\x82\x93\xE3\x81\x84\xE3\x82\x8D", 0xD44E45FF);//񂽂񂢂
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x82\x93\xE3\x81\x9F\xE3\x82\x93\xE8\x89\xB2", 0xD44E45FF);//񂽂F
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x82\x93\xE3\x81\x9F\xE3\x82\x93", 0xD44E45FF);//񂽂
REGISTER_COLOR_NAME("\xE7\xB4\x85\xE6\xB5\xB7\xE8\x80\x81\xE8\x8C\xB6", 0x713736FF);//gCV
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x81\xAB\xE3\x81\x88\xE3\x81\xB3\xE3\x81\xA1\xE3\x82\x83", 0x713736FF);//ׂɂт
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\xB3\xE8\x89\xB2", 0x7E4443FF);//ƂѐF
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\xB3\xE3\x81\x84\xE3\x82\x8D", 0x7E4443FF);//Ƃт
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\xB3", 0x7E4443FF);//Ƃ
REGISTER_COLOR_NAME("\xE9\xB3\xB6\xE8\x89\xB2", 0x7E4443FF);//ΐF
REGISTER_COLOR_NAME("\xE9\xB3\xB6", 0x7E4443FF);//
REGISTER_COLOR_NAME("\xE5\xB0\x8F\xE8\xB1\x86\xE8\x89\xB2", 0x96605DFF);//F
REGISTER_COLOR_NAME("\xE5\xB0\x8F\xE8\xB1\x86", 0x96605DFF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x9A\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0x96605DFF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x9A\xE3\x81\x8D\xE8\x89\xB2", 0x96605DFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x9A\xE3\x81\x8D", 0x96605DFF);//
REGISTER_COLOR_NAME("\xE5\xBC\x81\xE6\x9F\x84\xE8\x89\xB2", 0x8A3C38FF);//ٕF
REGISTER_COLOR_NAME("\xE5\xBC\x81\xE6\x9F\x84", 0x8A3C38FF);//ٕ
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x82\x93\xE3\x81\x8C\xE3\x82\x89\xE3\x81\x84\xE3\x82\x8D", 0x8A3C38FF);//ׂ񂪂炢
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x82\x93\xE3\x81\x8C\xE3\x82\x89\xE8\x89\xB2", 0x8A3C38FF);//ׂ񂪂F
REGISTER_COLOR_NAME("\xE3\x81\xB9\xE3\x82\x93\xE3\x81\x8C\xE3\x82\x89", 0x8A3C38FF);//ׂ񂪂
REGISTER_COLOR_NAME("\xE6\xB5\xB7\xE8\x80\x81\xE8\x8C\xB6", 0x6E3938FF);//CV
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x81\xB3\xE3\x81\xA1\xE3\x82\x83", 0x6E3938FF);//т
REGISTER_COLOR_NAME("\xE9\x87\x91\xE8\xB5\xA4", 0xEA5633FF);//
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x82\x93\xE3\x81\x82\xE3\x81\x8B", 0xEA5633FF);//񂠂
REGISTER_COLOR_NAME("\xE8\xB5\xA4\xE8\x8C\xB6", 0xB25040FF);//Ԓ
REGISTER_COLOR_NAME("\xE8\xB5\xA4\xE8\x8C\xB6\xE8\x89\xB2", 0xB25040FF);//ԒF
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\xA1\xE3\x82\x83\xE3\x81\x84\xE3\x82\x8D", 0xB25040FF);//Ⴂ
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\xA1\xE3\x82\x83\xE8\x89\xB2", 0xB25040FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\xA1\xE3\x82\x83", 0xB25040FF);//
REGISTER_COLOR_NAME("\xE8\xB5\xA4\xE9\x8C\x86\xE8\x89\xB2", 0x92362CFF);//ԎKF
REGISTER_COLOR_NAME("\xE8\xB5\xA4\xE9\x8C\x86", 0x92362CFF);//ԎK
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\x95\xE3\x81\xB3\xE3\x81\x84\xE3\x82\x8D", 0x92362CFF);//т
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\x95\xE3\x81\xB3\xE8\x89\xB2", 0x92362CFF);//ѐF
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\x95\xE3\x81\xB3", 0x92362CFF);//
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE4\xB8\xB9", 0xEE6F48FF);//O
REGISTER_COLOR_NAME("\xE3\x81\x8A\xE3\x81\x86\xE3\x81\xAB", 0xEE6F48FF);//
REGISTER_COLOR_NAME("\xE3\x81\x8A\xE3\x81\x86\xE3\x81\x9F\xE3\x82\x93", 0xEE6F48FF);//
REGISTER_COLOR_NAME("\xE8\xB5\xA4\xE6\xA9\x99", 0xE95926FF);//Ԟ
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x81\xA0\xE3\x81\x84\xE3\x81\xA0\xE3\x81\x84", 0xE95926FF);//
REGISTER_COLOR_NAME("\xE6\x9F\xBF\xE8\x89\xB2", 0xDF613AFF);//`F
REGISTER_COLOR_NAME("\xE6\x9F\xBF", 0xDF613AFF);//`
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0xDF613AFF);//
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x8D\xE8\x89\xB2", 0xDF613AFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x8D", 0xDF613AFF);//
REGISTER_COLOR_NAME("\xE8\x82\x89\xE6\xA1\x82\xE8\x89\xB2", 0xBB7668FF);//jF
REGISTER_COLOR_NAME("\xE8\x82\x89\xE6\xA1\x82", 0xBB7668FF);//j
REGISTER_COLOR_NAME("\xE3\x81\xAB\xE3\x81\xA3\xE3\x81\x91\xE3\x81\x84\xE3\x81\x84\xE3\x82\x8D", 0xBB7668FF);//ɂ
REGISTER_COLOR_NAME("\xE3\x81\xAB\xE3\x81\xA3\xE3\x81\x91\xE3\x81\x84\xE8\x89\xB2", 0xBB7668FF);//ɂF
REGISTER_COLOR_NAME("\xE3\x81\xAB\xE3\x81\xA3\xE3\x81\x91\xE3\x81\x84", 0xBB7668FF);//ɂ
REGISTER_COLOR_NAME("\xE6\xA8\xBA\xE8\x89\xB2", 0xBB492AFF);//F
REGISTER_COLOR_NAME("\xE6\xA8\xBA", 0xBB492AFF);//
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\xB0\xE3\x81\x84\xE3\x82\x8D", 0xBB492AFF);//΂
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\xB0\xE8\x89\xB2", 0xBB492AFF);//ΐF
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\xB0", 0xBB492AFF);//
REGISTER_COLOR_NAME("\xE3\x82\x8C\xE3\x82\x93\xE3\x81\x8C\xE3\x81\x84\xE3\x82\x8D", 0x974B3CFF);//񂪂
REGISTER_COLOR_NAME("\xE3\x82\x8C\xE3\x82\x93\xE3\x81\x8C\xE8\x89\xB2", 0x974B3CFF);//񂪐F
REGISTER_COLOR_NAME("\xE3\x82\x8C\xE3\x82\x93\xE3\x81\x8C", 0x974B3CFF);//
REGISTER_COLOR_NAME("\xE9\x8C\x86\xE8\x89\xB2", 0x673D39FF);//KF
REGISTER_COLOR_NAME("\xE9\x8C\x86", 0x673D39FF);//K
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x81\xB3\xE3\x81\x84\xE3\x82\x8D", 0x673D39FF);//т
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x81\xB3\xE8\x89\xB2", 0x673D39FF);//ѐF
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x81\xB3", 0x673D39FF);//
REGISTER_COLOR_NAME("\xE6\xAA\x9C\xE7\x9A\xAE\xE8\x89\xB2", 0x8D5D54FF);//wF
REGISTER_COLOR_NAME("\xE6\xA1\xA7\xE7\x9A\xAE\xE8\x89\xB2", 0x8D5D54FF);//OF
REGISTER_COLOR_NAME("\xE6\xAA\x9C\xE7\x9A\xAE", 0x8D5D54FF);//w
REGISTER_COLOR_NAME("\xE6\xA1\xA7\xE7\x9A\xAE", 0x8D5D54FF);//O
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x82\x8F\xE3\x81\xA0\xE3\x81\x84\xE3\x82\x8D", 0x8D5D54FF);//Ђ킾
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x82\x8F\xE3\x81\xA0\xE8\x89\xB2", 0x8D5D54FF);//Ђ킾F
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x82\x8F\xE3\x81\xA0", 0x8D5D54FF);//Ђ킾
REGISTER_COLOR_NAME("\xE6\xA0\x97\xE8\x89\xB2", 0x77493EFF);//IF
REGISTER_COLOR_NAME("\xE6\xA0\x97", 0x77493EFF);//I
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x82\x8A\xE3\x81\x84\xE3\x82\x8D", 0x77493EFF);//肢
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x82\x8A\xE8\x89\xB2", 0x77493EFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x82\x8A", 0x77493EFF);//
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE8\xB5\xA4", 0xDC6511FF);//
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE8\xB5\xA4\xE8\x89\xB2", 0xDC6511FF);//ԐF
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\x82\xE3\x81\x8B\xE3\x81\x84\xE3\x82\x8D", 0xDC6511FF);//
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\x82\xE3\x81\x8B\xE8\x89\xB2", 0xDC6511FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\x82\xE3\x81\x8B", 0xDC6511FF);//
REGISTER_COLOR_NAME("\xE3\x81\x9F\xE3\x81\x84\xE3\x81\x97\xE3\x82\x83", 0xBA633DFF);//
REGISTER_COLOR_NAME("\xE4\xBB\xA3\xE8\xB5\xAD", 0xBA633DFF);//
REGISTER_COLOR_NAME("\xE4\xBB\xA3\xE8\xB5\xAD\xE8\x89\xB2", 0xBA633DFF);//ސF
REGISTER_COLOR_NAME("\xE3\x81\x9F\xE3\x81\x84\xE3\x81\x97\xE3\x82\x83\xE3\x81\x84\xE3\x82\x8D", 0xBA633DFF);//Ⴂ
REGISTER_COLOR_NAME("\xE3\x81\x9F\xE3\x81\x84\xE3\x81\x97\xE3\x82\x83\xE8\x89\xB2", 0xBA633DFF);//F
REGISTER_COLOR_NAME("\xE3\x82\x89\xE3\x81\x8F\xE3\x81\xA0\xE8\x89\xB2", 0xB8795BFF);//炭F
REGISTER_COLOR_NAME("\xE3\x82\x89\xE3\x81\x8F\xE3\x81\xA0\xE3\x81\x84\xE3\x82\x8D", 0xB8795BFF);//炭
REGISTER_COLOR_NAME("\xE3\x82\x89\xE3\x81\x8F\xE3\x81\xA0", 0xB8795BFF);//炭
REGISTER_COLOR_NAME("\xE9\xA7\xB1\xE9\xA7\x9D", 0xB8795BFF);//pk
REGISTER_COLOR_NAME("\xE9\xA7\xB1\xE9\xA7\x9D\xE8\x89\xB2", 0xB8795BFF);//pkF
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE8\x8C\xB6", 0xB96530FF);//
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE8\x8C\xB6\xE8\x89\xB2", 0xB96530FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xA1\xE3\x82\x83\xE3\x81\x84\xE3\x82\x8D", 0xB96530FF);//Ⴂ
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xA1\xE3\x82\x83\xE8\x89\xB2", 0xB96530FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xA1\xE3\x82\x83", 0xB96530FF);//
REGISTER_COLOR_NAME("\xE8\x82\x8C\xE8\x89\xB2", 0xF7BEA2FF);//F
REGISTER_COLOR_NAME("\xE8\x82\x8C", 0xF7BEA2FF);//
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xA0\xE3\x81\x84\xE3\x82\x8D", 0xF7BEA2FF);//͂
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xA0\xE8\x89\xB2", 0xF7BEA2FF);//͂F
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xA0", 0xF7BEA2FF);//͂
REGISTER_COLOR_NAME("\xE6\xA9\x99\xE8\x89\xB2", 0xF38612FF);//F
REGISTER_COLOR_NAME("\xE6\xA9\x99", 0xF38612FF);//
REGISTER_COLOR_NAME("\xE3\x81\xA0\xE3\x81\x84\xE3\x81\xA0\xE3\x81\x84\xE3\x81\x84\xE3\x82\x8D", 0xF38612FF);//
REGISTER_COLOR_NAME("\xE3\x81\xA0\xE3\x81\x84\xE3\x81\xA0\xE3\x81\x84\xE8\x89\xB2", 0xF38612FF);//F
REGISTER_COLOR_NAME("\xE3\x81\xA0\xE3\x81\x84\xE3\x81\xA0\xE3\x81\x84", 0xF38612FF);//
REGISTER_COLOR_NAME("\xE7\x81\xB0\xE8\x8C\xB6", 0x886659FF);//D
REGISTER_COLOR_NAME("\xE7\x81\xB0\xE8\x8C\xB6\xE8\x89\xB2", 0x886659FF);//DF
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x84\xE3\x81\xA1\xE3\x82\x83\xE3\x81\x84\xE3\x82\x8D", 0x886659FF);//͂Ⴂ
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x84\xE3\x81\xA1\xE3\x82\x83\xE8\x89\xB2", 0x886659FF);//͂F
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x84\xE3\x81\xA1\xE3\x82\x83", 0x886659FF);//͂
REGISTER_COLOR_NAME("\xE8\x8C\xB6\xE8\x89\xB2", 0x744B38FF);//F
REGISTER_COLOR_NAME("\xE8\x8C\xB6", 0x744B38FF);//
REGISTER_COLOR_NAME("\xE3\x81\xA1\xE3\x82\x83\xE3\x81\x84\xE3\x82\x8D", 0x744B38FF);//Ⴂ
REGISTER_COLOR_NAME("\xE3\x81\xA1\xE3\x82\x83\xE8\x89\xB2", 0x744B38FF);//F
REGISTER_COLOR_NAME("\xE3\x81\xA1\xE3\x82\x83", 0x744B38FF);//
REGISTER_COLOR_NAME("\xE7\x84\xA6\xE8\x8C\xB6", 0x5B433CFF);//Œ
REGISTER_COLOR_NAME("\xE7\x84\xA6\xE8\x8C\xB6\xE8\x89\xB2", 0x5B433CFF);//ŒF
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x92\xE8\x8C\xB6\xE8\x89\xB2", 0x5B433CFF);//F
REGISTER_COLOR_NAME("\xE7\x84\xA6\xE3\x81\x92\xE8\x8C\xB6\xE8\x89\xB2", 0x5B433CFF);//łF
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x92\xE8\x8C\xB6", 0x5B433CFF);//
REGISTER_COLOR_NAME("\xE7\x84\xA6\xE3\x81\x92\xE8\x8C\xB6", 0x5B433CFF);//ł
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x92\xE3\x81\xA1\xE3\x82\x83\xE3\x81\x84\xE3\x82\x8D", 0x5B433CFF);//Ⴂ
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x92\xE3\x81\xA1\xE3\x82\x83\xE8\x89\xB2", 0x5B433CFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x92\xE3\x81\xA1\xE3\x82\x83", 0x5B433CFF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x86\xE3\x81\x98\xE8\x89\xB2", 0xFFA96AFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x86\xE3\x81\x98\xE3\x81\x84\xE3\x82\x8D", 0xFFA96AFF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x86\xE3\x81\x98\xE8\x89\xB2", 0xFFA96AFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x86\xE3\x81\x98", 0xFFA96AFF);//
REGISTER_COLOR_NAME("\xE6\x9D\x8F\xE8\x89\xB2", 0xDFA27CFF);//ǐF
REGISTER_COLOR_NAME("\xE6\x9D\x8F", 0xDFA27CFF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x82\x93\xE3\x81\x9A\xE3\x81\x84\xE3\x82\x8D", 0xDFA27CFF);//񂸂
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x82\x93\xE3\x81\x9A\xE8\x89\xB2", 0xDFA27CFF);//񂸐F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x82\x93\xE3\x81\x9A", 0xDFA27CFF);//
REGISTER_COLOR_NAME("\xE8\x9C\x9C\xE6\x9F\x91\xE8\x89\xB2", 0xEE8902FF);//F
REGISTER_COLOR_NAME("\xE8\x9C\x9C\xE6\x9F\x91", 0xEE8902FF);//
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\x8B\xE3\x82\x93\xE3\x81\x84\xE3\x82\x8D", 0xEE8902FF);//݂񂢂
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\x8B\xE3\x82\x93\xE8\x89\xB2", 0xEE8902FF);//݂F
REGISTER_COLOR_NAME("\xE3\x83\x9F\xE3\x82\xAB\xE3\x83\xB3\xE8\x89\xB2", 0xEE8902FF);//~JF
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\x8B\xE3\x82\x93", 0xEE8902FF);//݂
REGISTER_COLOR_NAME("\xE3\x83\x9F\xE3\x82\xAB\xE3\x83\xB3", 0xEE8902FF);//~J
REGISTER_COLOR_NAME("\xE8\xA4\x90\xE8\x89\xB2", 0x713A0AFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\xA3\xE3\x81\x97\xE3\x82\x87\xE3\x81\x8F", 0x713A0AFF);//傭
REGISTER_COLOR_NAME("\xE5\x9C\x9F\xE8\x89\xB2", 0xA96E38FF);//yF
REGISTER_COLOR_NAME("\xE3\x81\xA4\xE3\x81\xA1\xE3\x81\x84\xE3\x82\x8D", 0xA96E38FF);//
REGISTER_COLOR_NAME("\xE3\x81\xA4\xE3\x81\xA1\xE8\x89\xB2", 0xA96E38FF);//F
REGISTER_COLOR_NAME("\xE5\xB0\x8F\xE9\xBA\xA6\xE8\x89\xB2", 0xDCA476FF);//F
REGISTER_COLOR_NAME("\xE5\xB0\x8F\xE9\xBA\xA6", 0xDCA476FF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x82\x80\xE3\x81\x8E\xE3\x81\x84\xE3\x82\x8D", 0xDCA476FF);//ނ
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x82\x80\xE3\x81\x8E\xE8\x89\xB2", 0xDCA476FF);//ނF
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\xAF\xE3\x81\x8F\xE8\x89\xB2", 0xB47C49FF);//͂F
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\xAF\xE3\x81\x8F\xE3\x81\x84\xE3\x82\x8D", 0xB47C49FF);//͂
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\xAF\xE3\x81\x8F", 0xB47C49FF);//͂
REGISTER_COLOR_NAME("\xE7\x90\xA5\xE7\x8F\x80\xE8\x89\xB2", 0xB47C49FF);//ߐF
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\xAF\xE3\x81\x8F", 0xB47C49FF);//͂
REGISTER_COLOR_NAME("\xE9\x87\x91\xE8\x8C\xB6", 0xBD790DFF);//
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x82\x93\xE3\x81\xA1\xE3\x82\x83", 0xBD790DFF);//񂿂
REGISTER_COLOR_NAME("\xE5\x8D\xB5\xE8\x89\xB2", 0xFAC076FF);//F
REGISTER_COLOR_NAME("\xE5\x8D\xB5", 0xFAC076FF);//
REGISTER_COLOR_NAME("\xE3\x81\x9F\xE3\x81\xBE\xE3\x81\x94\xE3\x81\x84\xE3\x82\x8D", 0xFAC076FF);//܂
REGISTER_COLOR_NAME("\xE3\x81\x9F\xE3\x81\xBE\xE3\x81\x94\xE8\x89\xB2", 0xFAC076FF);//܂F
REGISTER_COLOR_NAME("\xE3\x81\x9F\xE3\x81\xBE\xE3\x81\x94", 0xFAC076FF);//܂
REGISTER_COLOR_NAME("\xE5\xB1\xB1\xE5\x90\xB9\xE8\x89\xB2", 0xFDAD00FF);//RF
REGISTER_COLOR_NAME("\xE5\xB1\xB1\xE5\x90\xB9", 0xFDAD00FF);//R
REGISTER_COLOR_NAME("\xE3\x82\x84\xE3\x81\xBE\xE3\x81\xB6\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0xFDAD00FF);//܂Ԃ
REGISTER_COLOR_NAME("\xE3\x82\x84\xE3\x81\xBE\xE3\x81\xB6\xE3\x81\x8D\xE8\x89\xB2", 0xFDAD00FF);//܂ԂF
REGISTER_COLOR_NAME("\xE3\x82\x84\xE3\x81\xBE\xE3\x81\xB6\xE3\x81\x8D", 0xFDAD00FF);//܂Ԃ
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE5\x9C\x9F\xE8\x89\xB2", 0xC18A42FF);//yF
REGISTER_COLOR_NAME("\xE3\x81\x8A\xE3\x81\x86\xE3\x81\xA9\xE3\x81\x84\xE3\x82\x8D", 0xC18A42FF);//ǂ
REGISTER_COLOR_NAME("\xE3\x81\x8A\xE3\x81\x86\xE3\x81\xA9\xE8\x89\xB2", 0xC18A42FF);//ǐF
REGISTER_COLOR_NAME("\xE3\x81\x8A\xE3\x81\x86\xE3\x81\xA9", 0xC18A42FF);//
REGISTER_COLOR_NAME("\xE6\x9C\xBD\xE8\x91\x89\xE8\x89\xB2", 0x8B776BFF);//tF
REGISTER_COLOR_NAME("\xE6\x9C\xBD\xE8\x91\x89", 0x8B776BFF);//t
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x81\xA1\xE3\x81\xB0\xE3\x81\x84\xE3\x82\x8D", 0x8B776BFF);//΂
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x81\xA1\xE3\x81\xB0\xE8\x89\xB2", 0x8B776BFF);//ΐF
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x81\xA1\xE3\x81\xB0", 0x8B776BFF);//
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x81\xBE\xE3\x82\x8F\xE3\x82\x8A\xE8\x89\xB2", 0xFFBE00FF);//Ђ܂F
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x81\xBE\xE3\x82\x8F\xE3\x82\x8A\xE3\x81\x84\xE3\x82\x8D", 0xFFBE00FF);//Ђ܂肢
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x81\xBE\xE3\x82\x8F\xE3\x82\x8A", 0xFFBE00FF);//Ђ܂
REGISTER_COLOR_NAME("\xE5\x90\x91\xE6\x97\xA5\xE8\x91\xB5\xE8\x89\xB2", 0xFFBE00FF);//F
REGISTER_COLOR_NAME("\xE5\x90\x91\xE6\x97\xA5\xE8\x91\xB5", 0xFFBE00FF);//
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x93\xE3\x82\x93\xE8\x89\xB2", 0xF2B202FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x93\xE3\x82\x93\xE3\x81\x84\xE3\x82\x8D", 0xF2B202FF);//񂢂
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x93\xE3\x82\x93", 0xF2B202FF);//
REGISTER_COLOR_NAME("\xE9\xAC\xB1\xE9\x87\x91", 0xF2B202FF);//T
REGISTER_COLOR_NAME("\xE9\xAC\xB1\xE9\x87\x91\xE8\x89\xB2", 0xF2B202FF);//TF
REGISTER_COLOR_NAME("\xE7\xA0\x82\xE8\x89\xB2", 0xCCBAA9FF);//F
REGISTER_COLOR_NAME("\xE7\xA0\x82", 0xCCBAA9FF);//
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\xAA\xE3\x81\x84\xE3\x82\x8D", 0xCCBAA9FF);//Ȃ
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\xAA\xE8\x89\xB2", 0xCCBAA9FF);//ȐF
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\xAA", 0xCCBAA9FF);//
REGISTER_COLOR_NAME("\xE8\x8A\xA5\xE5\xAD\x90\xE8\x89\xB2", 0xCFAA69FF);//HqF
REGISTER_COLOR_NAME("\xE8\x8A\xA5\xE5\xAD\x90", 0xCFAA69FF);//Hq
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x89\xE3\x81\x97\xE3\x81\x84\xE3\x82\x8D", 0xCFAA69FF);//炵
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x89\xE3\x81\x97\xE8\x89\xB2", 0xCFAA69FF);//炵F
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x89\xE3\x81\x97", 0xCFAA69FF);//炵
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE8\x89\xB2", 0xF3C600FF);//F
REGISTER_COLOR_NAME("\xE9\xBB\x84", 0xF3C600FF);//
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0xF3C600FF);//
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE8\x89\xB2", 0xF3C600FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8D", 0xF3C600FF);//
REGISTER_COLOR_NAME("\xE3\x81\x9F\xE3\x82\x93\xE3\x81\xBD\xE3\x81\xBD\xE3\x81\x84\xE3\x82\x8D", 0xF3C600FF);//ۂۂ
REGISTER_COLOR_NAME("\xE3\x81\x9F\xE3\x82\x93\xE3\x81\xBD\xE3\x81\xBD\xE8\x89\xB2", 0xF3C600FF);//ۂېF
REGISTER_COLOR_NAME("\xE3\x81\x9F\xE3\x82\x93\xE3\x81\xBD\xE3\x81\xBD", 0xF3C600FF);//ۂ
REGISTER_COLOR_NAME("\xE8\x92\xB2\xE5\x85\xAC\xE8\x8B\xB1\xE8\x89\xB2", 0xF3C600FF);//pF
REGISTER_COLOR_NAME("\xE8\x92\xB2\xE5\x85\xAC\xE8\x8B\xB1", 0xF3C600FF);//p
REGISTER_COLOR_NAME("\xE9\xB6\xAF\xE8\x8C\xB6", 0x705F3BFF);//
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x90\xE3\x81\x84\xE3\x81\x99\xE3\x81\xA1\xE3\x82\x83", 0x705F3BFF);//
REGISTER_COLOR_NAME("\xE4\xB8\xAD\xE9\xBB\x84", 0xF5D726FF);//
REGISTER_COLOR_NAME("\xE3\x81\xA1\xE3\x82\x85\xE3\x81\x86\xE3\x81\x8D", 0xF5D726FF);//イ
REGISTER_COLOR_NAME("\xE5\x88\x88\xE5\xAE\x89\xE8\x89\xB2", 0xEED77AFF);//F
REGISTER_COLOR_NAME("\xE5\x88\x88\xE5\xAE\x89", 0xEED77AFF);//
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x8A\xE3\x82\x84\xE3\x81\x99\xE3\x81\x84\xE3\x82\x8D", 0xEED77AFF);//₷
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x8A\xE3\x82\x84\xE3\x81\x99\xE8\x89\xB2", 0xEED77AFF);//₷F
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x8A\xE3\x82\x84\xE3\x81\x99", 0xEED77AFF);//₷
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xAF\xE3\x81\xA0\xE8\x89\xB2", 0xDCCC5EFF);//͂F
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xAF\xE3\x81\xA0\xE3\x81\x84\xE3\x82\x8D", 0xDCCC5EFF);//͂
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xAF\xE3\x81\xA0", 0xDCCC5EFF);//͂
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x82\x8B\xE8\x89\xB2", 0x756E54FF);//݂F
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x82\x8B\xE3\x81\x84\xE3\x82\x8D", 0x756E54FF);//݂邢
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x82\x8B", 0x756E54FF);//݂
REGISTER_COLOR_NAME("\xE6\xB5\xB7\xE6\x9D\xBE\xE8\x89\xB2", 0x756E54FF);//CF
REGISTER_COLOR_NAME("\xE6\xB5\xB7\xE6\x9D\xBE", 0x756E54FF);//C
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x82\x8F\xE8\x89\xB2", 0xC6C153FF);//ЂF
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x82\x8F\xE3\x81\x84\xE3\x82\x8D", 0xC6C153FF);//Ђ킢
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x82\x8F", 0xC6C153FF);//Ђ
REGISTER_COLOR_NAME("\xE9\xB6\xB8\xE8\x89\xB2", 0xC6C153FF);//SF
REGISTER_COLOR_NAME("\xE9\xB6\xB8", 0xC6C153FF);//S
REGISTER_COLOR_NAME("\xE9\xB6\xAF\xE8\x89\xB2", 0x737048FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x90\xE3\x81\x84\xE3\x81\x99\xE3\x81\x84\xE3\x82\x8D", 0x737048FF);//
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x90\xE3\x81\x84\xE3\x81\x99\xE8\x89\xB2", 0x737048FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x90\xE3\x81\x84\xE3\x81\x99", 0x737048FF);//
REGISTER_COLOR_NAME("\xE6\x8A\xB9\xE8\x8C\xB6\xE8\x89\xB2", 0xC2BF8EFF);//F
REGISTER_COLOR_NAME("\xE6\x8A\xB9\xE8\x8C\xB6", 0xC2BF8EFF);//
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\xA3\xE3\x81\xA1\xE3\x82\x83\xE3\x81\x84\xE3\x82\x8D", 0xC2BF8EFF);//܂Ⴂ
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\xA3\xE3\x81\xA1\xE3\x82\x83\xE8\x89\xB2", 0xC2BF8EFF);//܂F
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\xA3\xE3\x81\xA1\xE3\x82\x83", 0xC2BF8EFF);//܂
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE7\xB7\x91", 0xBEC500FF);//
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE7\xB7\x91\xE8\x89\xB2", 0xBEC500FF);//ΐF
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A\xE3\x81\x84\xE3\x82\x8D", 0xBEC500FF);//݂ǂ肢
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A\xE8\x89\xB2", 0xBEC500FF);//݂ǂF
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A", 0xBEC500FF);//݂ǂ
REGISTER_COLOR_NAME("\xE8\x8B\x94\xE8\x89\xB2", 0x7C7F43FF);//ېF
REGISTER_COLOR_NAME("\xE8\x8B\x94", 0x7C7F43FF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x91\xE3\x81\x84\xE3\x82\x8D", 0x7C7F43FF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x91\xE8\x89\xB2", 0x7C7F43FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x91", 0x7C7F43FF);//
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x8D\x89\xE8\x89\xB2", 0xADB71FFF);//ᑐF
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x8D\x89", 0xADB71FFF);//ᑐ
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\x8F\xE3\x81\x95\xE3\x81\x84\xE3\x82\x8D", 0xADB71FFF);//킩
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\x8F\xE3\x81\x95\xE8\x89\xB2", 0xADB71FFF);//킩F
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\x8F\xE3\x81\x95", 0xADB71FFF);//킩
REGISTER_COLOR_NAME("\xE8\x90\x8C\xE9\xBB\x84", 0x9AAB29FF);//G
REGISTER_COLOR_NAME("\xE3\x82\x82\xE3\x81\x88\xE3\x81\x8E", 0x9AAB29FF);//
REGISTER_COLOR_NAME("\xE8\x8D\x89\xE8\x89\xB2", 0x748049FF);//F
REGISTER_COLOR_NAME("\xE8\x8D\x89", 0x748049FF);//
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x81\x95\xE3\x81\x84\xE3\x82\x8D", 0x748049FF);//
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x81\x95\xE8\x89\xB2", 0x748049FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x81\x95", 0x748049FF);//
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x91\x89\xE8\x89\xB2", 0xB0C494FF);//tF
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x91\x89", 0xB0C494FF);//t
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\xB0\xE3\x81\x84\xE3\x82\x8D", 0xB0C494FF);//킩΂
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\xB0\xE8\x89\xB2", 0xB0C494FF);//킩ΐF
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\xB0", 0xB0C494FF);//킩
REGISTER_COLOR_NAME("\xE6\x9D\xBE\xE8\x91\x89\xE8\x89\xB2", 0x6D815EFF);//tF
REGISTER_COLOR_NAME("\xE6\x9D\xBE\xE8\x91\x89", 0x6D815EFF);//t
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\xA4\xE3\x81\xB0\xE3\x81\x84\xE3\x82\x8D", 0x6D815EFF);//܂΂
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\xA4\xE3\x81\xB0\xE8\x89\xB2", 0x6D815EFF);//܂ΐF
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\xA4\xE3\x81\xB0", 0x6D815EFF);//܂
REGISTER_COLOR_NAME("\xE7\x99\xBD\xE7\xB7\x91", 0xC5DED0FF);//
REGISTER_COLOR_NAME("\xE3\x81\xB3\xE3\x82\x83\xE3\x81\x8F\xE3\x82\x8D\xE3\x81\x8F", 0xC5DED0FF);//тႭ낭
REGISTER_COLOR_NAME("\xE7\xB7\x91", 0x3EB775FF);//
REGISTER_COLOR_NAME("\xE7\xB7\x91\xE8\x89\xB2", 0x3EB775FF);//ΐF
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A\xE3\x81\x84\xE3\x82\x8D", 0x3EB775FF);//݂ǂ肢
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A\xE8\x89\xB2", 0x3EB775FF);//݂ǂF
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A", 0x3EB775FF);//݂ǂ
REGISTER_COLOR_NAME("\xE5\xB8\xB8\xE7\xA3\x90\xE8\x89\xB2", 0x217C56FF);//֐F
REGISTER_COLOR_NAME("\xE5\xB8\xB8\xE7\xA3\x90", 0x217C56FF);//
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x8D\xE3\x82\x8F\xE3\x81\x84\xE3\x82\x8D", 0x217C56FF);//Ƃ킢
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x8D\xE3\x82\x8F\xE8\x89\xB2", 0x217C56FF);//ƂF
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x8D\xE3\x82\x8F", 0x217C56FF);//Ƃ
REGISTER_COLOR_NAME("\xE7\xB7\x91\xE9\x9D\x92\xE8\x89\xB2", 0x5C8371FF);//ΐF
REGISTER_COLOR_NAME("\xE7\xB7\x91\xE9\x9D\x92", 0x5C8371FF);//ΐ
REGISTER_COLOR_NAME("\xE3\x82\x8D\xE3\x81\x8F\xE3\x81\x97\xE3\x82\x87\xE3\x81\x86\xE8\x89\xB2", 0x5C8371FF);//낭傤F
REGISTER_COLOR_NAME("\xE3\x82\x8D\xE3\x81\x8F\xE3\x81\x97\xE3\x82\x87\xE3\x81\x86\xE3\x81\x84\xE3\x82\x8D", 0x5C8371FF);//낭傤
REGISTER_COLOR_NAME("\xE3\x82\x8D\xE3\x81\x8F\xE3\x81\x97\xE3\x82\x87\xE3\x81\x86", 0x5C8371FF);//낭傤
REGISTER_COLOR_NAME("\xE5\x8D\x83\xE6\xAD\xB3\xE7\xB7\x91", 0x466859FF);//Η
REGISTER_COLOR_NAME("\xE3\x81\xA1\xE3\x81\xA8\xE3\x81\x9B\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A", 0x466859FF);//Ƃ݂ǂ
REGISTER_COLOR_NAME("\xE6\xB7\xB1\xE7\xB7\x91", 0x00553BFF);//[
REGISTER_COLOR_NAME("\xE6\xB7\xB1\xE7\xB7\x91\xE8\x89\xB2", 0x00553BFF);//[ΐF
REGISTER_COLOR_NAME("\xE3\x82\x82\xE3\x81\x88\xE3\x81\x8E\xE8\x89\xB2", 0x125240FF);//F
REGISTER_COLOR_NAME("\xE3\x82\x82\xE3\x81\x88\xE3\x81\x8E\xE3\x81\x84\xE3\x82\x8D", 0x125240FF);//
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE7\xAB\xB9\xE8\x89\xB2", 0x42A686FF);//|F
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE7\xAB\xB9", 0x42A686FF);//|
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\x9F\xE3\x81\x91\xE3\x81\x84\xE3\x82\x8D", 0x42A686FF);//킩
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\x9F\xE3\x81\x91\xE8\x89\xB2", 0x42A686FF);//킩F
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\x9F\xE3\x81\x91", 0x42A686FF);//킩
REGISTER_COLOR_NAME("\xE9\x9D\x92\xE7\xA3\x81\xE8\x89\xB2", 0x7CAC9EFF);//F
REGISTER_COLOR_NAME("\xE9\x9D\x92\xE7\xA3\x81", 0x7CAC9EFF);//
REGISTER_COLOR_NAME("\xE3\x81\x9B\xE3\x81\x84\xE3\x81\x98\xE3\x81\x84\xE3\x82\x8D", 0x7CAC9EFF);//
REGISTER_COLOR_NAME("\xE3\x81\x9B\xE3\x81\x84\xE3\x81\x98\xE8\x89\xB2", 0x7CAC9EFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x9B\xE3\x81\x84\xE3\x81\x98", 0x7CAC9EFF);//
REGISTER_COLOR_NAME("\xE9\x9D\x92\xE7\xAB\xB9\xE8\x89\xB2", 0x78ACA7FF);//|F
REGISTER_COLOR_NAME("\xE9\x9D\x92\xE7\xAB\xB9", 0x78ACA7FF);//|
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE3\x81\x9F\xE3\x81\x91\xE3\x81\x84\xE3\x82\x8D", 0x78ACA7FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE3\x81\x9F\xE3\x81\x91\xE8\x89\xB2", 0x78ACA7FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE3\x81\x9F\xE3\x81\x91", 0x78ACA7FF);//
REGISTER_COLOR_NAME("\xE9\x89\x84\xE8\x89\xB2", 0x284141FF);//SF
REGISTER_COLOR_NAME("\xE9\x89\x84", 0x284141FF);//S
REGISTER_COLOR_NAME("\xE3\x81\xA6\xE3\x81\xA4\xE3\x81\x84\xE3\x82\x8D", 0x284141FF);//Ă
REGISTER_COLOR_NAME("\xE3\x81\xA6\xE3\x81\xA4\xE8\x89\xB2", 0x284141FF);//ĂF
REGISTER_COLOR_NAME("\xE3\x81\xA6\xE3\x81\xA4", 0x284141FF);//Ă
REGISTER_COLOR_NAME("\xE9\x9D\x92\xE7\xB7\x91", 0x0092A0FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A", 0x0092A0FF);//݂ǂ
REGISTER_COLOR_NAME("\xE9\x8C\x86\xE6\xB5\x85\xE8\x91\xB1", 0x6A8D99FF);//KK
REGISTER_COLOR_NAME("\xE3\x81\x95\xE3\x81\xB3\xE3\x81\x82\xE3\x81\x95\xE3\x81\x8E", 0x6A8D99FF);//т
REGISTER_COLOR_NAME("\xE6\xB0\xB4\xE6\xB5\x85\xE8\x91\xB1", 0x779AA8FF);//K
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\x9A\xE3\x81\x82\xE3\x81\x95\xE3\x81\x8E", 0x779AA8FF);//݂
REGISTER_COLOR_NAME("\xE6\x96\xB0\xE6\xA9\x8B\xE8\x89\xB2", 0x61ACC2FF);//VF
REGISTER_COLOR_NAME("\xE6\x96\xB0\xE6\xA9\x8B", 0x69AAC6FF);//V
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x93\xE3\x81\xB0\xE3\x81\x97\xE3\x81\x84\xE3\x82\x8D", 0x61ACC2FF);//΂
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x93\xE3\x81\xB0\xE3\x81\x97\xE8\x89\xB2", 0x61ACC2FF);//΂F
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x93\xE3\x81\xB0\xE3\x81\x97", 0x61ACC2FF);//΂
REGISTER_COLOR_NAME("\xE6\xB5\x85\xE8\x91\xB1\xE8\x89\xB2", 0x0089A7FF);//KF
REGISTER_COLOR_NAME("\xE6\xB5\x85\xE8\x91\xB1", 0x0089A7FF);//K
REGISTER_COLOR_NAME("\xE7\x99\xBD\xE7\xBE\xA4", 0x80B7CCFF);//Q
REGISTER_COLOR_NAME("\xE3\x81\xB3\xE3\x82\x83\xE3\x81\x8F\xE3\x81\x90\xE3\x82\x93", 0x80B7CCFF);//тႭ
REGISTER_COLOR_NAME("\xE7\xB4\x8D\xE6\x88\xB8\xE8\x89\xB2", 0x006A86FF);//[ːF
REGISTER_COLOR_NAME("\xE7\xB4\x8D\xE6\x88\xB8", 0x006A86FF);//[
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x82\x93\xE3\x81\xA9\xE3\x81\x84\xE3\x82\x8D", 0x006A86FF);//Ȃǂ
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x82\x93\xE3\x81\xA9\xE8\x89\xB2", 0x006A86FF);//ȂǐF
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x82\x93\xE3\x81\xA9", 0x006A86FF);//Ȃ
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x81\xE3\x81\xAE\xE3\x81\x9E\xE3\x81\x8D", 0x8AB6CCFF);//߂̂
REGISTER_COLOR_NAME("\xE7\x93\xB6\xE8\xA6\x97\xE3\x81\x8D", 0x8AB6CCFF);//r`
REGISTER_COLOR_NAME("\xE7\x94\x95\xE8\xA6\x97\xE3\x81\x8D", 0x8AB6CCFF);//P`
REGISTER_COLOR_NAME("\xE6\xB0\xB4\xE8\x89\xB2", 0xA6D0E9FF);//F
REGISTER_COLOR_NAME("\xE6\xB0\xB4", 0xA6D0E9FF);//
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\x9A\xE3\x81\x84\xE3\x82\x8D", 0xA6D0E9FF);//݂
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\x9A\xE8\x89\xB2", 0xA6D0E9FF);//݂F
REGISTER_COLOR_NAME("\xE3\x81\xBF\xE3\x81\x9A", 0xA6D0E9FF);//݂
REGISTER_COLOR_NAME("\xE8\x97\x8D\xE9\xBC\xA0", 0x5C7082FF);//l
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84\xE3\x81\xAD\xE3\x81\x9A", 0x5C7082FF);//˂
REGISTER_COLOR_NAME("\xE7\xA9\xBA\xE8\x89\xB2", 0x90C3E9FF);//F
REGISTER_COLOR_NAME("\xE7\xA9\xBA", 0x90C3E9FF);//
REGISTER_COLOR_NAME("\xE3\x81\x9D\xE3\x82\x89\xE3\x81\x84\xE3\x82\x8D", 0x90C3E9FF);//炢
REGISTER_COLOR_NAME("\xE3\x81\x9D\xE3\x82\x89\xE8\x89\xB2", 0x90C3E9FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x9D\xE3\x82\x89", 0x90C3E9FF);//
REGISTER_COLOR_NAME("\xE9\x9D\x92\xE8\x89\xB2", 0x006CC4FF);//F
REGISTER_COLOR_NAME("\xE9\x9D\x92", 0x006CC4FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE3\x81\x84\xE3\x82\x8D", 0x006CC4FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE8\x89\xB2", 0x006CC4FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A", 0x006CC4FF);//
REGISTER_COLOR_NAME("\xE8\x97\x8D\xE8\x89\xB2", 0x234A70FF);//F
REGISTER_COLOR_NAME("\xE8\x97\x8D", 0x234A70FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84\xE3\x81\x84\xE3\x82\x8D", 0x234A70FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84\xE8\x89\xB2", 0x234A70FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84", 0x234A70FF);//
REGISTER_COLOR_NAME("\xE6\xBF\x83\xE8\x97\x8D", 0x1E334BFF);//Z
REGISTER_COLOR_NAME("\xE6\xBF\x83\xE8\x97\x8D\xE8\x89\xB2", 0x1E334BFF);//ZF
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x84\xE3\x81\x82\xE3\x81\x84\xE3\x81\x84\xE3\x82\x8D", 0x1E334BFF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x84\xE3\x81\x82\xE3\x81\x84\xE8\x89\xB2", 0x1E334BFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x84\xE3\x81\x82\xE3\x81\x84", 0x1E334BFF);//
REGISTER_COLOR_NAME("\xE5\x8B\xBF\xE5\xBF\x98\xE8\x8D\x89\xE8\x89\xB2", 0x92B1E1FF);//ܖYF
REGISTER_COLOR_NAME("\xE5\x8B\xBF\xE5\xBF\x98\xE8\x8D\x89", 0x92B1E1FF);//ܖY
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x99\xE3\x82\x8C\xE3\x81\xAA\xE3\x81\x90\xE3\x81\x95\xE3\x81\x84\xE3\x82\x8D", 0x92B1E1FF);//킷Ȃ
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x99\xE3\x82\x8C\xE3\x81\xAA\xE3\x81\x90\xE3\x81\x95\xE8\x89\xB2", 0x92B1E1FF);//킷ȂF
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x99\xE3\x82\x8C\xE3\x81\xAA\xE3\x81\x90\xE3\x81\x95", 0x92B1E1FF);//킷Ȃ
REGISTER_COLOR_NAME("\xE9\x9C\xB2\xE8\x8D\x89\xE8\x89\xB2", 0x007FCEFF);//IF
REGISTER_COLOR_NAME("\xE9\x9C\xB2\xE8\x8D\x89", 0x007FCEFF);//I
REGISTER_COLOR_NAME("\xE3\x81\xA4\xE3\x82\x86\xE3\x81\x8F\xE3\x81\x95\xE3\x81\x84\xE3\x82\x8D", 0x007FCEFF);//䂭
REGISTER_COLOR_NAME("\xE3\x81\xA4\xE3\x82\x86\xE3\x81\x8F\xE3\x81\x95\xE8\x89\xB2", 0x007FCEFF);//䂭F
REGISTER_COLOR_NAME("\xE3\x81\xA4\xE3\x82\x86\xE3\x81\x8F\xE3\x81\x95", 0x007FCEFF);//䂭
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xAA\xE3\x81\xA0\xE8\x89\xB2", 0x2E649AFF);//͂ȂF
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xAA\xE3\x81\xA0\xE3\x81\x84\xE3\x82\x8D", 0x2E649AFF);//͂Ȃ
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xAA\xE3\x81\xA0", 0x2E649AFF);//͂Ȃ
REGISTER_COLOR_NAME("\xE7\xB8\xB9\xE8\x89\xB2", 0x2E649AFF);//|F
REGISTER_COLOR_NAME("\xE7\xB8\xB9", 0x2E649AFF);//|
REGISTER_COLOR_NAME("\xE7\xB4\xBA\xE9\x9D\x92", 0x3A486AFF);//
REGISTER_COLOR_NAME("\xE3\x82\x8B\xE3\x82\x8A\xE8\x89\xB2", 0x0C51A5FF);//F
REGISTER_COLOR_NAME("\xE3\x82\x8B\xE3\x82\x8A\xE3\x81\x84\xE3\x82\x8D", 0x0C51A5FF);//肢
REGISTER_COLOR_NAME("\xE3\x82\x8B\xE3\x82\x8A", 0x0C51A5FF);//
REGISTER_COLOR_NAME("\xE7\x91\xA0\xE7\x92\x83", 0x0C51A5FF);//ڗ
REGISTER_COLOR_NAME("\xE3\x82\x8B\xE3\x82\x8A\xE7\xB4\xBA", 0x284684FF);//荮
REGISTER_COLOR_NAME("\xE7\x91\xA0\xE7\x92\x83\xE7\xB4\xBA", 0x284684FF);//ڗ
REGISTER_COLOR_NAME("\xE3\x82\x8B\xE3\x82\x8A\xE3\x81\x93\xE3\x82\x93", 0x284684FF);//肱
REGISTER_COLOR_NAME("\xE7\xB4\xBA\xE8\x89\xB2", 0x333B5DFF);//F
REGISTER_COLOR_NAME("\xE7\xB4\xBA", 0x333B5DFF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x82\x93\xE3\x81\x84\xE3\x82\x8D", 0x333B5DFF);//񂢂
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x82\x93\xE8\x89\xB2", 0x333B5DFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x82\x93", 0x333B5DFF);//
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x8D\xE3\x81\xA4\xE3\x81\xB0\xE3\x81\x9F\xE8\x89\xB2", 0x475CABFF);//΂F
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x8D\xE3\x81\xA4\xE3\x81\xB0\xE3\x81\x9F\xE3\x81\x84\xE3\x82\x8D", 0x475CABFF);//΂
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x8D\xE3\x81\xA4\xE3\x81\xB0\xE3\x81\x9F", 0x475CABFF);//΂
REGISTER_COLOR_NAME("\xE6\x9D\x9C\xE8\x8B\xA5\xE8\x89\xB2", 0x475CABFF);//mF
REGISTER_COLOR_NAME("\xE6\x9D\x9C\xE8\x8B\xA5", 0x475CABFF);//m
REGISTER_COLOR_NAME("\xE5\x8B\x9D\xE8\x89\xB2", 0x393B56FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\xA1\xE3\x81\x84\xE3\x82\x8D", 0x393B56FF);//
REGISTER_COLOR_NAME("\xE7\xBE\xA4\xE9\x9D\x92\xE8\x89\xB2", 0x424CA4FF);//QF
REGISTER_COLOR_NAME("\xE7\xBE\xA4\xE9\x9D\x92", 0x424CA4FF);//Q
REGISTER_COLOR_NAME("\xE3\x81\x90\xE3\x82\x93\xE3\x81\x98\xE3\x82\x87\xE3\x81\x86\xE3\x81\x84\xE3\x82\x8D", 0x424CA4FF);//񂶂傤
REGISTER_COLOR_NAME("\xE3\x81\x90\xE3\x82\x93\xE3\x81\x98\xE3\x82\x87\xE3\x81\x86\xE8\x89\xB2", 0x424CA4FF);//񂶂傤F
REGISTER_COLOR_NAME("\xE3\x81\x90\xE3\x82\x93\xE3\x81\x98\xE3\x82\x87\xE3\x81\x86", 0x424CA4FF);//񂶂傤
REGISTER_COLOR_NAME("\xE9\x89\x84\xE7\xB4\xBA", 0x262538FF);//S
REGISTER_COLOR_NAME("\xE3\x81\xA6\xE3\x81\xA4\xE3\x81\x93\xE3\x82\x93", 0x262538FF);//Ă
REGISTER_COLOR_NAME("\xE8\x97\xA4\xE7\xB4\x8D\xE6\x88\xB8", 0x6C67A6FF);//[
REGISTER_COLOR_NAME("\xE3\x81\xB5\xE3\x81\x98\xE3\x81\xAA\xE3\x82\x93\xE3\x81\xA9", 0x6C67A6FF);//ӂȂ
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\x8D\xE3\x82\x87\xE3\x81\x86\xE8\x89\xB2", 0x5144ADFF);//傤F
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\x8D\xE3\x82\x87\xE3\x81\x86\xE3\x81\x84\xE3\x82\x8D", 0x5144ADFF);//傤
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\x8D\xE3\x82\x87\xE3\x81\x86", 0x5144ADFF);//傤
REGISTER_COLOR_NAME("\xE6\xA1\x94\xE6\xA2\x97\xE8\x89\xB2", 0x5144ADFF);//j[F
REGISTER_COLOR_NAME("\xE6\xA1\x94\xE6\xA2\x97", 0x5144ADFF);//j[
REGISTER_COLOR_NAME("\xE7\xB4\xBA\xE8\x97\x8D", 0x3C317DFF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x82\x93\xE3\x81\x82\xE3\x81\x84", 0x3C317DFF);//񂠂
REGISTER_COLOR_NAME("\xE8\x97\xA4\xE8\x89\xB2", 0xA49BD5FF);//F
REGISTER_COLOR_NAME("\xE3\x81\xB5\xE3\x81\x98\xE3\x81\x84\xE3\x82\x8D", 0xA49BD5FF);//ӂ
REGISTER_COLOR_NAME("\xE3\x81\xB5\xE3\x81\x98\xE8\x89\xB2", 0xA49BD5FF);//ӂF
REGISTER_COLOR_NAME("\xE3\x81\xB5\xE3\x81\x98", 0xA49BD5FF);//ӂ
REGISTER_COLOR_NAME("\xE8\x97\xA4\xE7\xB4\xAB", 0x9A8AD6FF);//
REGISTER_COLOR_NAME("\xE3\x81\xB5\xE3\x81\x98\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x9A8AD6FF);//ӂނ炳
REGISTER_COLOR_NAME("\xE9\x9D\x92\xE7\xB4\xAB", 0x7E45B4FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0x7E45B4FF);//ނ炳
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D\xE8\x89\xB2", 0x7E45B4FF);//ނ炳F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x7E45B4FF);//ނ炳
REGISTER_COLOR_NAME("\xE8\x8F\xAB\xE8\x89\xB2", 0x774DA5FF);//俐F
REGISTER_COLOR_NAME("\xE8\x8F\xAB", 0x774DA5FF);//
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\xBF\xE3\x82\x8C\xE8\x89\xB2", 0x774DA5FF);//݂F
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\xBF\xE3\x82\x8C\xE3\x81\x84\xE3\x82\x8D", 0x774DA5FF);//݂ꂢ
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\xBF\xE3\x82\x8C", 0x774DA5FF);//݂
REGISTER_COLOR_NAME("\xE9\xB3\xA9\xE7\xBE\xBD\xE8\x89\xB2", 0x6A5B7DFF);//HF
REGISTER_COLOR_NAME("\xE9\xB3\xA9\xE7\xBE\xBD", 0x6A5B7DFF);//H
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xA8\xE3\x81\xB0\xE3\x81\x84\xE3\x82\x8D", 0x6A5B7DFF);//͂Ƃ΂
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xA8\xE3\x81\xB0\xE8\x89\xB2", 0x6A5B7DFF);//͂ƂΐF
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xA8\xE3\x81\xB0", 0x6A5B7DFF);//͂Ƃ
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x87\xE3\x81\x86\xE3\x81\xB6\xE3\x81\x84\xE3\x82\x8D", 0x7A4CA4FF);//傤Ԃ
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x87\xE3\x81\x86\xE3\x81\xB6\xE8\x89\xB2", 0x7A4CA4FF);//傤ԐF
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x87\xE3\x81\x86\xE3\x81\xB6", 0x7A4CA4FF);//傤
REGISTER_COLOR_NAME("\xE8\x8F\x96\xE8\x92\xB2\xE8\x89\xB2", 0x7A4CA4FF);//ҊF
REGISTER_COLOR_NAME("\xE8\x8F\x96\xE8\x92\xB2", 0x7A4CA4FF);//Ҋ
REGISTER_COLOR_NAME("\xE6\xB1\x9F\xE6\x88\xB8\xE7\xB4\xAB", 0x654882FF);//]ˎ
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x81\xA9\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x654882FF);//ǂނ炳
REGISTER_COLOR_NAME("\xE7\xB4\xAB", 0xAC5CB5FF);//
REGISTER_COLOR_NAME("\xE7\xB4\xAB\xE8\x89\xB2", 0xAC5CB5FF);//F
REGISTER_COLOR_NAME("\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0xAC5CB5FF);//ނ炳
REGISTER_COLOR_NAME("\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D\xE8\x89\xB2", 0xAC5CB5FF);//ނ炳F
REGISTER_COLOR_NAME("\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0xAC5CB5FF);//ނ炳
REGISTER_COLOR_NAME("\xE5\x8F\xA4\xE4\xBB\xA3\xE7\xB4\xAB", 0x7B5381FF);//Ñ㎇
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\xA0\xE3\x81\x84\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x7B5381FF);//ނ炳
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\x99\xE7\xB4\xBA", 0x49364DFF);//Ȃ
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\x99\xE3\x81\x93\xE3\x82\x93", 0x49364DFF);//Ȃ
REGISTER_COLOR_NAME("\xE7\xB4\xAB\xE7\xB4\xBA", 0x422846FF);//
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x81\x93\xE3\x82\x93", 0x422846FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x82\x84\xE3\x82\x81\xE8\x89\xB2", 0xC97ABEFF);//ߐF
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x82\x84\xE3\x82\x81\xE3\x81\x84\xE3\x82\x8D", 0xC97ABEFF);//߂
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x82\x84\xE3\x82\x81", 0xC97ABEFF);//
REGISTER_COLOR_NAME("\xE7\x89\xA1\xE4\xB8\xB9\xE8\x89\xB2", 0xCB479FFF);//OF
REGISTER_COLOR_NAME("\xE7\x89\xA1\xE4\xB8\xB9", 0xCB479FFF);//O
REGISTER_COLOR_NAME("\xE3\x81\xBC\xE3\x81\x9F\xE3\x82\x93\xE3\x81\x84\xE3\x82\x8D", 0xCB479FFF);//ڂ񂢂
REGISTER_COLOR_NAME("\xE3\x81\xBC\xE3\x81\x9F\xE3\x82\x93\xE8\x89\xB2", 0xCB479FFF);//ڂF
REGISTER_COLOR_NAME("\xE3\x81\xBC\xE3\x81\x9F\xE3\x82\x93", 0xCB479FFF);//ڂ
REGISTER_COLOR_NAME("\xE8\xB5\xA4\xE7\xB4\xAB", 0xDC589AFF);//Ԏ
REGISTER_COLOR_NAME("\xE8\xB5\xA4\xE7\xB4\xAB\xE8\x89\xB2", 0xDC589AFF);//ԎF
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0xDC589AFF);//ނ炳
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D\xE8\x89\xB2", 0xDC589AFF);//ނ炳F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8B\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0xDC589AFF);//ނ炳
REGISTER_COLOR_NAME("\xE7\x99\xBD", 0xFFFFFFFF);//
REGISTER_COLOR_NAME("\xE7\x99\xBD\xE8\x89\xB2", 0xFFFFFFFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x8D\xE3\x81\x84\xE3\x82\x8D", 0xFFFFFFFF);//낢
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x8D\xE8\x89\xB2", 0xFFFFFFFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x8D", 0xFFFFFFFF);//
REGISTER_COLOR_NAME("\xE8\x83\xA1\xE7\xB2\x89\xE8\x89\xB2", 0xF2E9E8FF);//ӕF
REGISTER_COLOR_NAME("\xE8\x83\xA1\xE7\xB2\x89", 0xF2E9E8FF);//ӕ
REGISTER_COLOR_NAME("\xE3\x81\x94\xE3\x81\xB5\xE3\x82\x93\xE3\x81\x84\xE3\x82\x8D", 0xF2E9E8FF);//ӂ񂢂
REGISTER_COLOR_NAME("\xE3\x81\x94\xE3\x81\xB5\xE3\x82\x93\xE8\x89\xB2", 0xF2E9E8FF);//ӂF
REGISTER_COLOR_NAME("\xE3\x81\x94\xE3\x81\xB5\xE3\x82\x93", 0xF2E9E8FF);//ӂ
REGISTER_COLOR_NAME("\xE7\x94\x9F\xE6\x88\x90\xE3\x82\x8A\xE8\x89\xB2", 0xF1E2DEFF);//F
REGISTER_COLOR_NAME("\xE7\x94\x9F\xE6\x88\x90\xE3\x82\x8A", 0xF1E2DEFF);//
REGISTER_COLOR_NAME("\xE7\x94\x9F\xE6\x88\x90", 0xF1E2DEFF);//
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xAA\xE3\x82\x8A\xE3\x81\x84\xE3\x82\x8D", 0xF1E2DEFF);//Ȃ肢
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xAA\xE3\x82\x8A\xE8\x89\xB2", 0xF1E2DEFF);//ȂF
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xAA\xE3\x82\x8A", 0xF1E2DEFF);//Ȃ
REGISTER_COLOR_NAME("\xE8\xB1\xA1\xE7\x89\x99\xE8\x89\xB2", 0xE5D5C9FF);//ۉF
REGISTER_COLOR_NAME("\xE8\xB1\xA1\xE7\x89\x99", 0xE5D5C9FF);//ۉ
REGISTER_COLOR_NAME("\xE3\x81\x9E\xE3\x81\x86\xE3\x81\x92\xE3\x81\x84\xE3\x82\x8D", 0xE5D5C9FF);//
REGISTER_COLOR_NAME("\xE3\x81\x9E\xE3\x81\x86\xE3\x81\x92\xE8\x89\xB2", 0xE5D5C9FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x9E\xE3\x81\x86\xE3\x81\x92", 0xE5D5C9FF);//
REGISTER_COLOR_NAME("\xE9\x8A\x80\xE9\xBC\xA0", 0xA6A6A6FF);//l
REGISTER_COLOR_NAME("\xE3\x81\x8E\xE3\x82\x93\xE3\x81\xAD\xE3\x81\x9A", 0xA6A6A6FF);//˂
REGISTER_COLOR_NAME("\xE8\x8C\xB6\xE9\xBC\xA0", 0xA19190FF);//l
REGISTER_COLOR_NAME("\xE3\x81\xA1\xE3\x82\x83\xE3\x81\xAD\xE3\x81\x9A", 0xA19190FF);//˂
REGISTER_COLOR_NAME("\xE9\xBC\xA0\xE8\x89\xB2", 0x8C8C8CFF);//lF
REGISTER_COLOR_NAME("\xE3\x81\xAD\xE3\x81\x9A\xE3\x81\xBF\xE3\x81\x84\xE3\x82\x8D", 0x8C8C8CFF);//˂݂
REGISTER_COLOR_NAME("\xE3\x81\xAD\xE3\x81\x9A\xE3\x81\xBF\xE8\x89\xB2", 0x8C8C8CFF);//˂ݐF
REGISTER_COLOR_NAME("\xE3\x81\xAD\xE3\x81\x9A\xE3\x81\xBF", 0x8C8C8CFF);//˂
REGISTER_COLOR_NAME("\xE5\x88\xA9\xE4\xBC\x91\xE9\xBC\xA0", 0x767C7AFF);//xl
REGISTER_COLOR_NAME("\xE3\x82\x8A\xE3\x81\x8D\xE3\x82\x85\xE3\x81\x86\xE3\x81\xAD\xE3\x81\x9A", 0x767C7AFF);//肫イ˂
REGISTER_COLOR_NAME("\xE9\x89\x9B\xE8\x89\xB2", 0x797A85FF);//F
REGISTER_COLOR_NAME("\xE9\x89\x9B", 0x797A85FF);//
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\xBE\xE3\x82\x8A\xE3\x81\x84\xE3\x82\x8D", 0x797A85FF);//Ȃ܂肢
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\xBE\xE3\x82\x8A\xE8\x89\xB2", 0x797A85FF);//Ȃ܂F
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\xBE\xE3\x82\x8A", 0x797A85FF);//Ȃ܂
REGISTER_COLOR_NAME("\xE7\x81\xB0\xE8\x89\xB2", 0x808080FF);//DF
REGISTER_COLOR_NAME("\xE7\x81\xB0", 0x808080FF);//D
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x84\xE3\x81\x84\xE3\x82\x8D", 0x808080FF);//͂
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x84\xE8\x89\xB2", 0x808080FF);//͂F
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x84", 0x808080FF);//͂
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\x99\xE7\xAB\xB9\xE8\x89\xB2", 0x615149FF);//|F
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\x99\xE7\xAB\xB9", 0x615149FF);//|
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\x99\xE3\x81\x9F\xE3\x81\x91\xE3\x81\x84\xE3\x82\x8D", 0x615149FF);//
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\x99\xE3\x81\x9F\xE3\x81\x91\xE8\x89\xB2", 0x615149FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\x99\xE3\x81\x9F\xE3\x81\x91", 0x615149FF);//
REGISTER_COLOR_NAME("\xE9\xBB\x92\xE8\x8C\xB6", 0x3F2D2CFF);//
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x82\x8D\xE3\x81\xA1\xE3\x82\x83", 0x3F2D2CFF);//낿
REGISTER_COLOR_NAME("\xE5\xA2\xA8", 0x333333FF);//n
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\xBF", 0x333333FF);//
REGISTER_COLOR_NAME("\xE9\xBB\x92", 0x000000FF);//
REGISTER_COLOR_NAME("\xE9\xBB\x92\xE8\x89\xB2", 0x000000FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x82\x8D\xE3\x81\x84\xE3\x82\x8D", 0x000000FF);//낢
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x82\x8D\xE8\x89\xB2", 0x000000FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x82\x8D", 0x000000FF);//
REGISTER_COLOR_NAME("\xE9\x89\x84\xE9\xBB\x92", 0x1C1C1CFF);//S
REGISTER_COLOR_NAME("\xE3\x81\xA6\xE3\x81\xA4\xE3\x81\x90\xE3\x82\x8D", 0x1C1C1CFF);//Ă
REGISTER_COLOR_NAME("\xE3\x83\xAD\xE3\x83\xBC\xE3\x82\xBA\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0xCE4C7FFF);//[Ybh
REGISTER_COLOR_NAME("\xE3\x83\xAD\xE3\x83\xBC\xE3\x82\xBA\xE3\x83\x94\xE3\x83\xB3\xE3\x82\xAF", 0xF095AAFF);//[YsN
REGISTER_COLOR_NAME("\xE3\x82\xB3\xE3\x83\x81\xE3\x83\x8B\xE3\x83\xBC\xE3\x83\xAB\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0xB22C5BFF);//R`j[bh
REGISTER_COLOR_NAME("\xE3\x83\xAB\xE3\x83\x93\xE3\x83\xBC\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0xBD145AFF);//r[bh
REGISTER_COLOR_NAME("\xE3\x83\xAF\xE3\x82\xA4\xE3\x83\xB3\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0x861E46FF);//Cbh
REGISTER_COLOR_NAME("\xE3\x83\x90\xE3\x83\xBC\xE3\x82\xAC\xE3\x83\xB3\xE3\x83\x87\xE3\x82\xA3\xE3\x83\xBC", 0x452935FF);//o[KfB[
REGISTER_COLOR_NAME("\xE3\x82\xAA\xE3\x83\xBC\xE3\x83\xAB\xE3\x83\x89\xE3\x83\xAD\xE3\x83\xBC\xE3\x82\xBA", 0xCA808FFF);//I[h[Y
REGISTER_COLOR_NAME("\xE3\x83\xAD\xE3\x83\xBC\xE3\x82\xBA", 0xDC3F6AFF);//[Y
REGISTER_COLOR_NAME("\xE3\x82\xB9\xE3\x83\x88\xE3\x83\xAD\xE3\x83\x99\xE3\x83\xAA\xE3\x83\xBC", 0xBD1354FF);//Xgx[
REGISTER_COLOR_NAME("\xE3\x82\xB3\xE3\x83\xBC\xE3\x83\xA9\xE3\x83\xAB\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0xFF8899FF);//R[bh
REGISTER_COLOR_NAME("\xE3\x83\x94\xE3\x83\xB3\xE3\x82\xAF", 0xEB99A4FF);//sN
REGISTER_COLOR_NAME("\xE3\x83\x9C\xE3\x83\xAB\xE3\x83\x89\xE3\x83\xBC", 0x56333CFF);//{h[
REGISTER_COLOR_NAME("\xE3\x83\x99\xE3\x83\x93\xE3\x83\xBC\xE3\x83\x94\xE3\x83\xB3\xE3\x82\xAF", 0xFFCAD3FF);//xr[sN
REGISTER_COLOR_NAME("\xE3\x83\x9D\xE3\x83\x94\xE3\x83\xBC\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0xDE3F56FF);//|s[bh
REGISTER_COLOR_NAME("\xE3\x82\xB7\xE3\x82\xB0\xE3\x83\x8A\xE3\x83\xAB\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0xCE2A4BFF);//VOibh
REGISTER_COLOR_NAME("\xE3\x82\xAB\xE3\x83\xBC\xE3\x83\x9E\xE3\x82\xA4\xE3\x83\xB3", 0xBF163FFF);//J[}C
REGISTER_COLOR_NAME("\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0xDE404FFF);//bh
REGISTER_COLOR_NAME("\xE3\x83\x88\xE3\x83\x9E\xE3\x83\x88\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0xDE404FFF);//g}gbh
REGISTER_COLOR_NAME("\xE3\x83\x9E\xE3\x83\xAB\xE3\x83\xBC\xE3\x83\xB3", 0x682832FF);//}[
REGISTER_COLOR_NAME("\xE3\x83\x90\xE3\x83\xBC\xE3\x83\x9F\xE3\x83\xAA\xE3\x82\xAA\xE3\x83\xB3", 0xED5153FF);//o[~I
REGISTER_COLOR_NAME("\xE3\x82\xB9\xE3\x82\xAB\xE3\x83\xBC\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x88", 0xDE423EFF);//XJ[bg
REGISTER_COLOR_NAME("\xE3\x83\x86\xE3\x83\xA9\xE3\x82\xB3\xE3\x83\x83\xE3\x82\xBF", 0xAD534DFF);//eRb^
REGISTER_COLOR_NAME("\xE3\x82\xB5\xE3\x83\xBC\xE3\x83\xA2\xE3\x83\xB3\xE3\x83\x94\xE3\x83\xB3\xE3\x82\xAF", 0xFFA59AFF);//T[sN
REGISTER_COLOR_NAME("\xE3\x82\xB7\xE3\x82\xA7\xE3\x83\xAB\xE3\x83\x94\xE3\x83\xB3\xE3\x82\xAF", 0xFDCCC9FF);//VFsN
REGISTER_COLOR_NAME("\xE3\x83\x8D\xE3\x83\xBC\xE3\x83\xAB\xE3\x83\x94\xE3\x83\xB3\xE3\x82\xAF", 0xF3BDB8FF);//l[sN
REGISTER_COLOR_NAME("\xE3\x83\x81\xE3\x83\xA3\xE3\x82\xA4\xE3\x83\x8B\xE3\x83\xBC\xE3\x82\xBA\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x89", 0xFD6328FF);//`Cj[Ybh
REGISTER_COLOR_NAME("\xE3\x82\xAD\xE3\x83\xA3\xE3\x83\xAD\xE3\x83\x83\xE3\x83\x88\xE3\x82\xAA\xE3\x83\xAC\xE3\x83\xB3\xE3\x82\xB8", 0xCB5735FF);//LbgIW
REGISTER_COLOR_NAME("\xE3\x83\x90\xE3\x83\xBC\xE3\x83\xB3\xE3\x83\x88\xE3\x82\xB7\xE3\x82\xA7\xE3\x83\xB3\xE3\x83\x8A", 0xA95645FF);//o[gVFi
REGISTER_COLOR_NAME("\xE3\x83\x81\xE3\x83\xA7\xE3\x82\xB3\xE3\x83\xAC\xE3\x83\xBC\xE3\x83\x88", 0x533534FF);//`R[g
REGISTER_COLOR_NAME("\xE3\x82\xB3\xE3\x82\xB3\xE3\x82\xA2\xE3\x83\x96\xE3\x83\xA9\xE3\x82\xA6\xE3\x83\xB3", 0x77493EFF);//RRAuE
REGISTER_COLOR_NAME("\xE3\x83\x94\xE3\x83\xBC\xE3\x83\x81", 0xEDC0B5FF);//s[`
REGISTER_COLOR_NAME("\xE3\x83\xAD\xE3\x83\xBC\xE3\x82\xB7\xE3\x82\xA7\xE3\x83\xB3\xE3\x83\x8A", 0xB96530FF);//[VFi
REGISTER_COLOR_NAME("\xE3\x82\xAA\xE3\x83\xAC\xE3\x83\xB3\xE3\x82\xB8", 0xF38612FF);//IW
REGISTER_COLOR_NAME("\xE3\x83\x96\xE3\x83\xA9\xE3\x82\xA6\xE3\x83\xB3", 0x744B38FF);//uE
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x83\x97\xE3\x83\xAA\xE3\x82\xB3\xE3\x83\x83\xE3\x83\x88", 0xDFA27CFF);//AvRbg
REGISTER_COLOR_NAME("\xE3\x82\xBF\xE3\x83\xB3", 0xA76E48FF);//^
REGISTER_COLOR_NAME("\xE3\x83\x9E\xE3\x83\xB3\xE3\x83\x80\xE3\x83\xAA\xE3\x83\xB3\xE3\x82\xAA\xE3\x83\xAC\xE3\x83\xB3\xE3\x82\xB8", 0xF69A32FF);//}_IW
REGISTER_COLOR_NAME("\xE3\x82\xB3\xE3\x83\xAB\xE3\x82\xAF", 0xA87F67FF);//RN
REGISTER_COLOR_NAME("\xE3\x82\xA8\xE3\x82\xAF\xE3\x83\xAB\xE3\x83\x99\xE3\x82\xA4\xE3\x82\xB8\xE3\x83\xA5", 0xFCCFB3FF);//GNxCW
REGISTER_COLOR_NAME("\xE3\x82\xA8\xE3\x82\xAF\xE3\x83\xAB\xE3\x83\x99\xE3\x83\xBC\xE3\x82\xB8\xE3\x83\xA5", 0xFCCFB3FF);//GNx[W
REGISTER_COLOR_NAME("\xE3\x82\xB4\xE3\x83\xBC\xE3\x83\xAB\xE3\x83\x87\xE3\x83\xB3\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC", 0xEF9E48FF);//S[fCG[
REGISTER_COLOR_NAME("\xE3\x83\x9E\xE3\x83\xAA\xE3\x83\xBC\xE3\x82\xB4\xE3\x83\xBC\xE3\x83\xAB\xE3\x83\x89", 0xFFA80EFF);//}[S[h
REGISTER_COLOR_NAME("\xE3\x83\x90\xE3\x83\x95", 0xC89874FF);//ot
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x83\xB3\xE3\x83\x90\xE3\x83\xBC", 0xB47C49FF);//Ao[
REGISTER_COLOR_NAME("\xE3\x83\x96\xE3\x83\xAD\xE3\x83\xB3\xE3\x82\xBA", 0x825833FF);//uY
REGISTER_COLOR_NAME("\xE3\x83\x99\xE3\x83\xBC\xE3\x82\xB8\xE3\x83\xA5", 0xC4AB99FF);//x[W
REGISTER_COLOR_NAME("\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC\xE3\x82\xAA\xE3\x83\xBC\xE3\x82\xAB\xE3\x83\xBC", 0xC18A42FF);//CG[I[J[
REGISTER_COLOR_NAME("\xE3\x83\x90\xE3\x83\xBC\xE3\x83\xB3\xE3\x83\x88\xE3\x82\xA2\xE3\x83\xB3\xE3\x83\x90\xE3\x83\xBC", 0x5B442FFF);//o[gAo[
REGISTER_COLOR_NAME("\xE3\x82\xBB\xE3\x83\x94\xE3\x82\xA2", 0x4A3A2EFF);//ZsA
REGISTER_COLOR_NAME("\xE3\x82\xAB\xE3\x83\xBC\xE3\x82\xAD", 0x9B743FFF);//J[L
REGISTER_COLOR_NAME("\xE3\x82\xAB\xE3\x83\xBC\xE3\x82\xAD\xE3\x83\xBC", 0x9B743FFF);//J[L[
REGISTER_COLOR_NAME("\xE3\x83\x96\xE3\x83\xAD\xE3\x83\xB3\xE3\x83\x89", 0xE3B56AFF);//uh
REGISTER_COLOR_NAME("\xE3\x83\x8D\xE3\x83\xBC\xE3\x83\x97\xE3\x83\xAB\xE3\x82\xB9\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC", 0xF2C36FFF);//l[vXCG[
REGISTER_COLOR_NAME("\xE3\x83\xAC\xE3\x82\xB0\xE3\x83\x9B\xE3\x83\xBC\xE3\x83\xB3", 0xE3C69DFF);//Oz[
REGISTER_COLOR_NAME("\xE3\x83\xAD\xE3\x83\xBC\xE3\x82\xA2\xE3\x83\xB3\xE3\x83\x90\xE3\x83\xBC", 0x7C5C1EFF);//[Ao[
REGISTER_COLOR_NAME("\xE3\x82\xAF\xE3\x83\xAD\xE3\x83\xA0\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC", 0xFCC115FF);//NCG[
REGISTER_COLOR_NAME("\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC", 0xFFD400FF);//CG[
REGISTER_COLOR_NAME("\xE3\x82\xAF\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xA0\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC", 0xEAD6ADFF);//N[CG[
REGISTER_COLOR_NAME("\xE3\x82\xB8\xE3\x83\xA7\xE3\x83\xB3\xE3\x83\x96\xE3\x83\xAA\xE3\x82\xA2\xE3\x83\xB3", 0xFFD400FF);//WuA
REGISTER_COLOR_NAME("\xE3\x82\xAB\xE3\x83\x8A\xE3\x83\xAA\xE3\x83\xBC\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC", 0xF3D749FF);//Ji[CG[
REGISTER_COLOR_NAME("\xE3\x82\xAA\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\x96\xE3\x83\x89\xE3\x83\xA9\xE3\x83\x96", 0x69604DFF);//I[uhu
REGISTER_COLOR_NAME("\xE3\x82\xAA\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\x96", 0x60542AFF);//I[u
REGISTER_COLOR_NAME("\xE3\x83\xAC\xE3\x83\xA2\xE3\x83\xB3\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC", 0xE4CB00FF);//CG[
REGISTER_COLOR_NAME("\xE3\x82\xAA\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\x96\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x555739FF);//I[uO[
REGISTER_COLOR_NAME("\xE3\x82\xB7\xE3\x83\xA3\xE3\x83\x88\xE3\x83\xAB\xE3\x83\xBC\xE3\x82\xBA\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0xC3D447FF);//Vg[YO[
REGISTER_COLOR_NAME("\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\x95\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x8B9D46FF);//[tO[
REGISTER_COLOR_NAME("\xE3\x82\xB0\xE3\x83\xA9\xE3\x82\xB9\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x748049FF);//OXO[
REGISTER_COLOR_NAME("\xE3\x82\xB7\xE3\x83\xBC\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x9DBA5BFF);//V[O[
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x82\xA4\xE3\x83\x93\xE3\x83\xBC\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x4E693BFF);//ACr[O[
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x83\x83\xE3\x83\x97\xE3\x83\xAB\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0xAED4A7FF);//AbvO[
REGISTER_COLOR_NAME("\xE3\x83\x9F\xE3\x83\xB3\xE3\x83\x88\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x78CF98FF);//~gO[
REGISTER_COLOR_NAME("\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x029C5EFF);//O[
REGISTER_COLOR_NAME("\xE3\x82\xB3\xE3\x83\x90\xE3\x83\xAB\xE3\x83\x88\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x51C391FF);//RogO[
REGISTER_COLOR_NAME("\xE3\x82\xA8\xE3\x83\xA1\xE3\x83\xA9\xE3\x83\xAB\xE3\x83\x89\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x44A67BFF);//GhO[
REGISTER_COLOR_NAME("\xE3\x83\x9E\xE3\x83\xA9\xE3\x82\xAB\xE3\x82\xA4\xE3\x83\x88\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x008054FF);//}JCgO[
REGISTER_COLOR_NAME("\xE3\x83\x9C\xE3\x83\x88\xE3\x83\xAB\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x284239FF);//{gO[
REGISTER_COLOR_NAME("\xE3\x83\x95\xE3\x82\xA9\xE3\x83\xAC\xE3\x82\xB9\xE3\x83\x88\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x3A7969FF);//tHXgO[
REGISTER_COLOR_NAME("\xE3\x83\x93\xE3\x83\xAA\xE3\x82\xB8\xE3\x82\xA2\xE3\x83\xB3", 0x146E5CFF);//rWA
REGISTER_COLOR_NAME("\xE3\x83\x93\xE3\x83\xAA\xE3\x83\xA4\xE3\x83\xBC\xE3\x83\x89\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x00463DFF);//r[hO[
REGISTER_COLOR_NAME("\xE3\x83\x94\xE3\x83\xBC\xE3\x82\xB3\xE3\x83\x83\xE3\x82\xAF\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0x007F8BFF);//s[RbNO[
REGISTER_COLOR_NAME("\xE3\x83\x8A\xE3\x82\xA4\xE3\x83\xAB\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x4A92A0FF);//iCu[
REGISTER_COLOR_NAME("\xE3\x83\x94\xE3\x83\xBC\xE3\x82\xB3\xE3\x83\x83\xE3\x82\xAF\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x007087FF);//s[RbNu[
REGISTER_COLOR_NAME("\xE3\x82\xBF\xE3\x83\xBC\xE3\x82\xB3\xE3\x82\xA4\xE3\x82\xBA\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x19A1CAFF);//^[RCYu[
REGISTER_COLOR_NAME("\xE3\x83\x9E\xE3\x83\xAA\xE3\x83\xB3\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x005175FF);//}u[
REGISTER_COLOR_NAME("\xE3\x83\x9B\xE3\x83\xAA\xE3\x82\xBE\xE3\x83\xB3\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x90B4CFFF);//z]u[
REGISTER_COLOR_NAME("\xE3\x82\xB7\xE3\x82\xA2\xE3\x83\xB3", 0x00A1DCFF);//VA
REGISTER_COLOR_NAME("\xE3\x82\xB9\xE3\x82\xAB\xE3\x82\xA4\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x90C3E9FF);//XJCu[
REGISTER_COLOR_NAME("\xE3\x82\xBB\xE3\x83\xAB\xE3\x83\xAA\xE3\x82\xA2\xE3\x83\xB3\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x0076AEFF);//ZAu[
REGISTER_COLOR_NAME("\xE3\x83\x99\xE3\x83\x93\xE3\x83\xBC\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0xACBED7FF);//xr[u[
REGISTER_COLOR_NAME("\xE3\x82\xB5\xE3\x83\x83\xE3\x82\xAF\xE3\x82\xB9\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x607D9EFF);//TbNXu[
REGISTER_COLOR_NAME("\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x0072B9FF);//u[
REGISTER_COLOR_NAME("\xE3\x82\xB3\xE3\x83\x90\xE3\x83\xAB\xE3\x83\x88\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x0064ABFF);//Rogu[
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x82\xA4\xE3\x82\xA2\xE3\x83\xB3\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x3A486AFF);//ACAu[
REGISTER_COLOR_NAME("\xE3\x83\x97\xE3\x83\xAB\xE3\x82\xB7\xE3\x83\xA3\xE3\x83\xB3\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x3A486AFF);//vVu[
REGISTER_COLOR_NAME("\xE3\x83\x97\xE3\x83\xAB\xE3\x82\xB7\xE3\x82\xA2\xE3\x83\xB3\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x3A486AFF);//vVAu[
REGISTER_COLOR_NAME("\xE3\x83\x9F\xE3\x83\x83\xE3\x83\x89\xE3\x83\x8A\xE3\x82\xA4\xE3\x83\x88\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x222738FF);//~bhiCgu[
REGISTER_COLOR_NAME("\xE3\x83\x92\xE3\x83\xA4\xE3\x82\xB7\xE3\x83\xB3\xE3\x82\xB9", 0x7587B8FF);//qVX
REGISTER_COLOR_NAME("\xE3\x83\x8D\xE3\x83\xBC\xE3\x83\x93\xE3\x83\xBC\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x333B5DFF);//l[r[u[
REGISTER_COLOR_NAME("\xE3\x83\x8D\xE3\x82\xA4\xE3\x83\x93\xE3\x83\xBC\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x333B5DFF);//lCr[u[
REGISTER_COLOR_NAME("\xE3\x82\xA6\xE3\x83\xAB\xE3\x83\x88\xE3\x83\xA9\xE3\x83\x9E\xE3\x83\xAA\xE3\x83\xB3\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x424CA4FF);//Eg}u[
REGISTER_COLOR_NAME("\xE3\x82\xA6\xE3\x83\xAB\xE3\x83\x88\xE3\x83\xA9\xE3\x83\x9E\xE3\x83\xAA\xE3\x83\xB3", 0x424CA4FF);//Eg}
REGISTER_COLOR_NAME("\xE3\x82\xAA\xE3\x83\xAA\xE3\x82\xA8\xE3\x83\xB3\xE3\x82\xBF\xE3\x83\xAB\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x384091FF);//IG^u[
REGISTER_COLOR_NAME("\xE3\x82\xA6\xE3\x82\xA3\xE3\x82\xB9\xE3\x82\xBF\xE3\x83\xAA\xE3\x82\xA2", 0x7F6CCFFF);//EBX^A
REGISTER_COLOR_NAME("\xE3\x83\x91\xE3\x83\xB3\xE3\x82\xB8\xE3\x83\xBC", 0x4B2B7BFF);//pW[
REGISTER_COLOR_NAME("\xE3\x83\x98\xE3\x83\xAA\xE3\x82\xAA\xE3\x83\x88\xE3\x83\xAD\xE3\x83\xBC\xE3\x83\x97", 0x8C6AC0FF);//wIg[v
REGISTER_COLOR_NAME("\xE3\x83\x90\xE3\x82\xA4\xE3\x82\xAA\xE3\x83\xAC\xE3\x83\x83\xE3\x83\x88", 0x774DA5FF);//oCIbg
REGISTER_COLOR_NAME("\xE3\x83\xA9\xE3\x83\x99\xE3\x83\xB3\xE3\x83\x80\xE3\x83\xBC", 0xA08FACFF);//x_[
REGISTER_COLOR_NAME("\xE3\x83\xA2\xE3\x83\xBC\xE3\x83\x96", 0x895CA3FF);//[u
REGISTER_COLOR_NAME("\xE3\x83\xA9\xE3\x82\xA4\xE3\x83\xA9\xE3\x83\x83\xE3\x82\xAF", 0xC6A3D5FF);//CbN
REGISTER_COLOR_NAME("\xE3\x82\xAA\xE3\x83\xBC\xE3\x82\xAD\xE3\x83\x83\xE3\x83\x89", 0xCCA2D0FF);//I[Lbh
REGISTER_COLOR_NAME("\xE3\x83\x91\xE3\x83\xBC\xE3\x83\x97\xE3\x83\xAB", 0xAC5CB5FF);//p[v
REGISTER_COLOR_NAME("\xE3\x83\x9E\xE3\x82\xBC\xE3\x83\xB3\xE3\x82\xBF", 0xD3438FFF);//}[^
REGISTER_COLOR_NAME("\xE3\x83\x81\xE3\x82\xA7\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\x94\xE3\x83\xB3\xE3\x82\xAF", 0xD55F93FF);//`F[sN
REGISTER_COLOR_NAME("\xE3\x83\x9B\xE3\x83\xAF\xE3\x82\xA4\xE3\x83\x88", 0xFFFFFFFF);//zCg
REGISTER_COLOR_NAME("\xE3\x82\xB9\xE3\x83\x8E\xE3\x83\xBC\xE3\x83\x9B\xE3\x83\xAF\xE3\x82\xA4\xE3\x83\x88", 0xFAFAFAFF);//Xm[zCg
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x82\xA4\xE3\x83\x9C\xE3\x83\xAA\xE3\x83\xBC", 0xE5D5C9FF);//AC{[
REGISTER_COLOR_NAME("\xE3\x82\xB9\xE3\x82\xAB\xE3\x82\xA4\xE3\x82\xB0\xE3\x83\xAC\xE3\x82\xA4", 0xBCBCC3FF);//XJCOC
REGISTER_COLOR_NAME("\xE3\x83\x91\xE3\x83\xBC\xE3\x83\xAB\xE3\x82\xB0\xE3\x83\xAC\xE3\x82\xA4", 0xB3B3B3FF);//p[OC
REGISTER_COLOR_NAME("\xE3\x82\xB7\xE3\x83\xAB\xE3\x83\x90\xE3\x83\xBC\xE3\x82\xB0\xE3\x83\xAC\xE3\x82\xA4", 0xA6A6A6FF);//Vo[OC
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x83\x83\xE3\x82\xB7\xE3\x83\xA5\xE3\x82\xB0\xE3\x83\xAC\xE3\x82\xA4", 0x999999FF);//AbVOC
REGISTER_COLOR_NAME("\xE3\x83\xAD\xE3\x83\xBC\xE3\x82\xBA\xE3\x82\xB0\xE3\x83\xAC\xE3\x82\xA4", 0x93848AFF);//[YOC
REGISTER_COLOR_NAME("\xE3\x82\xB0\xE3\x83\xAC\xE3\x82\xA4", 0x808080FF);//OC
REGISTER_COLOR_NAME("\xE3\x82\xB9\xE3\x83\x81\xE3\x83\xBC\xE3\x83\xAB\xE3\x82\xB0\xE3\x83\xAC\xE3\x82\xA4", 0x736B77FF);//X`[OC
REGISTER_COLOR_NAME("\xE3\x82\xB9\xE3\x83\xAC\xE3\x83\xBC\xE3\x83\x88\xE3\x82\xB0\xE3\x83\xAC\xE3\x82\xA4", 0x55545BFF);//X[gOC
REGISTER_COLOR_NAME("\xE3\x83\x81\xE3\x83\xA3\xE3\x82\xB3\xE3\x83\xBC\xE3\x83\xAB\xE3\x82\xB0\xE3\x83\xAC\xE3\x82\xA4", 0x4E4552FF);//`R[OC
REGISTER_COLOR_NAME("\xE3\x83\xA9\xE3\x83\xB3\xE3\x83\x97\xE3\x83\x96\xE3\x83\xA9\xE3\x83\x83\xE3\x82\xAF", 0x1C1C1CFF);//vubN
REGISTER_COLOR_NAME("\xE3\x83\x96\xE3\x83\xA9\xE3\x83\x83\xE3\x82\xAF", 0x000000FF);//ubN
//Japanese color names that are not defined by JIS Z 8102
REGISTER_COLOR_NAME("\xE8\x97\x8D\xE9\x8C\x86\xE8\x89\xB2", 0x475189FF);//KF
REGISTER_COLOR_NAME("\xE8\x97\x8D\xE9\x8C\x86", 0x475189FF);//K
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84\xE3\x81\x95\xE3\x81\xB3\xE8\x89\xB2", 0x475189FF);//ѐF
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84\xE3\x81\x95\xE3\x81\xB3\xE3\x81\x84\xE3\x82\x8D", 0x475189FF);//т
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84\xE3\x81\x95\xE3\x81\xB3", 0x475189FF);//
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x82\xA4\xE3\x82\xB9\xE3\x82\xB0\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xB3", 0xA3D6CCFF);//ACXO[
REGISTER_COLOR_NAME("\xE8\x97\x8D\xE7\x99\xBD\xE8\x89\xB2", 0xEBF6F7FF);//F
REGISTER_COLOR_NAME("\xE8\x97\x8D\xE7\x99\xBD", 0xEBF6F7FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84\xE3\x81\x98\xE3\x82\x8D\xE8\x89\xB2", 0xEBF6F7FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84\xE3\x81\x98\xE3\x82\x8D\xE3\x81\x84\xE3\x82\x8D", 0xEBF6F7FF);//낢
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x84\xE3\x81\x98\xE3\x82\x8D", 0xEBF6F7FF);//
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x82\xAF\xE3\x82\xA2\xE3\x83\x9E\xE3\x83\xAA\xE3\x83\xB3", 0x006F86FF);//ANA}
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x82\xB8\xE3\x83\xA5\xE3\x83\xBC\xE3\x83\xAB", 0x007FFFFF);//AW[
REGISTER_COLOR_NAME("\xE7\xB7\x8B\xE8\x89\xB2", 0xE54848FF);//F
REGISTER_COLOR_NAME("\xE7\xB7\x8B", 0xE54848FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x91\xE3\x81\x84\xE3\x82\x8D", 0xE54848FF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x91\xE8\x89\xB2", 0xE54848FF);//F
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x81\x84\xE3\x82\x8D", 0xE54848FF);//Ђ
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE8\x89\xB2", 0xE54848FF);//АF
REGISTER_COLOR_NAME("\xE6\xB7\xB1\xE7\xB7\x8B", 0xB2243CFF);//[
REGISTER_COLOR_NAME("\xE3\x81\xB5\xE3\x81\x8B\xE3\x81\xB2", 0xB2243CFF);//ӂ
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x8D\xE3\x81\xB2", 0xB2243CFF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x8D\xE3\x81\x82\xE3\x81\x91", 0xB2243CFF);//
REGISTER_COLOR_NAME("\xE6\x9B\x99\xE8\x89\xB2", 0xF19072FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x91\xE3\x81\xBC\xE3\x81\xAE\xE3\x81\x84\xE3\x82\x8D", 0xF19072FF);//ڂ̂
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x91\xE3\x81\xBC\xE3\x81\xAE\xE8\x89\xB2", 0xF19072FF);//ڂ̐F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x91\xE3\x81\xBC\xE3\x81\xAE", 0xF19072FF);//ڂ
REGISTER_COLOR_NAME("\xE6\xB5\x85\xE9\xBB\x84\xE8\x89\xB2", 0xFFCC33FF);//󉩐F
REGISTER_COLOR_NAME("\xE6\xB5\x85\xE9\xBB\x84", 0xFFCC33FF);//
REGISTER_COLOR_NAME("\xE4\xBA\x9C\xE9\xBA\xBB\xE8\x89\xB2", 0xD6C6AFFF);//F
REGISTER_COLOR_NAME("\xE4\xBA\x9C\xE9\xBA\xBB", 0xD6C6AFFF);//
REGISTER_COLOR_NAME("\xE6\xB5\x85\xE7\xB4\xAB", 0xC4A3BFFF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x95\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0xC4A3BFFF);//ނ炳
REGISTER_COLOR_NAME("\xE3\x82\xA2\xE3\x83\xAA\xE3\x82\xB9\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0xF0F8FFFF);//AXu[
REGISTER_COLOR_NAME("\xE6\x9A\x97\xE9\xBB\x92\xE8\x89\xB2", 0x00001CFF);//ÍF
REGISTER_COLOR_NAME("\xE6\x9A\x97\xE9\xBB\x92", 0x00001CFF);//Í
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x82\x93\xE3\x81\x93\xE3\x81\x8F\xE3\x81\x84\xE3\x82\x8D", 0x00001CFF);//񂱂
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x82\x93\xE3\x81\x93\xE3\x81\x8F\xE8\x89\xB2", 0x00001CFF);//񂱂F
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x82\x93\xE3\x81\x93\xE3\x81\x8F", 0x00001CFF);//񂱂
REGISTER_COLOR_NAME("\xE4\xB8\x80\xE6\x96\xA4\xE6\x9F\x93", 0xFFD3E4FF);//Ґ
REGISTER_COLOR_NAME("\xE4\xBB\x8A\xE6\xA7\x98\xE8\x89\xB2", 0xD0576BFF);//lF
REGISTER_COLOR_NAME("\xE4\xBB\x8A\xE6\xA7\x98", 0xD0576BFF);//l
REGISTER_COLOR_NAME("\xE3\x82\xA4\xE3\x83\xB3\xE3\x83\x87\xE3\x82\xA3\xE3\x82\xA2\xE3\x83\xB3\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC", 0xE3A857FF);//CfBACG[
REGISTER_COLOR_NAME("\xE3\x82\xA4\xE3\x83\xB3\xE3\x83\x87\xE3\x82\xA3\xE3\x82\xB4", 0x234794FF);//CfBS
REGISTER_COLOR_NAME("\xE8\x96\x84\xE9\x9D\x92\xE7\xB7\x91", 0x319DA6FF);//
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x81\x82\xE3\x81\x8A\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A", 0x319DA6FF);//݂ǂ
REGISTER_COLOR_NAME("\xE8\x96\x84\xE9\x9D\x92", 0x6AA0D5FF);//
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x81\x82\xE3\x81\x8A", 0x6AA0D5FF);//
REGISTER_COLOR_NAME("\xE8\x96\x84\xE7\xBE\xA4\xE9\x9D\x92\xE8\x89\xB2", 0x5E83BCFF);//QF
REGISTER_COLOR_NAME("\xE8\x96\x84\xE7\xBE\xA4\xE9\x9D\x92", 0x5E83BCFF);//Q
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x81\x90\xE3\x82\x93\xE3\x81\x98\xE3\x82\x87\xE3\x81\x86\xE3\x81\x84\xE3\x82\x8D", 0x5E83BCFF);//񂶂傤
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x81\x90\xE3\x82\x93\xE3\x81\x98\xE3\x82\x87\xE3\x81\x86\xE8\x89\xB2", 0x5E83BCFF);//񂶂傤F
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x81\x90\xE3\x82\x93\xE3\x81\x98\xE3\x82\x87\xE3\x81\x86", 0x5E83BCFF);//񂶂傤
REGISTER_COLOR_NAME("\xE8\x96\x84\xE7\xB4\x85\xE8\x89\xB2", 0xC56770FF);//gF
REGISTER_COLOR_NAME("\xE8\x96\x84\xE7\xB4\x85", 0xC56770FF);//g
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x81\xB9\xE3\x81\xAB\xE3\x81\x84\xE3\x82\x8D", 0xC56770FF);//ׂɂ
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x81\xB9\xE3\x81\xAB\xE8\x89\xB2", 0xC56770FF);//ׂɐF
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x81\xB9\xE3\x81\xAB", 0xC56770FF);//ׂ
REGISTER_COLOR_NAME("\xE8\x96\x84\xE7\xB4\xAB", 0x9D759CFF);//
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x9D759CFF);//ނ炳
REGISTER_COLOR_NAME("\xE7\xA9\xBA\xE4\xBA\x94\xE5\x80\x8D\xE5\xAD\x90\xE8\x89\xB2", 0x9D896CFF);//ܔ{qF
REGISTER_COLOR_NAME("\xE7\xA9\xBA\xE4\xBA\x94\xE5\x80\x8D\xE5\xAD\x90", 0x9D896CFF);//ܔ{q
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\xA4\xE3\x81\xB6\xE3\x81\x97\xE3\x81\x84\xE3\x82\x8D", 0x9D896CFF);//Ԃ
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\xA4\xE3\x81\xB6\xE3\x81\x97\xE8\x89\xB2", 0x9D896CFF);//ԂF
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\xA4\xE3\x81\xB6\xE3\x81\x97", 0x9D896CFF);//Ԃ
REGISTER_COLOR_NAME("\xE3\x82\xA8\xE3\x82\xB8\xE3\x83\x97\xE3\x82\xB7\xE3\x83\xA3\xE3\x83\xB3\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x1034A6FF);//GWvVu[
REGISTER_COLOR_NAME("\xE8\x91\xA1\xE8\x90\x84\xE8\x89\xB2", 0x640125FF);//F
REGISTER_COLOR_NAME("\xE8\x91\xA1\xE8\x90\x84", 0x640125FF);//
REGISTER_COLOR_NAME("\xE6\xB5\xB7\xE8\x80\x81\xE8\x89\xB2", 0x640125FF);//CVF
REGISTER_COLOR_NAME("\xE6\xB5\xB7\xE8\x80\x81", 0x640125FF);//CV
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x81\xB3\xE3\x81\x84\xE3\x82\x8D", 0x640125FF);//т
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x81\xB3\xE8\x89\xB2", 0x640125FF);//ѐF
REGISTER_COLOR_NAME("\xE3\x81\x88\xE3\x81\xB3", 0x640125FF);//
REGISTER_COLOR_NAME("\xE8\xB2\x9D\xE7\xB4\xAB\xE8\x89\xB2", 0x7F1184FF);//LF
REGISTER_COLOR_NAME("\xE8\xB2\x9D\xE7\xB4\xAB", 0x7F1184FF);//L
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x84\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0x7F1184FF);//ނ炳
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x84\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D\xE8\x89\xB2", 0x7F1184FF);//ނ炳F
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x84\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x7F1184FF);//ނ炳
REGISTER_COLOR_NAME("\xE3\x82\xAB\xE3\x83\x89\xE3\x83\x9F\xE3\x82\xA6\xE3\x83\xA0\xE3\x82\xA4\xE3\x82\xA8\xE3\x83\xAD\xE3\x83\xBC", 0xFAC61EFF);//Jh~ECG[
REGISTER_COLOR_NAME("\xE9\xB4\xA8\xE3\x81\xAE\xE7\xBE\xBD\xE8\x89\xB2", 0x006956FF);//̉HF
REGISTER_COLOR_NAME("\xE9\xB4\xA8\xE3\x81\xAE\xE7\xBE\xBD", 0x006956FF);//̉H
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x82\xE3\x81\xAE\xE3\x81\xAF\xE3\x81\x84\xE3\x82\x8D", 0x006956FF);//̂͂
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x82\xE3\x81\xAE\xE3\x81\xAF\xE8\x89\xB2", 0x006956FF);//̂͐F
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x82\x82\xE3\x81\xAE\xE3\x81\xAF", 0x006956FF);//̂
REGISTER_COLOR_NAME("\xE9\xBA\xB9\xE5\xA1\xB5", 0xBFBD9EFF);//o
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\x8F\xE3\x81\x98\xE3\x82\x93", 0xBFBD9EFF);//
REGISTER_COLOR_NAME("\xE5\xB1\xB1\xE9\xB3\xA9\xE8\x89\xB2", 0xBFBD9EFF);//RF
REGISTER_COLOR_NAME("\xE3\x82\x84\xE3\x81\xBE\xE3\x81\xB0\xE3\x81\xA8\xE3\x81\x84\xE3\x82\x8D", 0xBFBD9EFF);//܂΂Ƃ
REGISTER_COLOR_NAME("\xE3\x82\x84\xE3\x81\xBE\xE3\x81\xB0\xE3\x81\xA8\xE8\x89\xB2", 0xBFBD9EFF);//܂΂ƐF
REGISTER_COLOR_NAME("\xE3\x82\x84\xE3\x81\xBE\xE3\x81\xB0\xE3\x81\xA8", 0xBFBD9EFF);//܂΂
REGISTER_COLOR_NAME("\xE9\x9D\x92\xE7\x99\xBD\xE6\xA9\xA1", 0xBFBD9EFF);//
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x8A\xE3\x81\x97\xE3\x82\x8D\xE3\x81\xA4\xE3\x82\x8B\xE3\x81\xB0\xE3\x81\xBF", 0xBFBD9EFF);//΂
REGISTER_COLOR_NAME("\xE7\x8B\x90\xE8\x89\xB2", 0xC48847FF);//ϐF
REGISTER_COLOR_NAME("\xE7\x8B\x90", 0xC48847FF);//
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xA4\xE3\x81\xAD\xE3\x81\x84\xE3\x82\x8D", 0xC48847FF);//˂
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xA4\xE3\x81\xAD\xE8\x89\xB2", 0xC48847FF);//ːF
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x81\xA4\xE3\x81\xAD", 0xC48847FF);//
REGISTER_COLOR_NAME("\xE4\xBA\xAC\xE7\xB4\xAB", 0x9D5B8BFF);//
REGISTER_COLOR_NAME("\xE3\x81\x8D\xE3\x82\x87\xE3\x81\x86\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x9D5B8BFF);//傤ނ炳
REGISTER_COLOR_NAME("\xE6\xBB\x85\xE7\xB4\xAB", 0x594255FF);//Ŏ
REGISTER_COLOR_NAME("\xE3\x81\x91\xE3\x81\x97\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x594255FF);//ނ炳
REGISTER_COLOR_NAME("\xE3\x82\x81\xE3\x81\xA3\xE3\x81\x97", 0x594255FF);//߂
REGISTER_COLOR_NAME("\xE6\xA2\x94\xE5\xAD\x90\xE8\x89\xB2", 0xFFD768FF);//qF
REGISTER_COLOR_NAME("\xE6\xA2\x94\xE5\xAD\x90", 0xFFD768FF);//q
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x81\xA1\xE3\x81\xAA\xE3\x81\x97\xE3\x81\x84\xE3\x82\x8D", 0xFFD768FF);//Ȃ
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x81\xA1\xE3\x81\xAA\xE3\x81\x97\xE8\x89\xB2", 0xFFD768FF);//ȂF
REGISTER_COLOR_NAME("\xE3\x81\x8F\xE3\x81\xA1\xE3\x81\xAA\xE3\x81\x97", 0xFFD768FF);//Ȃ
REGISTER_COLOR_NAME("\xE3\x82\xAF\xE3\x83\xAA\xE3\x83\xBC\xE3\x83\xA0", 0xFFEDB3FF);//N[
REGISTER_COLOR_NAME("\xE3\x82\xAF\xE3\x83\xAA\xE3\x83\xA0\xE3\x82\xBE\xE3\x83\xB3", 0xDC143CFF);//N]
REGISTER_COLOR_NAME("\xE6\x9C\x88\xE7\x99\xBD", 0xEAF4FCFF);//
REGISTER_COLOR_NAME("\xE3\x81\x92\xE3\x81\xA3\xE3\x81\xB1\xE3\x81\x8F", 0xEAF4FCFF);//ς
REGISTER_COLOR_NAME("\xE6\x86\xB2\xE6\xB3\x95\xE8\x89\xB2", 0x543f32FF);//@F
REGISTER_COLOR_NAME("\xE3\x81\x91\xE3\x82\x93\xE3\x81\xBD\xE3\x81\x86\xE3\x81\x84\xE3\x82\x8D", 0x543f32FF);//ۂ
REGISTER_COLOR_NAME("\xE3\x81\x91\xE3\x82\x93\xE3\x81\xBD\xE3\x81\x86\xE8\x89\xB2", 0x543f32FF);//ۂF
REGISTER_COLOR_NAME("\xE3\x81\x91\xE3\x82\x93\xE3\x81\xBD\xE3\x81\x86", 0x543f32FF);//ۂ
REGISTER_COLOR_NAME("\xE6\x86\xB2\xE6\xB3\x95\xE8\x8C\xB6", 0x543f32FF);//@
REGISTER_COLOR_NAME("\xE6\x86\xB2\xE6\xB3\x95\xE9\xBB\x92", 0x543f32FF);//@
REGISTER_COLOR_NAME("\xE6\xBF\x83\xE8\xB5\xA4\xE7\xB4\xAB", 0x643C51FF);//ZԎ
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x84\xE3\x81\x82\xE3\x81\x8B\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x643C51FF);//ނ炳
REGISTER_COLOR_NAME("\xE9\xA6\x99\xE8\x89\xB2", 0xEFCD9AFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x86\xE3\x81\x84\xE3\x82\x8D", 0xEFCD9AFF);//
REGISTER_COLOR_NAME("\xE9\xBB\x84\xE6\xAB\xA8\xE6\x9F\x93", 0xD99502FF);//
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x86\xE3\x82\x8D\xE3\x81\x9C\xE3\x82\x93", 0xD99502FF);//낺
REGISTER_COLOR_NAME("\xE5\x9B\xBD\xE9\x98\xB2\xE8\x89\xB2", 0x7B6C3EFF);//hF
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x81\x8F\xE3\x81\xBC\xE3\x81\x86\xE3\x81\x84\xE3\x82\x8D", 0x7B6C3EFF);//ڂ
REGISTER_COLOR_NAME("\xE7\xB4\xBA\xE6\xA1\x94\xE6\xA2\x97", 0x4DDAAFFF);//j[
REGISTER_COLOR_NAME("\xE3\x81\x93\xE3\x82\x93\xE3\x81\x8D\xE3\x81\x8D\xE3\x82\x87\xE3\x81\x86", 0x4DDAAFFF);//񂫂傤
REGISTER_COLOR_NAME("\xE9\x87\x91\xE9\x9D\x92", 0x694F94FF);//
REGISTER_COLOR_NAME("\xE7\xB4\xAB\xE8\x8B\x91\xE8\x89\xB2", 0x867BA9FF);//F
REGISTER_COLOR_NAME("\xE7\xB4\xAB\xE8\x8B\x91", 0x867BA9FF);//
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x81\x8A\xE3\x82\x93\xE3\x81\x84\xE3\x82\x8D", 0x867BA9FF);//񂢂
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x81\x8A\xE3\x82\x93\xE8\x89\xB2", 0x867BA9FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x81\x8A\xE3\x82\x93", 0x867BA9FF);//
REGISTER_COLOR_NAME("\xE8\x8A\x9D\xE7\xBF\xAB\xE8\x8C\xB6", 0xAD7E4EFF);//Ŋ㒃
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x81\x8B\xE3\x82\x93\xE3\x81\xA1\xE3\x82\x83", 0xAD7E4EFF);//񂿂
REGISTER_COLOR_NAME("\xE7\x8C\xA9\xE3\x80\x85\xE7\xB7\x8B", 0xCE313DFF);//́X
REGISTER_COLOR_NAME("\xE7\x8C\xA9\xE7\x8C\xA9\xE7\xB7\x8B", 0xCE313DFF);//͔
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x87\xE3\x81\x86\xE3\x81\x98\xE3\x82\x87\xE3\x81\x86\xE3\x81\xB2", 0xCE313DFF);//傤傤
REGISTER_COLOR_NAME("\xE9\x9B\x80\xE8\x89\xB2", 0x864337FF);//F
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\x9A\xE3\x82\x81\xE3\x81\x84\xE3\x82\x8D", 0x864337FF);//߂
REGISTER_COLOR_NAME("\xE3\x81\x99\xE3\x81\x9A\xE3\x82\x81\xE8\x89\xB2", 0x864337FF);//ߐF
REGISTER_COLOR_NAME("\xE7\x9F\xB3\xE7\xAB\xB9\xE8\x89\xB2", 0xF9C1CFFF);//Β|F
REGISTER_COLOR_NAME("\xE7\x9F\xB3\xE7\xAB\xB9", 0xF9C1CFFF);//Β|
REGISTER_COLOR_NAME("\xE3\x81\x9B\xE3\x81\x8D\xE3\x81\xA1\xE3\x81\x8F\xE3\x81\x84\xE3\x82\x8D", 0xF9C1CFFF);//
REGISTER_COLOR_NAME("\xE3\x81\x9B\xE3\x81\x8D\xE3\x81\xA1\xE3\x81\x8F\xE8\x89\xB2", 0xF9C1CFFF);//F
REGISTER_COLOR_NAME("\xE3\x81\x9B\xE3\x81\x8D\xE3\x81\xA1\xE3\x81\x8F", 0xF9C1CFFF);//
REGISTER_COLOR_NAME("\xE4\xBB\x99\xE6\x96\x8E\xE8\x8C\xB6", 0x474B42FF);//֒
REGISTER_COLOR_NAME("\xE3\x81\x9B\xE3\x82\x93\xE3\x81\x95\xE3\x81\x84\xE3\x81\xA1\xE3\x82\x83", 0x474B42FF);//񂳂
REGISTER_COLOR_NAME("\xE8\xB5\xAD", 0xAB6953FF);//
REGISTER_COLOR_NAME("\xE3\x81\x9D\xE3\x81\xBB", 0xAB6953FF);//
REGISTER_COLOR_NAME("\xE7\x8E\x89\xE8\x9C\x80\xE9\xBB\x8D\xE8\x89\xB2", 0xEEC362FF);//冋oF
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x86\xE3\x82\x82\xE3\x82\x8D\xE3\x81\x93\xE3\x81\x97\xE3\x81\x84\xE3\x82\x8D", 0xEEC362FF);//Ƃ낱
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x86\xE3\x82\x82\xE3\x82\x8D\xE3\x81\x93\xE3\x81\x97\xE8\x89\xB2", 0xEEC362FF);//Ƃ낱F
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x86\xE3\x82\x82\xE3\x82\x8D\xE3\x81\x93\xE3\x81\x97", 0xEEC362FF);//Ƃ낱
REGISTER_COLOR_NAME("\xE6\x9C\xA8\xE8\xB3\x8A\xE8\x89\xB2", 0x3B7960FF);//ؑF
REGISTER_COLOR_NAME("\xE6\x9C\xA8\xE8\xB3\x8A", 0x3B7960FF);//ؑ
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x8F\xE3\x81\x95\xE3\x81\x84\xE3\x82\x8D", 0x3B7960FF);//Ƃ
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x8F\xE3\x81\x95\xE8\x89\xB2", 0x3B7960FF);//ƂF
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\x8F\xE3\x81\x95", 0x3B7960FF);//Ƃ
REGISTER_COLOR_NAME("\xE3\x81\x8B\xE3\x81\x92\xE8\x90\x8C\xE9\xBB\x84", 0x3B7960FF);//G
REGISTER_COLOR_NAME("\xE3\x83\x89\xE3\x82\xB8\xE3\x83\xA3\xE3\x83\xBC\xE3\x83\x96\xE3\x83\xAB\xE3\x83\xBC", 0x1E90FFFF);//hW[u[
REGISTER_COLOR_NAME("\xE7\x95\x99\xE7\xB4\xBA", 0x1C305CFF);//
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x82\x81\xE3\x81\x93\xE3\x82\x93", 0x1C305CFF);//Ƃ߂
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x81\xBE\xE3\x82\x8A\xE3\x81\x93\xE3\x82\x93", 0x1C305CFF);//Ƃ܂肱
REGISTER_COLOR_NAME("\xE9\xB3\xA5\xE3\x81\xAE\xE5\xAD\x90\xE8\x89\xB2", 0xFFF1CFFF);//̎qF
REGISTER_COLOR_NAME("\xE9\xB3\xA5\xE3\x81\xAE\xE5\xAD\x90", 0xFFF1CFFF);//̎q
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x82\x8A\xE3\x81\xAE\xE3\x81\x93\xE3\x81\x84\xE3\x82\x8D", 0xFFF1CFFF);//Ƃ̂
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x82\x8A\xE3\x81\xAE\xE3\x81\x93\xE8\x89\xB2", 0xFFF1CFFF);//Ƃ̂F
REGISTER_COLOR_NAME("\xE3\x81\xA8\xE3\x82\x8A\xE3\x81\xAE\xE3\x81\x93", 0xFFF1CFFF);//Ƃ̂
REGISTER_COLOR_NAME("\xE6\x92\xAB\xE5\xAD\x90\xE8\x89\xB2", 0xF6ADC6FF);//qF
REGISTER_COLOR_NAME("\xE6\x92\xAB\xE5\xAD\x90", 0xF6ADC6FF);//q
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\xA7\xE3\x81\x97\xE3\x81\x93\xE3\x81\x84\xE3\x82\x8D", 0xF6ADC6FF);//Ȃł
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\xA7\xE3\x81\x97\xE3\x81\x93\xE8\x89\xB2", 0xF6ADC6FF);//ȂłF
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\xA7\xE3\x81\x97\xE3\x81\x93", 0xF6ADC6FF);//Ȃł
REGISTER_COLOR_NAME("\xE7\x94\x9F\xE7\x89\xA1\xE8\xA0\xA3\xE8\x89\xB2", 0xEDF0E0FF);//yF
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\xBE\xE3\x81\x8C\xE3\x81\x8D\xE3\x81\x84\xE3\x82\x8D", 0xEDF0E0FF);//Ȃ܂
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\xBE\xE3\x81\x8C\xE3\x81\x8D\xE8\x89\xB2", 0xEDF0E0FF);//Ȃ܂F
REGISTER_COLOR_NAME("\xE4\xBC\xBC\xE7\xB4\xAB", 0x513743FF);//
REGISTER_COLOR_NAME("\xE3\x81\xAB\xE3\x81\x9B\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x513743FF);//ɂނ炳
REGISTER_COLOR_NAME("\xE9\x88\x8D\xE8\x89\xB2", 0x727171FF);//ݐF
REGISTER_COLOR_NAME("\xE3\x81\xAB\xE3\x81\xB3\xE3\x81\x84\xE3\x82\x8D", 0x727171FF);//ɂт
REGISTER_COLOR_NAME("\xE3\x81\xAB\xE3\x81\xB3\xE8\x89\xB2", 0x727171FF);//ɂѐF
REGISTER_COLOR_NAME("\xE6\xBF\xA1\xE7\x83\x8F", 0x000B00FF);//GG
REGISTER_COLOR_NAME("\xE3\x81\xAC\xE3\x82\x8C\xE3\x81\x8C\xE3\x82\x89\xE3\x81\x99", 0x000B00FF);//ʂꂪ炷
REGISTER_COLOR_NAME("\xE6\xBF\xA1\xE3\x82\x8C\xE7\xBE\xBD\xE8\x89\xB2", 0x000B00FF);//GHF
REGISTER_COLOR_NAME("\xE7\x83\x8F\xE7\xBE\xBD\xE8\x89\xB2", 0x000B00FF);//GHF
REGISTER_COLOR_NAME("\xE7\xB7\xB4\xE8\x89\xB2", 0xEDE4CDFF);//F
REGISTER_COLOR_NAME("\xE3\x81\xAD\xE3\x82\x8A\xE3\x81\x84\xE3\x82\x8D", 0xEDE4CDFF);//˂肢
REGISTER_COLOR_NAME("\xE3\x81\xAD\xE3\x82\x8A\xE8\x89\xB2", 0xEDE4CDFF);//˂F
REGISTER_COLOR_NAME("\xE7\x81\xB0\xE7\xB7\x91", 0x51653CFF);//D
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x84\xE3\x81\xBF\xE3\x81\xA9\xE3\x82\x8A", 0x51653CFF);//݂͂ǂ
REGISTER_COLOR_NAME("\xE7\x81\xB0\xE6\xA1\x9C", 0xEBD4E0FF);//D
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x84\xE3\x81\x96\xE3\x81\x8F\xE3\x82\x89", 0xEBD4E0FF);//͂
REGISTER_COLOR_NAME("\xE6\xA6\x9B\xE8\x89\xB2", 0xBFA46FFF);//YF
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x97\xE3\x81\xB0\xE3\x81\xBF\xE3\x81\x84\xE3\x82\x8D", 0xBFA46FFF);//͂΂݂
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\x97\xE3\x81\xB0\xE3\x81\xBF\xE8\x89\xB2", 0xBFA46FFF);//͂΂ݐF
REGISTER_COLOR_NAME("\xE6\xB7\xB1\xE7\xB8\xB9", 0x274765FF);//[|
REGISTER_COLOR_NAME("\xE3\x81\xB5\xE3\x81\x8B\xE3\x81\x8D\xE3\x81\xAF\xE3\x81\xAA\xE3\x81\xA0", 0x274765FF);//ӂ͂Ȃ
REGISTER_COLOR_NAME("\xE4\xB8\xAD\xE7\xB8\xB9", 0x3F729FFF);//|
REGISTER_COLOR_NAME("\xE3\x81\xAA\xE3\x81\x8B\xE3\x81\xAE\xE3\x81\xAF\xE3\x81\xAA\xE3\x81\xA0", 0x3F729FFF);//Ȃ̂͂Ȃ
REGISTER_COLOR_NAME("\xE6\xAC\xA1\xE7\xB8\xB9", 0x618BADFF);//|
REGISTER_COLOR_NAME("\xE3\x81\xA4\xE3\x81\x8E\xE3\x81\xAE\xE3\x81\xAF\xE3\x81\xAA\xE3\x81\xA0", 0x618BADFF);//̂͂Ȃ
REGISTER_COLOR_NAME("\xE6\xB5\x85\xE7\xB8\xB9", 0x67A7CCFF);//|
REGISTER_COLOR_NAME("\xE3\x81\x82\xE3\x81\x95\xE3\x81\x8D\xE3\x81\xAF\xE3\x81\xAA\xE3\x81\xA0", 0x67A7CCFF);//͂Ȃ
REGISTER_COLOR_NAME("\xE7\x99\xBD\xE7\xB8\xB9", 0xA3D8F6FF);//|
REGISTER_COLOR_NAME("\xE3\x81\x97\xE3\x82\x8D\xE3\x81\x8D\xE3\x81\xAF\xE3\x81\xAA\xE3\x81\xA0", 0xA3D8F6FF);//낫͂Ȃ
REGISTER_COLOR_NAME("\xE8\x96\x84\xE7\xB8\xB9", 0x9BC9D5FF);//|
REGISTER_COLOR_NAME("\xE3\x81\x86\xE3\x81\x99\xE3\x81\xAF\xE3\x81\xAA\xE3\x81\xA0", 0x9BC9D5FF);//͂Ȃ
REGISTER_COLOR_NAME("\xE6\x9C\x88\xE8\x8D\x89\xE8\x89\xB2", 0x21A0DBFF);//F
REGISTER_COLOR_NAME("\xE5\x8D\x83\xE8\x8D\x89\xE8\x89\xB2", 0x21A0DBFF);//瑐F
REGISTER_COLOR_NAME("\xE8\x8A\xB1\xE8\x89\xB2", 0x426AB3FF);//ԐF
REGISTER_COLOR_NAME("\xE6\xB7\xB1\xE7\xB4\xAB", 0x493759FF);//[
REGISTER_COLOR_NAME("\xE3\x81\xB5\xE3\x81\x8B\xE3\x82\x80\xE3\x82\x89\xE3\x81\x95\xE3\x81\x8D", 0x493759FF);//ӂނ炳
REGISTER_COLOR_NAME("\xE9\xBB\x92\xE7\xB4\xAB", 0x493759FF);//
REGISTER_COLOR_NAME("\xE5\x94\x90\xE6\xA3\xA3\xE8\x89\xB2", 0xF0908DFF);//F
REGISTER_COLOR_NAME("\xE5\x94\x90\xE6\xA3\xA3", 0xF0908DFF);//
REGISTER_COLOR_NAME("\xE3\x81\xAF\xE3\x81\xAD\xE3\x81\x9A\xE8\x89\xB2", 0xF0908DFF);//͂˂F
REGISTER_COLOR_NAME("\xE6\x9C\xB1\xE8\x8F\xAF", 0xF0908DFF);//
REGISTER_COLOR_NAME("\xE5\x94\x90\xE6\xA3\xA3\xE8\x8A\xB1", 0xF0908DFF);//
REGISTER_COLOR_NAME("\xE6\xA3\xA0\xE6\xA3\xA3", 0xF0908DFF);//
REGISTER_COLOR_NAME("\xE3\x83\x94\xE3\x83\xA5\xE3\x83\xBC\xE3\x82\xB9", 0xCC8899FF);//s[X
REGISTER_COLOR_NAME("\xE9\xB6\xB8\xE8\x90\x8C\xE9\xBB\x84", 0x82AE46FF);//SG
REGISTER_COLOR_NAME("\xE3\x81\xB2\xE3\x82\x8F\xE3\x82\x82\xE3\x81\x88\xE3\x81\x8E", 0x82AE46FF);//Ђ
REGISTER_COLOR_NAME("\xE3\x83\x95\xE3\x82\xAF\xE3\x82\xB7\xE3\x83\xA3", 0xCC1669FF);//tNV
REGISTER_COLOR_NAME("\xE4\xBA\x8C\xE8\x97\x8D", 0x6B4978FF);//
REGISTER_COLOR_NAME("\xE3\x81\xB5\xE3\x81\x9F\xE3\x81\x82\xE3\x81\x84", 0x6B4978FF);//ӂ
REGISTER_COLOR_NAME("\xE7\xB4\x9B\xE7\xB4\x85", 0xE9756BFF);//g
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\x8C\xE3\x81\x84\xE3\x81\xB9\xE3\x81\xAB", 0xE9756BFF);//܂ׂ
REGISTER_COLOR_NAME("\xE7\x9C\x9F\xE8\xB5\xAD", 0xD57C6BFF);//^
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\x9D\xE3\x81\x8A", 0xD57C6BFF);//܂
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\x9D\xE3\x81\xBB", 0xD57C6BFF);//܂
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\x99\xE3\x81\x8A", 0xD57C6BFF);//܂
REGISTER_COLOR_NAME("\xE3\x81\xBE\xE3\x81\x99\xE3\x81\xBB", 0xD57C6BFF);//܂
REGISTER_COLOR_NAME("\xE6\x9D\xBE\xE5\xB4\x8E\xE3\x81\x97\xE3\x81\x92\xE3\x82\x8B", 0xA55A4AFF);//肵
REGISTER_COLOR_NAME("\xE6\x9C\xA8\xE8\x98\xAD\xE8\x89\xB2", 0xC7B370FF);//ؗF
REGISTER_COLOR_NAME("\xE6\x9C\xA8\xE8\x98\xAD", 0xC7B370FF);//ؗ
REGISTER_COLOR_NAME("\xE3\x82\x82\xE3\x81\x8F\xE3\x82\x89\xE3\x82\x93\xE8\x89\xB2", 0xC7B370FF);//F
REGISTER_COLOR_NAME("\xE3\x82\x82\xE3\x81\x8F\xE3\x82\x89\xE3\x82\x93", 0xC7B370FF);//
REGISTER_COLOR_NAME("\xE7\x99\xBE\xE5\xA1\xA9\xE8\x8C\xB6", 0x542D24FF);//S
REGISTER_COLOR_NAME("\xE3\x82\x82\xE3\x82\x82\xE3\x81\x97\xE3\x81\x8A\xE3\x81\xA1\xE3\x82\x83", 0x542D24FF);//
REGISTER_COLOR_NAME("\xE7\xBE\x8A\xE7\xBE\xB9\xE8\x89\xB2", 0x542D24FF);//r㻐F
REGISTER_COLOR_NAME("\xE6\x9F\xB3\xE8\x89\xB2", 0xA8C97FFF);//F
REGISTER_COLOR_NAME("\xE3\x82\x84\xE3\x81\xAA\xE3\x81\x8E\xE3\x81\x84\xE3\x82\x8D", 0xA8C97FFF);//Ȃ
REGISTER_COLOR_NAME("\xE3\x82\x84\xE3\x81\xAA\xE3\x81\x8E\xE8\x89\xB2", 0xA8C97FFF);//ȂF
REGISTER_COLOR_NAME("\xE9\x9B\x84\xE9\xBB\x84", 0xF2B16AFF);//Y
REGISTER_COLOR_NAME("\xE3\x82\x86\xE3\x81\x86\xE3\x81\x8A\xE3\x81\x86", 0xF2B16AFF);//䂤
REGISTER_COLOR_NAME("\xE8\x93\xAC\xE8\x89\xB2", 0x616B07FF);//HF
REGISTER_COLOR_NAME("\xE8\x93\xAC", 0x616B07FF);//H
REGISTER_COLOR_NAME("\xE3\x82\x88\xE3\x82\x82\xE3\x81\x8E\xE3\x81\x84\xE3\x82\x8D", 0x616B07FF);//
REGISTER_COLOR_NAME("\xE3\x82\x88\xE3\x82\x82\xE3\x81\x8E\xE8\x89\xB2", 0x616B07FF);//F
REGISTER_COLOR_NAME("\xE3\x82\x88\xE3\x82\x82\xE3\x81\x8E", 0x616B07FF);//
REGISTER_COLOR_NAME("\xE7\x92\x83\xE5\xAF\x9B\xE8\x8C\xB6", 0x6E6636FF);//
REGISTER_COLOR_NAME("\xE3\x82\x8A\xE3\x81\x8B\xE3\x82\x93\xE3\x81\xA1\xE3\x82\x83", 0x6E6636FF);//肩񂿂
REGISTER_COLOR_NAME("\xE3\x82\x8C\xE3\x82\x82\xE3\x82\x93\xE8\x89\xB2", 0xFFF450FF);//F
REGISTER_COLOR_NAME("\xE3\x83\xAC\xE3\x83\xA2\xE3\x83\xB3\xE8\x89\xB2", 0xFFF450FF);//F
REGISTER_COLOR_NAME("\xE3\x82\x8C\xE3\x82\x82\xE3\x82\x93\xE3\x81\x84\xE3\x82\x8D", 0xFFF450FF);//񂢂
REGISTER_COLOR_NAME("\xE3\x83\xAC\xE3\x83\xA2\xE3\x83\xB3\xE3\x81\x84\xE3\x82\x8D", 0xFFF450FF);//
REGISTER_COLOR_NAME("\xE3\x82\x8C\xE3\x82\x82\xE3\x82\x93", 0xFFF450FF);//
REGISTER_COLOR_NAME("\xE3\x83\xAC\xE3\x83\xA2\xE3\x83\xB3", 0xFFF450FF);//
REGISTER_COLOR_NAME("\xE8\xB7\xAF\xE8\x80\x83\xE8\x8C\xB6", 0x8C7042FF);//Hl
REGISTER_COLOR_NAME("\xE3\x82\x8D\xE3\x81\x93\xE3\x81\x86\xE3\x81\xA1\xE3\x82\x83", 0x8C7042FF);//낱
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x8F\x9C\xE8\x89\xB2", 0xD8E698FF);//ؐF
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x8F\x9C", 0xD8E698FF);//
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\xAA\xE3\x81\x84\xE3\x82\x8D", 0xD8E698FF);//킩Ȃ
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\xAA\xE8\x89\xB2", 0xD8E698FF);//킩ȐF
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\xAA", 0xD8E698FF);//킩
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x8B\x97\xE8\x89\xB2", 0xC7DC68FF);//cF
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x8B\x97", 0xC7DC68FF);//c
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\xAA\xE3\x81\x88\xE3\x81\x84\xE3\x82\x8D", 0xC7DC68FF);//킩Ȃ
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\xAA\xE3\x81\x88\xE8\x89\xB2", 0xC7DC68FF);//킩ȂF
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x81\xAA\xE3\x81\x88", 0xC7DC68FF);//킩Ȃ
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x8A\xBD\xE8\x89\xB2", 0xE0EBAFFF);//F
REGISTER_COLOR_NAME("\xE8\x8B\xA5\xE8\x8A\xBD", 0xE0EBAFFF);//
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x82\x81\xE3\x81\x84\xE3\x82\x8D", 0xE0EBAFFF);//킩߂
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x82\x81\xE8\x89\xB2", 0xE0EBAFFF);//킩ߐF
REGISTER_COLOR_NAME("\xE3\x82\x8F\xE3\x81\x8B\xE3\x82\x81", 0xE0EBAFFF);//킩
}

	return;
}

gboolean color_from_str (gchar *name, Color *color)
{
	gchar *uname = NULL;
	gboolean result = FALSE;

	uname = g_utf8_strup(name, -1);

	//#FFFFFF Style
	if (*uname == '#') {
		int r, g, b;
		if (g_utf8_strlen(uname, -1) < 7) {
			return FALSE;
		}
#ifdef _WIN32
		sscanf_s(uname, "#%2x%2x%2x", &r, &g, &b);
#else
		sscanf(uname, "#%2x%2x%2x", &r, &g, &b);
#endif
		color->r = (guchar)CLAMP(r,0,255);
		color->g = (guchar)CLAMP(g,0,255);
		color->b = (guchar)CLAMP(b,0,255);
		color->a = 0xFF;
		result = TRUE;
		goto done;
	}
	//hsl(DEG,N%,N%)
	{
		GRegex *regex = g_regex_new("HSL\\( *(-?[\\d\\.]+) *, *(-?[\\d\\.]+)%? *, *(-?[\\d\\.]+)%? *\\)", 0, 0, NULL);
		GMatchInfo *match_info;

		g_regex_match(regex, uname, 0, &match_info);
		if (g_match_info_matches(match_info)) {
			gchar *str_h = g_match_info_fetch(match_info, 1);
			gchar *str_s = g_match_info_fetch(match_info, 2);
			gchar *str_l = g_match_info_fetch(match_info, 3);
			double h = atof(str_h);
			double s = atof(str_s);
			double l = atof(str_l);
			double r, g, b;
			h = fmod(fmod(h,360)+360,360)/360.0;
			s = CLAMP(s,0,100) / 100.0;
			l = CLAMP(l,0,100) / 100.0;
			hsl_to_rgb(h, s, l, &r, &g, &b);
			color->r = (guchar)CLAMP(ROUND(r*255),0,255);
			color->g = (guchar)CLAMP(ROUND(g*255),0,255);
			color->b = (guchar)CLAMP(ROUND(b*255),0,255);
			color->a = 0xFF;
			if (str_h) g_free(str_h);
			if (str_s) g_free(str_s);
			if (str_l) g_free(str_l);
			result = TRUE;
		}
		g_match_info_free(match_info);
		g_regex_unref(regex);
		if (result == TRUE) goto done;
	}
	//hsv(DEG,N%,N%)
	{
		GRegex *regex = g_regex_new("HSV\\( *(-?[\\d\\.]+) *, *(-?[\\d\\.]+)%? *, *(-?[\\d\\.]+)%? *\\)", 0, 0, NULL);
		GMatchInfo *match_info;

		g_regex_match(regex, uname, 0, &match_info);
		if (g_match_info_matches(match_info)) {
			gchar *str_h = g_match_info_fetch(match_info, 1);
			gchar *str_s = g_match_info_fetch(match_info, 2);
			gchar *str_v = g_match_info_fetch(match_info, 3);
			double h = atof(str_h);
			double s = atof(str_s);
			double v = atof(str_v);
			double r, g, b;
			h = fmod(fmod(h,360)+360,360)/360.0;
			s = CLAMP(s,0,100) / 100;
			v = CLAMP(v,0,100) / 100;
			hsv_to_rgb(h, s, v, &r, &g, &b);
			color->r = (guchar)CLAMP(ROUND(r*255),0,255);
			color->g = (guchar)CLAMP(ROUND(g*255),0,255);
			color->b = (guchar)CLAMP(ROUND(b*255),0,255);
			color->a = 0xFF;
			if (str_h) g_free(str_h);
			if (str_s) g_free(str_s);
			if (str_v) g_free(str_v);
			result = TRUE;
		}
		g_match_info_free(match_info);
		g_regex_unref(regex);
		if (result == TRUE) goto done;
	}

	if (!color_table) {
		init_color_table();
	}

	{
		gpointer k, v;
		if (g_hash_table_lookup_extended(color_table, uname, &k , &v)) {
			gint i = GPOINTER_TO_INT(v);
			color->r = R_FROM_INT(i);
			color->g = G_FROM_INT(i);
			color->b = B_FROM_INT(i);
			color->a = 0xFF;
			result = TRUE;
			goto done;
		}
	}

done:
	if (uname) g_free(uname);
	return result;
}


guint8 color_to_intensity_uchar (Color color)
{
	return (unsigned char)MIN(0xFF, ceil(color_to_intensity(color)));
}

double color_to_intensity (Color color)
{
	//NTSC
	return 0.298912 * color.r + 0.586611 * color.g + 0.114478 * color.b;
}

void rgb_to_hsv (double r, double g, double b, double *out_h, double *out_s, double *out_v) {
	double max, min, h, s, v;
	max = MAX(MAX(r,g),b);
	min = MIN(MIN(r,g),b);
	h   = max - min;

	if (h > 0.0) {
		if (max == r) {
			h = (g - b) / h;
			if (h < 0.0) {
				h = h + 6.0;
			}
		} else if (max == g) {
			h = 2.0 + (b - r) / h;
		} else {
			h = 4.0 + (r - g) / h;
		}
	}
	h = h / 6.0;
	s = max - min;
	if (max != 0.0) {
		s = s / max;
	}
	v = max;
	*out_h = h;
	*out_s = s;
	*out_v = v;
	return;
}

void hsv_to_rgb (double h, double s, double v, double *out_r, double *out_g, double *out_b) {
	int hi;
	double f, p, q, t;
	hi = (int)fmod((h*6.0), 6.0);
	f  = (h*6.0) - hi;
	p  = v * (1.0 - s);
	q  = v * (1.0 - s * f);
	t  = v * (1.0 - s * (1 - f));
	switch (hi) {
		case 0:  *out_r = v; *out_g = t; *out_b = p; break;
		case 1:  *out_r = q; *out_g = v; *out_b = p; break;
		case 2:  *out_r = p; *out_g = v; *out_b = t; break;
		case 3:  *out_r = p; *out_g = q; *out_b = v; break;
		case 4:  *out_r = t; *out_g = p; *out_b = v; break;
		case 5:  *out_r = v; *out_g = p; *out_b = q; break;
	}
	return;
}

void rgb_to_hsl (double r, double g, double b, double *out_h, double *out_s, double *out_l) {
	double max, min, h, s, l;
	max = MAX(MAX(r,g),b);
	min = MIN(MIN(r,g),b);
	h = 0;
	s = 0;
	l = (max + min) / 2.0;

	if (max != min) {
		double diff = max - min;
		if (l > 0.5) {
			s = diff / (2.0 - max - min);
		} else {
			s = diff / (max + min);
		}
		if (max == r) {
			h = (g - b) / diff + (g < b ? 6.0 : 0.0);
		} else
		if (max == g) {
			h = (b - r) / diff + 2.0;
		} else
		if (max == b) {
			h = (r - g) / diff + 4.0;
		}
		h = h / 6.0;
	}
	*out_h = h;
	*out_s = s;
	*out_l = l;
}

double hue_to_rgb (double m1, double m2, double h) {
	h = fmod(fmod(h,1)+1,1);
	if (h*6 < 1) {
		return m1 + (m2-m1) * h * 6;
	}
	if (h*2 < 1) {
		return m2;
	}
	if (h*3 < 2) {
		return m1 + (m2-m1) * (2.0/3.0-h) * 6.0;
	}
	return m1;
}

void hsl_to_rgb (double h, double s, double l, double *out_r, double *out_g, double *out_b) {
	double m1, m2;
	if (l <= 0.5) {
		m2 = l * (s + 1);
	} else {
		m2 = l + s - l * s;
	}
	m1 = l * 2 - m2;
	*out_r = hue_to_rgb(m1, m2, h+1/3.0);
	*out_g = hue_to_rgb(m1, m2, h);
	*out_b = hue_to_rgb(m1, m2, h-1/3.0);
}

void hsv_to_hsl (double h, double s, double v, double *out_h, double *out_s, double *out_l) {
	double tmp = (2-s) * v;
	*out_h = h;
	*out_s = s * v / (tmp<1 ? tmp : 2-tmp);
	*out_l = tmp / 2.0;
	return;
}

void hsl_to_hsv (double h, double s, double l, double *out_h, double *out_s, double *out_v) {
	double tmp = s * (l<0.5 ? l:1.0-l);
	*out_h = h;
	*out_s = 2 * tmp/(l+tmp);
	*out_v = l + tmp;
	return;
}

/* vim: set fdm=marker ts=4 sw=4:*/
